﻿import os

import app.util
import app.defaults
import app.logutil
import app.actions.lockedaction

class InitializeServiceAction(app.actions.lockedaction.LockedControllerAction):
    def __init__(self, services, logger):
        for service in services:
            if not service.isDaemon():
                raise Exception("Service %s is not a daemon and cannot be initialized" % service.getName())

        self.__services = services

        super(InitializeServiceAction, self).__init__(logger)

    def __getContainedServices(self):
        result = []

        for service in self.__services:
            # Add any services this daemon acts as container for
            if service.getContainedServices():
                result += service.getContainedServices()

        return result

    def chain(self, action):
        assert isinstance(action, InitializeServiceAction)

        return InitializeServiceAction(
            self.__services + action.getInvolvedServices(),
            self.logger)

    def getInvolvedServices(self):
        return self.__services

    def confirm(self):
        return True

    def __initializeServices(self):
        allOk = True

        # Run hooks for daemons, and then also for all contained services.
        for service in self.__services + self.__getContainedServices():
            scriptsFolder = os.path.join(
                app.defaults.DEFAULT_SERVICE_DIR,
                service.getName(),
                "hooks",
                "initialize")

            if os.path.isdir(scriptsFolder):
                self.printLog('title', "Initializing %s..." % service.getName())

                batch = app.batch.InitializationBatch(
                    self.logger,
                    app.util.getRuntimeConfiguration())

                batch.addFolder(scriptsFolder)

                try:
                    batch.execute()
                except Exception:
                    self.logger.error(''.join(app.util.getStackTraceAndException()).strip())
                    allOk = False

        if not allOk:
            self.printLog("error", "\nInitialization failed. See %s for details." % (app.logutil.getFileName(self.logger) or "log"))

        return allOk

    def lockedExecute(self):
        initResult = self.__initializeServices()
        return  initResult
