﻿import app.util
import app.servicecontrol
import app.defaults
import app.logutil
import app.actions.lockedaction

class KillServiceAction(app.actions.lockedaction.LockedControllerAction):
    def __init__(self, services, logger):
        self.__services = services
        super(KillServiceAction, self).__init__(logger)

    def chain(self, action):
        assert isinstance(action, KillServiceAction)

        return KillServiceAction(
            self.__services + action.getInvolvedServices(),
            self.logger)

    def getInvolvedServices(self):
        return self.__services

    def confirm(self, isRedeployment=False):
        # Stopping a service is nothing worth asking for confirmation for.
        return True

    def lockedExecute(self):
        self.printLog("title", "Terminating Services");

        allOk = True

        for service in self.__services:
            try:
                controller = service.getController(self.logger)

                if controller.getStatus() == app.servicecontrol.ServiceStatus.STOPPED_AUTOSTART or \
                   controller.getStatus() == app.servicecontrol.ServiceStatus.STOPPED_DEMANDSTART:
                    # Great, nothing to do here
                    pass
                else:
                    service.getController(self.logger).killDaemon()

                statusAction = service.getController(self.logger).createPrintStatusAction()
                statusAction.setHeaderEnabled(False)
                statusAction.execute()

            except Exception as err:
                self.printLog("error", "Caught exception while trying to terminate service '%s': %s" % (service.getName(), str(err)))
                self.logger.error(''.join(app.util.getStackTraceAndException()).strip())
                allOk = False

        if not allOk:
            self.printLog("error", "\nTerminating services failed. See %s for details." % (app.logutil.getFileName(self.logger) or "log"))

        return allOk
