﻿import app.util
import app.servicecontrol
import app.actions.lockedaction
import time

class PrintStatusServiceAction(app.servicecontrol.ControllerAction):
    def __init__(self, services, logger):
        self.__services = services
        self.__enableHeader = True
        super(PrintStatusServiceAction, self).__init__(logger)

    def setHeaderEnabled(self, enabled):
        self.__enableHeader = enabled

    def getInvolvedServices(self):
        return self.__services

    def execute(self):
        if self.__enableHeader:
            self.printLog("info", "%6s %-40s" % ("PID", "Service"))

        allOk = True

        metastatusMapping = {
            app.servicecontrol.ServiceMetaStatus.OK : "success",
            app.servicecontrol.ServiceMetaStatus.INCONCLUSIVE : "warn",
            app.servicecontrol.ServiceMetaStatus.BAD: "error"
        }

        for service in self.__services:
            controller = service.getController(self.logger)
            status = controller.getStatus()

            allOk = allOk and status.getMetastatus() == app.servicecontrol.ServiceMetaStatus.OK

            self.printLog("info", "%6s %-53s" % (controller.getPid() or "", service.getName()), False)
            self.printLog(metastatusMapping[status.getMetastatus()], "[%s]" % status)

        return allOk

class SnapshotAction(app.actions.lockedaction.LockedControllerAction):
    def __init__(self, services, logger):
        self.__services = services
        super(SnapshotAction, self).__init__(logger)

    def chain(self, action):
        assert isinstance(action, SnapshotAction)

        return SnapshotAction(
            self.__services + action.getInvolvedServices(),
            self.logger)

    def getInvolvedServices(self):
        return self.__services

    def __snaphshotAllservices(self):
        label = "snapshot-%04d%02d%02d%02d%02d%02d" % time.localtime()[:6]
        if len(self.__services) > 1:
            startStr = "**************************BEGIN - %s*****************************"%label
            endStr = "****************************END - %s.******************************"%label
        else:
            startStr = ""
            endStr = ""

        print startStr
        for service in self.__services:
            service.getController(self.logger).printSnapshot(label)
        print endStr

    def lockedExecute(self):
        self.__snaphshotAllservices()
        return True
