﻿import app.util
import app.servicecontrol
import app.colors

class DumpEndpointsSiteAction(app.servicecontrol.ControllerAction):
    def __init__(self, site, logger):
        self.__site = site

        super(DumpEndpointsSiteAction, self).__init__(logger)

    def __printService(self, service):
        self.printLog("info", "    + %s" % app.colors.Color.blue(service.getName()))
        self.printLog("info", "    |   Internal address: %s" % service.getInternalAddress())
        self.printLog("info", "    |   Public address:   %s" % service.getPublicAddress())

        if service.getProvidedEndpoints():
            self.printLog("info", "    |   Provided endpoints:")
            for provEndpoint in service.getProvidedEndpoints():
                self.printLog("info", "    |       %s" % provEndpoint)
                if not provEndpoint.isViaOtherPort():
                    self.printLog("info", "    |             Binding address:   %s:%s" % (provEndpoint.getBindingAddress(), provEndpoint.getPort()))
                self.printLog("info", "    |             Canonical address: %s:%s" % (provEndpoint.getAddress(), provEndpoint.getPort()))
                self.printLog("info", "    |             Clients:")

                for client in provEndpoint.getClients():
                    self.printLog("info", "    |               <- %s on %s" %
                                  (client.getName(), client.getMachine().getPhysicalName()))

                for viaTargetEndpoint in provEndpoint.getViaTargetEndpoints():
                    self.printLog("info", "    |               <-(via)- %s %s" %
                                  (viaTargetEndpoint.getService(), viaTargetEndpoint))


        if service.getRequiredEndpoints():
            self.printLog("info", "    |   Required endpoints:")
            for reqEndpoint in service.getRequiredEndpoints():
                self.printLog("info", "    |       %s" % reqEndpoint)

    def __printMachine(self, machine):
        self.printLog("info", "Machine '%s' (%s)" % \
            (machine.getPhysicalName(), "local" if machine.isLocal() else "remote"))

        if machine.isClustered():
            self.printLog("info", "  Cluster:       %s" % machine.getCluster().getName())
        self.printLog("info", "  Physical name: %s" % machine.getPhysicalAddress())
        self.printLog("info", "  Internal name: %s" % machine.getInternalAddress())
        self.printLog("info", "  Public name:   %s" % machine.getPublicAddress())
        self.printLog("info", "")
        self.printLog("info", "  Service Endpoints (services listed in deployment order):" )

        for service in machine.getServicesInDeploymentOrder():
            self.__printService(service)

    def getInvolvedServices(self):
        # All services of site.
        services = []
        for machine in self.__site.getAllMachines():
            services += machine.getServices()

        return services

    def execute(self):
        for machine in self.__site.getAllMachines():
            self.__printMachine(machine)


class DumpDeploymentDependenciesSiteAction(app.servicecontrol.ControllerAction):
    def __init__(self, site, logger):
        self.__site = site

        super(DumpDeploymentDependenciesSiteAction, self).__init__(logger)

    def __printService(self, service, recurseFunc = None, indent=0):
        self.printLog("info", "    %s%s %s" % (
            " " * (indent * 2),
            "+" if recurseFunc and recurseFunc(service) else "-",
            service.getName()))

        if recurseFunc:
            for depService in sorted(recurseFunc(service), key=lambda s : s.getName()):
                self.__printService(depService, recurseFunc, indent + 2)

    def __printMachine(self, machine):
        self.printLog("info", "Machine '%s' (%s)" % \
            (machine.getPhysicalName(), "local" if machine.isLocal() else "remote"))
        self.printLog("info", "  Physical name: %s" % machine.getPhysicalAddress().getName())
        self.printLog("info", "  Public name:   %s" % machine.getPublicAddress().getName())
        self.printLog("info", "")
        self.printLog("info", "  Deployment Dependencies:" )

        for service in sorted(machine.getServices(), key=lambda s : s.getName()):
            self.__printService(service, lambda service : service.getDeploymentDependencies())

        self.printLog("info", "")
        self.printLog("info", "  Total Deployment Order:" )

        for service in machine.getServicesInDeploymentOrder():
            self.__printService(service)

        self.printLog("info", "")
        self.printLog("info", "  Startup Dependencies:" )

        for service in sorted(machine.getServices(), key=lambda s : s.getName()):
            self.__printService(service, lambda service : service.getStartupDependencies(machine))

        self.printLog("info", "")
        self.printLog("info", "  Total Startup Order:" )

        for service in machine.getServicesInStartOrder():
            self.__printService(service)

    def getInvolvedServices(self):
        # All services of site.
        services = []
        for machine in self.__site.getAllMachines():
            services += machine.getServices()

        return services

    def execute(self):
        for machine in self.__site.getAllMachines():
            self.__printMachine(machine)

        self.printLog("info", "")
        self.printLog("info", "Machine Startup Order:" )
        for machine in self.__site.getMachineStartupOrder():
            print "  - %s" % machine.getPhysicalName()

