﻿import datetime
import app.defaults
import app.tokens.constants
import app.tokens.defines
import app.tokens.generations
import app.options

class AutoData:
    def __init__(self, tokenDef, key = None, autoDataValues = {}):
        if key is None:
            # Generate new key.
            key = app.tokens.generations.TokenGenerations().genPassword(18)

        self.__tokenDef = tokenDef
        self.__autoDataValues = autoDataValues
        self.__key = key

    def isEmpty(self):
        return len(self.__autoDataValues) == 0

    def get(self, key):
        return self.__autoDataValues[key]

    def __generateValueForToken(self, key):
        prop = self.__tokenDef.getProperty(key, 'autogen')
        if prop is None:
            raise Exception("Missing token definition for %s" % key)

        autoGenFunction = prop.get('generate', 'genPassword(10)')

        try:
            return eval('app.tokens.generations.TokenGenerations().' + autoGenFunction)
        except Exception as e:
            raise Exception ("AutoGen Exception: Error occurred while generating value for the token [%s]. Reason: %s!!"%(key, str(e)))

    def resolveAutoToken(self, key, value):
        """ Resolves $auto$ in a given key/value token """

        if value != app.tokens.constants.AUTO_KEYWORD:
            # Nothing to do here.
            return value

        if self.__autoDataValues.has_key(key):
            # We already have a value for this.
            return self.__autoDataValues[key]
        else:
            # Auto value needs to be generated
            autoValue = self.__generateValueForToken(key)
            self.__autoDataValues[key] = autoValue
            return autoValue

    def toString(self):
        fieldDelim  = app.tokens.constants.FIELD_DELIMITER
        recordDelim = app.tokens.constants.RECORD_DELIMITER

        # Serialize values in sorted order so that we get the same
        # representation each time.
        if self.isEmpty():
            return ""

        valArray = []
        for key,value in sorted(self.__autoDataValues.iteritems()):
            valArray.append(fieldDelim.join([str(key), str(value)]))

        obfuscator = app.options.Obfuscator("", self.__deriveObfuscationKey(self.__key), self.__tokenDef)
        encData = obfuscator.obfuscate(recordDelim.join(valArray))

        return self.__key + encData

    def updateSiteOptionsFile(self, siteOptionsPath):
        newAutoDataValue = self.toString()
        existingSiteOptions = app.options.SiteOptions.readFromFilePath(siteOptionsPath)

        if existingSiteOptions.get(app.tokens.constants.AUTO_DATA_TOKEN) != newAutoDataValue:
            # Update token in file.
            app.options.PropertiesFile.updateFile(
                 siteOptionsPath,
                  app.tokens.constants.AUTO_DATA_TOKEN,
                  newAutoDataValue,
                  'This token has been generated automatically on %s. Do not modify.' % datetime.datetime.now().strftime("%Y-%m-%d %H:%M"))

    @staticmethod
    def __deriveObfuscationKey(keyData):
        posVal   = (ord(keyData[0]) + ord(keyData[1])) % 8
        keyChars = keyData[2:]
        encKey   = keyChars[posVal:posVal + 8]

        return encKey

    @staticmethod
    def readFromString(tokenDef, autoDataToken):
        """ parse AUTO_DATA token """

        if not autoDataToken:
            return AutoData(tokenDef, None, {})

        autoDataKey = autoDataToken[:18]
        encData = autoDataToken[18:]

        obfuscator = app.options.Obfuscator("",  AutoData.__deriveObfuscationKey(autoDataKey), tokenDef)
        autoData = obfuscator.deobfuscate(encData)

        autoDataValues = {}

        if autoData:
            # autoData is not empty, so parse records
            recordList = autoData.split(app.tokens.constants.RECORD_DELIMITER)
            for record in recordList:
                (key, value) = record.split(app.tokens.constants.FIELD_DELIMITER)
                autoDataValues[key] = value

        return AutoData(tokenDef, autoDataKey, autoDataValues)

    @staticmethod
    def readFromSiteOptions(tokenDef, siteOptions):
        """ parse AUTO_DATA token """
        if siteOptions.has_key(app.tokens.constants.AUTO_DATA_TOKEN):
            return AutoData.readFromString(tokenDef, siteOptions[app.tokens.constants.AUTO_DATA_TOKEN])
        else:
            return AutoData(tokenDef)


