#!/usr/bin/env python2

__author__ = 'Moorthy Rajendran <rmoorthy@collab.net>'

import os
import app.clireports.parser

class CliReportsList:

    def __init__(self, reportsDir):

        if not os.path.isdir(reportsDir):
           raise Exception("ERROR: %s is not a valid directory"%(reportsDir))

        self.reportsDir  = reportsDir
        self.reportsList = {}

        self._initReportsList()

        return

    def iter(self):
        return self.__iter__()

    def __iter__(self):
        self._iterPos = 0
        self._iterMax = len(self.reportsList)
        self._iterKeys = self.reportsList.keys()

        return self

    def next(self):

        if self._iterPos < self._iterMax:
           item = self._iterKeys[self._iterPos]
           self._iterPos = self._iterPos + 1
           return (item, self.reportsList[item])
        else:
           raise StopIteration()

        return

    def _initReportsList(self):

       for dirItem in os.listdir(self.reportsDir):
           reportData = {}
           dirPath = os.path.join(self.reportsDir, dirItem)
           if not os.path.isdir(dirPath):
              continue
           try:
              reportConfig = self._extractReportConfig(dirPath)
              if reportConfig is None:
                 continue
           except:
              # Skip if any issues while parsing the report config.ini
              continue

           reportConfigData = {}
           for section in reportConfig.sections():
               optionData = {}
               for option in reportConfig.options(section):
                   optionData[option] = reportConfig.get(section, option)
               reportConfigData[section] = optionData
           reportData['dirPath'] = dirPath
           reportData['dirName'] = dirItem
           # Send the report type as directory name itself.
           #reportData['reportType'] = dirItem[0].capitalize() + dirItem[1:]
           reportData['reportType'] = dirItem
           reportData['reportConfigData'] = reportConfigData
           self.reportsList[dirItem] = reportData

       return True

    def _extractReportConfig(self, dirPath):

        configFile = os.path.join(dirPath, 'config.ini')

        if not os.path.isfile(configFile):
           return None
        parseObj = app.clireports.parser.ConfigParser()
        parseObj.read(configFile);

        return parseObj


