﻿#!/usr/bin/env python2

__author__ = 'Moorthy Rajendran <rmoorthy@collab.net>'

import os
import commands
import app.clireports.info
import app.clireports.metainfo
import app.clireports.constants
import app.clireports.dbstore
import json

class SyncApp:

    def __init__(self, reportsDir):

        try:
           self.dbi     = app.clireports.dbstore.DBase()
           self.cliInfo = app.clireports.info.CliReportsList(reportsDir)
           self.runtimeCfg  = app.util.getRuntimeConfiguration()
           self.cliMetaInfo = app.clireports.metainfo.CliReportsMetaInfo(os.path.join(self.runtimeCfg.get('DATA_DIR'), 'etc', 'clireportspkg.ini'))
           self.reportsDir = reportsDir
        except Exception as e:
           raise e

        return

    def prepareSyncCliReports(self):

        try:
           syncedReportsPkgValue = self.cliMetaInfo.getConfig(self.reportsDir, 'reportsPackages')
           if syncedReportsPkgValue is None:
              return
           syncedReportsPackages = set(syncedReportsPkgValue.split(','))
           currntReportsPackages = set(self.cliInfo.reportsList.keys())
           removedReportsPackages = syncedReportsPackages.difference(currntReportsPackages)
           for rptPackage in removedReportsPackages:
               rptMetaInfoVal = self.cliMetaInfo.getConfig(self.reportsDir, rptPackage)
               if rptMetaInfoVal is None:
                  continue
               (reportType, category) = rptMetaInfoVal.split(',')
               self.dbi.softDeleteReportTypeMetaData(reportType)
               self.cliMetaInfo.delConfig(self.reportsDir, rptPackage)
        except Exception:
           pass

    def finaliseSyncCliReports(self):

        try:
           self.cliMetaInfo.setConfig(self.reportsDir, 'reportsPackages', ','.join(self.cliInfo.reportsList.keys()))
           for reportName, reportData in self.cliInfo.iter():
               try:
                  reportType = reportData['reportType']
               except:
                  reportType = reportName

               try:
                  category = reportData['reportConfigData']['main']['category']
               except:
                  category = app.clireports.constants.DEFAULT_CATEGORY
               self.cliMetaInfo.setConfig(self.reportsDir, reportName, ','.join([reportType, category]))
        except Exception:
           pass

    def convertToBool(self, value):

        return True if value.lower() == 'true' else False

    def syncCliReports(self):

        self.prepareSyncCliReports()

        for reportName, reportData in self.cliInfo.iter():

            try:
               reportType = reportData['reportType']
            except:
               reportType = reportName

            try:
               category = reportData['reportConfigData']['main']['category']
            except:
               category = app.clireports.constants.DEFAULT_CATEGORY

            try:
               timeToLive = reportData['reportConfigData']['main']['timeToLive']
            except:
               timeToLive = app.clireports.constants.DEFAULT_TIME_TO_LIVE

            try:
               almRequired = reportData['reportConfigData']['main']['almRequired']
            except:
               almRequired = app.clireports.constants.DEFAULT_FILTER_TYPE_VALUE

            try:
               datamartRequired = reportData['reportConfigData']['main']['datamartRequired']
            except:
               datamartRequired = app.clireports.constants.DEFAULT_FILTER_TYPE_VALUE

            try:
               eventsRequired = reportData['reportConfigData']['main']['eventsRequired']
            except:
               eventsRequired = app.clireports.constants.DEFAULT_FILTER_TYPE_VALUE

            try:
               devOpsRequired = reportData['reportConfigData']['main']['devOpsRequired']
            except:
               devOpsRequired = app.clireports.constants.DEFAULT_FILTER_TYPE_VALUE

            try:
               collabRequired = reportData['reportConfigData']['main']['collabRequired']
            except:
               collabRequired = app.clireports.constants.DEFAULT_FILTER_TYPE_VALUE

            try:
               scmRequired = reportData['reportConfigData']['main']['scmRequired']
            except:
               scmRequired = app.clireports.constants.DEFAULT_FILTER_TYPE_VALUE

            almRequired = self.convertToBool(almRequired)
            datamartRequired = self.convertToBool(datamartRequired)
            eventsRequired = self.convertToBool(eventsRequired)
            devOpsRequired = self.convertToBool(devOpsRequired)
            collabRequired = self.convertToBool(collabRequired)
            scmRequired = self.convertToBool(scmRequired)

            try:
               sampleImageFile  = os.path.join(reportData['dirPath'], app.clireports.constants.DEFAULT_SAMPLE_IMAGE)
               dstImageFileName = None
               if os.path.isfile(sampleImageFile):
                  dstImageFile = self.syncImage(reportType, sampleImageFile)
                  dstImageFileName = os.path.basename(dstImageFile)
               else:
                  raise Exception("CLI Report sample image file %s sync failed. Using default image url"%sampleImageFile)
               imageUrl = "%s/%s"%(app.clireports.constants.DEFAULT_SAMPLE_IMAGE_URL_LOCATION,dstImageFileName)
            except Exception as e:
               imageUrl = app.clireports.constants.DEFAULT_SAMPLE_IMAGE_URL

            try:
               jsonFormConfigData  = self.extractFormConfigAsJSON(reportData['reportConfigData'].copy())
               jsonFormConfigData = '' if jsonFormConfigData is None else jsonFormConfigData

               jsonChartConfigData = self.extractChartConfigAsJSON(reportData['reportConfigData'].copy())
               jsonChartConfigData = '' if jsonChartConfigData is None else jsonChartConfigData

               # Synchronise Report Category
               if not self.dbi.ifReportCategoryExist(category):
                  try:
                     categoryTitle = app.clireports.constants.CATEGORY_DESCRIPTION_MAP.get(category).get('title') \
                                     if app.clireports.constants.CATEGORY_DESCRIPTION_MAP.has_key(category) \
                                     else category
                  except:
                     categoryTitle = category
                  try:
                      categoryDesc = app.clireports.constants.CATEGORY_DESCRIPTION_MAP.get(category).get('description') \
                                     if app.clireports.constants.CATEGORY_DESCRIPTION_MAP.has_key(category) \
                                     else category
                  except:
                     categoryDesc = category
                  self.dbi.addReportCategory(category, categoryTitle, categoryDesc)

               # Synchronise Report Type
               if not self.dbi.ifReportTypeExist(reportType):
                  self.dbi.addReportType(reportType, reportData['reportConfigData']['main'].get('title', reportType),
                       reportData['reportConfigData']['main'].get('description', reportType), imageUrl, almRequired, datamartRequired, eventsRequired, devOpsRequired, collabRequired, scmRequired)
               else:
                  self.dbi.modifyReportType(reportType, reportData['reportConfigData']['main'].get('title', reportType),
                       reportData['reportConfigData']['main'].get('description', reportType), imageUrl, almRequired, datamartRequired, eventsRequired, devOpsRequired, collabRequired, scmRequired)

               # Synchronise Report Type Meta Data
               # one active type can exist in one category, the rest needs to be made as inactive.
               if not self.dbi.ifReportTypeMetaDataExist(reportType):
                  self.dbi.addReportTypeMetaData(reportType, category, jsonFormConfigData, jsonChartConfigData, timeToLive)
               else:
                  if self.dbi.ifReportTypeCategoryMetaDataExist(reportType, category):
                      self.dbi.modifyReportTypeMetaData(reportType, category, jsonFormConfigData, jsonChartConfigData, timeToLive)
                  else:
                      self.dbi.softDeleteReportTypeMetaData(reportType)
                      self.dbi.addReportTypeMetaData(reportType, category, jsonFormConfigData, jsonChartConfigData, timeToLive)

            except Exception as e:
               print e
               raise e

        self.finaliseSyncCliReports()

    def extractFormConfigAsJSON(self, configData):
        """
        Filters out all the chart config sections
        """

        if not configData.has_key('main'):
           return None

        reportFieldConfigList = []
        try:
           if configData['main'].has_key('fields'):
              reportFields = configData['main']['fields'].replace(' ','').split(',')
              for field in reportFields:
                 if not configData.has_key(field):
                    continue
                 fieldConfig = {}
                 fieldConfig['name'] = field
                 for key,value in configData[field].items():
                    fieldConfig[key] = value
                 reportFieldConfigList.append(fieldConfig)
              if not len(reportFieldConfigList):
                 return None
           # Generate chartDisplayType field
           if configData['main'].has_key('outputType'):
              outputTypeVal = configData['main']['outputType']
              fieldConfig = {}
              fieldConfig['name']  = 'chartDisplayType'
              fieldConfig['label'] = 'Chart Display Type'
              fieldConfig['type']  = 'select'
              fieldConfig['values']  = outputTypeVal
              fieldConfig['default'] = outputTypeVal.split(',')[0]
              reportFieldConfigList.append(fieldConfig)
           # End Generate chartDisplayType field
           jsonConfigData  = json.dumps({'fields': reportFieldConfigList})
           return jsonConfigData
        except:
           return None

    def extractChartConfigAsJSON(self, configData):
        """
        Filters out all the config sections other than chart config sections
        """

        if not configData.has_key('main'):
           return None

        if configData['main'].has_key('outputType'):
           outputTypeVal = configData['main']['outputType']

        try:
           chartConfig = {}
           # delete chart config sections
           for outputType in outputTypeVal.split(','):
               outputType = outputType.strip()
               if configData.has_key(outputType):
                  chartConfig[outputType] = configData[outputType]
           jsonConfigData = json.dumps(chartConfig)
           return jsonConfigData
        except:
           return None

    def syncImage(self, reportType, sampleImageFile):

        try:
           dstImageFile = os.path.join(app.clireports.constants.DEFAULT_SAMPLE_IMAGE_FILE_DIRECTORY,
                            "%s_%s"%(reportType, app.clireports.constants.DEFAULT_SAMPLE_IMAGE))
           self._runSystem('cp -f "%s" "%s"'%(sampleImageFile, dstImageFile), True)
           return dstImageFile
        except Exception as e:
           raise Exception("Copying sample image file failed %s!!"%(str(e)))

    def _runSystem(self, cmd, raiseError=False):

        (code, output) = commands.getstatusoutput(cmd)
        if code != 0 and raiseError == True:
           raise Exception()
        return code
