﻿import socket
import urllib2

class HealthCheck(object):
    def __init__(self, name, url, logger):
        self.__name = name
        self.__url = url
        self.__logger = logger

    def invoke(self, timeoutRetry=3):
        timeoutSecs = 5
        try:
            statusCode = urllib2.urlopen(self.__url, timeout=timeoutSecs).getcode()
            self.__logger.info("Health check %s at %s returned: %d" % \
                             (self.__name, self.__url, statusCode))

            return statusCode == 200

        except urllib2.HTTPError as e:
            self.__logger.error("Health check %s at %s failed with status: %d, details: %s" % \
                              (self.__name, self.__url, e.code, e.read()))
            return False

        except urllib2.URLError as e:
            self.__logger.error("Health check %s at %s failed: %s" % \
                              (self.__name, self.__url, e.reason))
            return False
        except socket.timeout as e:
            timeoutRetry -= 1
            if timeoutRetry:
                self.invoke(timeoutRetry)
            else:
                self.__logger.error("Health check %s at %s timed out after %d sec" % \
                                  (self.__name, self.__url, timeoutSecs))
                return False

        except socket.error as e:
            self.__logger.error("Health check %s at %s failed with ERRNO %r" % \
                              (self.__name, self.__url, e.errno))
            return False

        except:
            self.__logger.error("Health check %s at %s failed" % \
                              (self.__name, self.__url))
            return False
