﻿import os
import subprocess
import app.util

class Migratorator(object):
    EXIT_SUCCESS = 0
    EXIT_MIGRATION_REQUIRED = 1
    EXIT_MIGRATION_FAILED = 2
    EXIT_WRONG_USAGE= 3

    def __init__(self, jarPath, args):

        self.__javaCommand = [
            os.path.join(app.defaults.DEFAULT_JAVA_HOME, 'bin/java'),
            '-Djava.security.egd=file:///dev/urandom',
            '-Xms128m',
            '-Xmx1024m',
            '-Xss8m',
            '-jar',
            jarPath
            ]
        self.__args = args

    @staticmethod
    def forDatabase(jarPath, jdbcUrl, username, password):
        return Migratorator(
            jarPath,
            [
                jdbcUrl,
                username,
                password,
                "",
                "false"
            ])

    @staticmethod
    def forDatamart(jarPath, databaseJdbcUrl, databaseUsername, databasePassword,
                             datamartJdbcUrl, datamartUsername, datamartPassword):
        return Migratorator(
            jarPath,
            [
                databaseJdbcUrl,
                databaseUsername,
                databasePassword,
                "",
                "true",
                datamartJdbcUrl,
                datamartUsername,
                datamartPassword,
                ""
            ])

    def migrate(self, logger):
        logger.debug("Invoking migratorator to migrate schema: %s" % " ".join(self.__args))

        process = subprocess.Popen(
            self.__javaCommand + self.__args,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)

        stdOut, stdErr = process.communicate()
        exitCode = process.poll()

        if exitCode != 0:
            logger.error("Migratorator exited with status code %d", exitCode)
            logger.error(stdOut)
            logger.error(stdErr)
            raise MigratoratorException("Invoking migratorator failed with exit code %d" % exitCode)


    def isSchemaUpToDate(self, logger):
        logger.debug("Invoking migratorator to check if schema up to date: %s" % " ".join(self.__args))

        process = subprocess.Popen(
            self.__javaCommand + ["--validate-only"] + self.__args,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)

        stdOut, stdErr = process.communicate()
        exitCode = process.poll()

        if exitCode == 0:
            return True
        elif exitCode == Migratorator.EXIT_MIGRATION_REQUIRED:
            # Not up to date.
            logger.info("Schema not up to date")
            return False
        else:
            logger.error("Migratorator exited with status code %d", exitCode)
            logger.error(stdOut)
            logger.error(stdErr)
            raise MigratoratorException("Invoking migratorator failed with exit code %d" % exitCode)


class MigratoratorException(Exception):
    pass
