import app.util

class DictEqualityMixin(object):
    """ Mixing providing default implementation for equality tests used by set() """

    def __eq__(self, other):
        return (isinstance(other, self.__class__)
            and self.__dict__ == other.__dict__)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash(self.__repr__())


class SelinuxMixin(object):
    """ Mixing provide functionality to get and set the selinux mode """

    def getSelinuxMode(self):
        return app.util.executeCommand("/usr/sbin/getenforce").strip()

    def setSelinuxPermissive(self):
        app.util.executeCommand("/usr/sbin/setenforce 0")

    def resetSelinuxMode(self, mode):
        if mode == "Enforcing":
            app.util.executeCommand("/usr/sbin/setenforce 1")
