﻿import signal
import socket
from contextlib import closing
import urllib2
import ssl
import sys

def isPortListening(host, port, logger, socketTimeout=20):
    """ Performs a shallow check on whether a TCP port is listening """

    def alarmHandler(signum,frame):
        raise ConnectionTimeoutException("Connection timeout")

    logger.info("Checking endpoint %s:%d..." % (host, port))
    try:
        # Install signal handler to avoid connect() from blocking indefinitely
        signal.signal(signal.SIGALRM, alarmHandler)
        signal.alarm(socketTimeout)

        with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
            errno = sock.connect_ex((host, port))
            if errno == 0:
                logger.info("Endpoint %s:%d is available" % (host, port))
                return True
            else:
                logger.error("Endpoint %s:%d not available: %d" % (host, port, errno))
                return False

    except ConnectionTimeoutException:
        logger.error("Connectingt to endpoint %s:%d timed out" % (host, port))
        return False
    except Exception, sockErrMsg:
        if socket.herror:
            logger.error("Host not found %s" % host)
        else:
            logger.error("Connecting to endpoint %s:%d is failed and error is %s" %
                    (host, port, sockErrMsg))
        return False
    finally:
        signal.alarm(0)


def isHttpListening(host, port, logger, timeoutSecs = 5):
    """ Performs a shallow check on whether an HTTP endpoint is listening """

    url = "%s://%s:%d" % ("https" if port == 443 else "http", host, port)

    try:
        if sys.version_info >= (2, 7):
            ctx = ssl.create_default_context()  # pylint: disable=E1101
            ctx.check_hostname = False
            ctx.verify_mode = ssl.CERT_NONE

            statusCode = urllib2.urlopen(  # pylint: disable=E1123
                url,
                timeout=timeoutSecs,
                context=ctx).getcode()
        else:
            # Python 2.6 does not validate SSL certs and does not have
            # ssl.create_default_context().
            statusCode = urllib2.urlopen(
                url,
                timeout=timeoutSecs).getcode()

        logger.info("HTTP endpoint %s returned: %d" % (url, statusCode))

        # Any 2xx, 3xx, 4xx means that something is available (otherwise, an
        # exception would have been raised).
        return True

    except urllib2.HTTPError as e:
        logger.error("HTTP endpoint %s failed with status: %d, details: %s" % \
                            (url, e.code, e.read()))
        return False

    except urllib2.URLError as e:
        logger.error("HTTP endpoint %s failed: %s" % (url, e.reason))
        return False

    except socket.timeout as e:
        logger.error("HTTP endpoint %s timed out after %d sec" % (url, timeoutSecs))
        return False

    except socket.error as e:
        logger.error("HTTP endpoint %s failed with ERRNO %r" % (url, e.errno))
        return False

    except:
        logger.error("HTTP endpoint %s failed: %r" % (url, sys.exc_info()))
        return False

class ConnectionTimeoutException(Exception):
    pass
