import os
import pwd
import grp
import stat

class Permissions(object):

    @staticmethod
    def isPermissionApplied(path, mode=None, owner=None, group=None):
        if not os.path.exists(path):
            return True
        stat_info = os.stat(path)
        if mode and Permissions.getMode(mode) != stat.S_IMODE(stat_info.st_mode):
            return False
        if owner and group:
            actualUId = stat_info.st_uid
            actualGId = stat_info.st_gid
            expectedUId = Permissions.getOwnerUid(owner)
            expectedGId = Permissions.getGroupGid(group)
            if expectedUId != actualUId or expectedGId != actualGId:
                return False

        return True

    @staticmethod
    def getMode(mode):
        return int(str(mode), 8)

    @staticmethod
    def getOwnerUid(owner=None):
        if owner:
            try:
                return pwd.getpwnam(owner).pw_uid
            except:
                raise PermissionsException("Unknown user '%s'" % owner)
        else:
            return os.geteuid()

    @staticmethod
    def getGroupGid(group=None):
        if group:
            try:
                return grp.getgrnam(group).gr_gid
            except:
                raise PermissionsException("Unknown group '%s'" % group)
        else:
            return os.getegid()

    @staticmethod
    def applyPermissions(path, mode=None, owner=None, group=None, recursive=True):
        uid = None
        gid = None
        if not os.path.exists(path):
            return
        if os.path.isfile(path):
            recursive = False

        if mode:
            mode = Permissions.getMode(mode)
        if owner and group:
            uid = Permissions.getOwnerUid(owner)
            gid = Permissions.getGroupGid(group)

        print "Applying permissions to %s" % path
        if recursive:
            Permissions.__applyPermissionsRecursively(path, mode, uid, gid)
        else:
            Permissions.__applyPermissions(path, mode, uid, gid)

    @staticmethod
    def __applyPermissionsRecursively(path, mode, uid, gid):
        # Apply to folder itself....
        Permissions.__applyPermissions(path, mode, uid, gid)

        # ...and all its descendants (not following links).
        for root, dirs, files in os.walk(path):
            for dir in dirs:
                Permissions.__applyPermissions(os.path.join(root, dir), mode, uid, gid)
            for file in files:
                Permissions.__applyPermissions(os.path.join(root, file), mode, uid, gid)


    @staticmethod
    def __applyPermissions(path, mode, uid, gid):
        if mode:
            os.chmod(path, mode)
        if uid and gid:
            os.chown(path, uid, gid)

class PermissionsException(Exception):
    pass
