﻿
import os
import StringIO
import commands
import time
import sys

import app.util
import app.logutil
import app.landscape
import app.defaults
from app.linuxservice import ServiceManager
from app.servicecontrol import ServiceControlGlobalFlags

class PGDBAdmin(object):
    def __init__(self, dbname, user, password, readOnlyUser,
                 readOnlyPassword, logger, port):
        self.dbname = dbname
        self.user = user
        self.password = password
        self.readOnlyUser = readOnlyUser
        self.readOnlyPassword = readOnlyPassword
        self.logger = logger
        self.port = port

    def __runAsPostgres(self,cmd):
        pgCmd = "su - postgres -c \"\"\" %s \"\"\"" % cmd
        return app.util.runCommand(pgCmd, logger=self.logger)

    def dropDBIfExists(self):
        dropDbCmd = "dropdb -p %s --if-exists %s" % (self.port, self.dbname )
        self.__runAsPostgres(dropDbCmd)

    def _alterDBPasswd(self, user, password):

        setPasswordCmd = "psql -p %s -c \\\"ALTER USER \\\\\\\"%s\\\\\\\" WITH PASSWORD '%s' \\\"" % (self.port, user, password)
        self.__runAsPostgres(setPasswordCmd)


    def __reCreateUserWithPermission(self, user, password, permission_flags):
        dropOwnedCmd = "psql -p %s -c \\\"DROP OWNED BY \\\\\\\"%s\\\\\\\" \\\"" % (self.port, user)
        try:
            self.__runAsPostgres(dropOwnedCmd)
        except:
            pass

        dropUsrCmd = "dropuser -p %s --if-exists %s" % (self.port, user)
        self.__runAsPostgres(dropUsrCmd)

        createUsrCmd = "createuser -p %s %s %s" %(self.port, ' '.join(permission_flags), user)
        self.__runAsPostgres(createUsrCmd)

        self._alterDBPasswd(user, password)

    def reCreateRWUser(self, user, password):
        return self.__reCreateUserWithPermission(user, password,
            ["--login", "--createdb", "--no-createrole"])

    def reCreateROUser(self, user, password):
        return self.__reCreateUserWithPermission(user, password,
            ["--login", "--no-createdb", "--no-createrole"])

    def reCreateUsers(self):

        try:
            userName = self.user
            self.reCreateRWUser(self.user,self.password)

            if self.readOnlyUser:
                userName = self.readOnlyUser
                self.reCreateROUser(self.readOnlyUser, self.readOnlyPassword)
        except Exception:
            raise Exception(
                    "Dropping the Postgres user %s failed.Please make sure"
                    " that there are no other databases referring to this user" %
                    userName )

    def createDB(self):
        createDBCmd = "createdb -p %s --encoding UTF8  -O %s %s" % (self.port,  self.user, self.dbname)
        self.__runAsPostgres(createDBCmd)

    def isDatabaseExisting(self):

        command = "su - postgres -c \"psql -p %s -tAc 'SELECT datname FROM pg_database' | grep -qw %s\"" % (self.port, self.dbname)
        (exitCode, output) = commands.getstatusoutput(command)

        return exitCode == 0

    def createDBUser(self, permissionFlags):

        createUsrCmd = "createuser -p %s %s %s" % (self.port, ' '.join(permissionFlags), self.user)
        self.__runAsPostgres(createUsrCmd)

    def dropDBUserIfExists(self):

        dropUsrCmd = "dropuser -p %s --if-exists %s" % (self.port, self.user)
        self.__runAsPostgres(dropUsrCmd)

    def setDBPasswd(self):

        self._alterDBPasswd(self.user, self.password)



class PGCTFDBManager(PGDBAdmin):

    def __init__(self, dbname, user, password, readOnlyUser,
                 readOnlyPassword, logger, port,host,passFile):
        super(PGCTFDBManager, self).__init__(dbname, user, password, readOnlyUser, readOnlyPassword, logger, port)
        self.host = host
        self.passFile = passFile

    def __runSQLAsDBAdmin(self, sql):
        pgCmd = """ psql -U'%s' -h'%s' -p'%s' '%s' -c ' %s '  """ % (self.user, self.host,self.port,self.dbname,sql)
        os.environ['PGPASSWORD'] = self.password
        os.environ['PGPASSFILE'] = self.passFile

        return app.util.runCommand(pgCmd, logger=self.logger)

    def grantReadOnlyPermissionsForAllTablesInSchema(self):
        grantPermission = "GRANT SELECT ON ALL TABLES IN SCHEMA public TO \"%s\" " % self.readOnlyUser
        self.__runSQLAsDBAdmin(grantPermission)

        alterDefaultPrivileges = "ALTER DEFAULT PRIVILEGES FOR ROLE \"%s\"  IN SCHEMA public GRANT SELECT ON TABLES TO \"%s\" ;" % (self.user, self.readOnlyUser)
        self.__runSQLAsDBAdmin(alterDefaultPrivileges)

    def revokePermissionForTables(self, tableList):
        revokePermission = "REVOKE ALL ON table %s FROM \"%s\" " % (",".join(tableList ), self.readOnlyUser)
        self.__runSQLAsDBAdmin(revokePermission)

class HbaConfig(object):
    """ Helper class for managing pg_hba.conf """

    def __init__(self):
        self.__buffer = StringIO.StringIO()

    def __repr__(self):
        return self.__buffer.getvalue()

    def format(self):
        return self.__buffer.getvalue()

    def allowAccessForIp(self, databaseName, user, ip, isSsl):
        if not ip:
            raise AttributeError("ip")

        self.__buffer.write("\n%s   %s  %s  %s         255.255.255.255   md5" % (
            "hostssl" if isSsl else "host",
            databaseName,
            user,
            ip))
        return self

    def allowAccessForLocalHost(self, databaseName, user, isSsl):
        self.__buffer.write("\n%s   %s  %s  samehost  md5" % (
            "hostssl" if isSsl else "host",
            databaseName,
            user))
        return self

    def allowAccessForLocalSubnet(self, databaseName, user, isSsl):
        self.__buffer.write("\n%s   %s  %s  samenet  md5" % (
            "hostssl" if isSsl else "host",
            databaseName,
            user))
        return self

    def allowAccessForMachine(self, databaseName, user, machine, isSsl):
        for ip in ["127.0.0.1", machine.getPhysicalAddress().getIp()]:
            self.allowAccessForIp(databaseName, user, ip, isSsl)

        return self

    def allowAccessForService(self, databaseName, user, serviceName, isSsl):
        self.__buffer.write("\n# <%s>" % serviceName)

        if ServiceManager.isRunningInContainer():
            # In a container, any kind of IP ACLs are not very helpful as IPs
            # are dynamiy. Therefore, apply subnet ACLs only.
            self.allowAccessForLocalSubnet(databaseName, user, isSsl)

        else:
            ips = [machine.getPhysicalAddress().getIp() for machine in app.landscape.Landscape.getSite().getMachinesByService(serviceName)]
            ips.append("127.0.0.1")

            for ip in ips:
                self.allowAccessForIp(databaseName, user, ip, isSsl)

        self.__buffer.write("\n# </%s>\n" % serviceName)
        return self

class PGCluster:
    """ Helper class to initialize postgres cluster, migrate database to latest """
    @staticmethod
    def doInitializeandMigrate(logger, pgHomeDir, oldPGDataDir, pgDataDir, initCommand):

        cfg = app.util.getRuntimeConfiguration()

        # Display the warning if 9.2 cluster found
        if not os.path.exists(oldPGDataDir) and not os.path.exists(pgDataDir) and os.path.exists("/var/lib/pgsql/9.2"):
            msgData = "\nA Postgres 9.2 instance has been found on your system."\
                      " The data from Postgres 9.2 cannot be migrated automatically"\
                      "\nTo migrate manually, please..."\
                      "\n . Dump your existing databases"\
                      "\n . Remove the Postgres cluster in /var/lib/pgsql/9.2"\
                      "\n . Run 'teamforge provision' again to create a new database"\
                      "\n . Stop the site"\
                      "\n . Start the postgres server only"\
                      "\n . Drop and recreate the teamforge database"\
                      "\n . Restore your databases by loading the dumps"\
                      "\n . Rerun the 'teamforge provision' to do the migration and start the site"

            app.logutil.printLog(logger, "warn", msgData)
            if ServiceControlGlobalFlags.confirmDangerousActions:
                choice = app.util.readFromUser("Do you want to ignore your existing data and have the "\
                                               "installer create new databases instead? [y/N]", ["y", "N"])
            else:
                choice = "y"

            if choice != "y":
                sys.exit(0)

        # Initialize the cluster
        PGCluster.initializeCluster(logger, pgHomeDir, pgDataDir, initCommand)

        if os.path.exists(oldPGDataDir):
            if ServiceControlGlobalFlags.confirmDangerousActions:
                opt = app.util.readFromUser("Your PostgreSQL installation has been found to be outdated and " \
                                            "needs to be upgraded to version %s. This can be done automatically."\
                                            "Do you wish to proceed? [y/N]" % cfg["PGSQL_VERSION"], ["y","N"])
            else:
                opt = "y"

            if opt == "y":
                if not PGCluster.arePostgresRpmPackagesInstalled(app.defaults.OLD_PGVERSION.split(".")[0] +
                                                                 app.defaults.OLD_PGVERSION.split(".")[1], logger):
                    logMsg = "To migrate your existing data, the old PostgreSQL %s RPM packages are needed."\
                             "Please reinstall these and try again." % app.defaults.OLD_PGVERSION
                    raise Exception(logMsg)

                # Create table space dir's if not exists
                # This is for 8.x compatibility and actually does not have any significance for later releases

                dataDir = os.path.basename(pgDataDir)
                tableSpaceDir = os.path.join(app.defaults.DEFAULT_SITE_DIR, "pgsqltmp")

                if os.path.isdir(tableSpaceDir):
                    os.system("rm -rf %s"%tableSpaceDir)

                if dataDir == "data":
                    PGCluster.createTableSpaceDir(logger, os.path.join(tableSpaceDir, "ctf_pgsql_tmp"))
                elif dataDir == "reports":
                    PGCluster.createTableSpaceDir(logger, os.path.join(tableSpaceDir, "reports_pgsql_tmp"))

                # Migrating cluster
                try:
                    PGCluster.migrateCluster(logger, oldPGDataDir, pgDataDir)
                except Exception as e:
                    os.system("rm -rf %s"%pgDataDir)
                    raise e

                # Renaming old pgcluster after migration
                PGCluster.renameOldCluster(logger, oldPGDataDir)

    @staticmethod
    def initializeCluster(logger, pgHomeDir, pgDataDir, initCommand):
        if os.path.islink(pgHomeDir):
            pgHomeDir= os.readlink(pgHomeDir)

        pgPath = os.path.join(pgDataDir, "base")

        if not os.path.exists(pgPath):
            app.util.runSystem("mkdir -p %s" % pgHomeDir, logger)
            app.util.runSystem("chown -R postgres:postgres %s" % pgHomeDir, logger)
            logger.debug("Initializing the cluster in %s...Please wait" % pgDataDir)
            app.util.runCommand("%s initdb" % initCommand, logger)

    @staticmethod
    def migrateCluster(logger, oldPGDataDir, pgDataDir):
        cfg = app.util.getRuntimeConfiguration()
        oldPGVersion = app.defaults.OLD_PGVERSION
        pgVersion = cfg["PGSQL_VERSION"]

        if os.path.islink(oldPGDataDir):
            oldPGDataDir = os.readlink(oldPGDataDir)

        logger.debug("Stopping the old postgresql %s instance for migration..." % oldPGVersion)
        ServiceManager.getService("postgresql-%s" % oldPGVersion).stop(False)

        # In upgrade new h/w the DB backup also contains the pid file
        # Installer assumes old postgres is running and causing the error
        # Handle it by removing the pid file
        oldPGPidfile = os.path.join(oldPGDataDir, "postmaster.pid")
        if os.path.isfile(oldPGPidfile):
            os.remove(oldPGPidfile)

        logger.debug("Migrating the cluster in %s...Please wait" % pgDataDir)
        pgUpgradeCmd = "su - postgres -c '/usr/pgsql-%s/bin/pg_upgrade -d %s -D %s -b /usr/pgsql-%s/bin -B /usr/pgsql-%s/bin'"%(pgVersion,
                                                                         oldPGDataDir, pgDataDir, oldPGVersion, pgVersion)

        app.util.runCommand(pgUpgradeCmd, logger)
        msgData = "Please run the './analyze_new_cluster.sh' from '/var/lib/pgsql/' after finishing the provision."
        app.logutil.printLog(logger, "warn", msgData)

    @staticmethod
    def arePostgresRpmPackagesInstalled(version, logger):
        return os.system("rpm -q postgresql%s-server >>/dev/null" %version) == 0

    @staticmethod
    def createTableSpaceDir(logger, pgTablespaceDir):
        if not os.path.exists(pgTablespaceDir):
            logger.debug("Creating table space directory %s" % pgTablespaceDir)
            os.makedirs(pgTablespaceDir)
            app.util.runSystem("chown -R postgres:postgres %s" %pgTablespaceDir, logger)

    @staticmethod
    def renameOldCluster(logger, oldPGCluster):
        migratedDate = time.strftime("%Y-%m-%d")
        os.rename(oldPGCluster, oldPGCluster+"-migratedon-%s"%migratedDate)

        dataDir = os.path.basename(oldPGCluster)
        # Renaming if exists in /var/lib/pgsql also
        oldlibPGCluster = os.path.join("/var/lib/pgsql/", app.defaults.OLD_PGVERSION, dataDir)
        if os.path.exists(oldlibPGCluster):
            os.rename(oldlibPGCluster, oldlibPGCluster+"-migratedon-%s"%migratedDate)
