import os
import subprocess
import app.util

class Schemerator(object):
    EXIT_SUCCESS = 0
    EXIT_SCHEMA_EXISTS_ALREADY = 2

    def __init__(self, jarPath, dbUrlOrName, username, password):
        self.__javaCommand = [
            os.path.join(app.defaults.DEFAULT_JAVA_HOME, 'bin/java'),
            '-Djava.security.egd=file:///dev/urandom',
            '-Xms128m',
            '-Xmx1024m',
            '-Xss8m',
            '-jar',
            jarPath,
            dbUrlOrName,
            username,
            password
            ]

    def run(self, logger):
        logger.debug("Running %s " % " ".join(self.__javaCommand))

        process = subprocess.Popen(
            self.__javaCommand,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)

        stdOut, stdErr = process.communicate()
        exitCode = process.poll()

        if exitCode == Schemerator.EXIT_SCHEMA_EXISTS_ALREADY:
            raise SchemeratorSchemaExistsException("Database schema could not ve created as the database already contains a schema")

        elif exitCode != Schemerator.EXIT_SUCCESS:
            logger.error("Schemerator exited with status code %d", exitCode)
            logger.error(stdOut)
            logger.error(stdErr)
            raise SchemeratorException("Invoking schemerator failed with exit code %d" % exitCode)

class SchemeratorException(Exception):
    pass

class SchemeratorSchemaExistsException(SchemeratorException):
    pass
