﻿#!/usr/bin/env python2

import os.path

import app.util
import app.logutil
import commands
import app.colors

class SELinuxSetting:
    cfg = app.util.getRuntimeConfiguration()
    logger = app.logutil.getBootstrapLogger()

    def __init__(self):
        pass

    def changeContext(self, context, location, recurFlag='Yes'):
        if os.path.exists(location):
            reallocation = os.path.realpath(location)
            if location != reallocation:
                # Set context for symlinks
                isContextNotSet = self.matchPathconCheck(location, context)
                if isContextNotSet:
                   self.logger.info("Changing context of %s to %s" %(location, context))
                   self.doSemanage(context, location,recurFlag)
                   self.doRestorecon(location,recurFlag)
                else:
                   self.doRestorecon(location,recurFlag)
            isContextNotSet = self.matchPathconCheck(reallocation, context)
            if isContextNotSet:
               self.logger.info("Changing context of %s to %s" %(reallocation, context))
               self.doSemanage(context, reallocation,recurFlag)
               self.doRestorecon(reallocation,recurFlag)
            else:
               self.doRestorecon(location,recurFlag)

    def GetEquivalenceRule (self,reallocation):
        seEquivalenceCmd = '/usr/sbin/semanage fcontext -l -C'
        matching = 0
        (ret,cmdoutput) = commands.getstatusoutput(seEquivalenceCmd)
        if not ret:
           output = cmdoutput.split("SELinux Local fcontext Equivalence")
           if len(output) > 1:
              equivalence_list = output[1].strip(" \n ")
              Lines = equivalence_list.split("\n")
              for line in Lines:
                  splline = line.split(" ")
                  if reallocation.startswith(splline[0]):
                     matching = 1
                     break
        else:
            self.logger.error("Failed while running the command - %s" %seEquivalenceCmd)

        return matching


    def doSemanageEqual(self, sourcepath, targetpath):
        seLinkContextCmd = "/usr/sbin/semanage fcontext -a -e '%s' '%s'" %(sourcepath, targetpath)
        try:
            self.logger.info("Invoke doSemanageEqual - %s" %seLinkContextCmd)
            app.util.runCommand(seLinkContextCmd, self.logger)
        except:
            self.logger.error("Failed while running the command - %s" %seLinkContextCmd)

    def doSemanage(self, type, object,recurFlag):
        if os.path.isfile(object) or recurFlag == "No":
            seContextCmd = "/usr/sbin/semanage fcontext -a -t %s %s" %(type, object)
        else:
            seContextCmd = "/usr/sbin/semanage fcontext -a -t %s '%s(/.*)?'" %(type, object)
        try:
            self.logger.info("Invoke doSemanage - %s" %seContextCmd)
            app.util.runCommand(seContextCmd, self.logger)
        except:
            self.logger.error("Failed while running the command - %s" %seContextCmd)

    def doRestorecon(self, object, recurFlag):
        restoreCmd = '/sbin/restorecon'
        if recurFlag == 'Yes' or recurFlag == '':
           restoreCmd = restoreCmd + ' -R'
        restoreCmd = restoreCmd + ' -v %s'  %object
        try:
            app.util.runCommand(restoreCmd, self.logger)
        except:
            self.logger.error("Failed while running the command - %s" %restoreCmd)

    def changeBoolean(self, value, bool):
        seBooleanCmd = "/usr/sbin/semanage boolean -m %s %s" %(value, bool)
        try:
            self.logger.info("Setting boolean - %s" %seBooleanCmd)
            app.util.runCommand(seBooleanCmd, self.logger)
        except:
            self.logger.error("Failed while running the command - %s" %seBooleanCmd)

    def checkBooleanOn(self, bool):
        getseBooleanCmd = "/usr/sbin/getsebool %s" %( bool)
        (retVal, outputVal) = commands.getstatusoutput(getseBooleanCmd)
        booleanVal = outputVal.split("--> ")[1]
        return booleanVal

    def matchPathconCheck(self, path, value):
        match = 1
        matchPathconCheckCmd  = "/usr/sbin/matchpathcon %s" %path
        (retVal, outputVal) = commands.getstatusoutput(matchPathconCheckCmd)
        if not retVal:
            splitVal = outputVal.split('\t')[1]
            if splitVal != "<<none>>":
                context_list = splitVal.split(":")
                context = context_list[2]
                if context == value:
                    match = 0
        return match


    def createDir(self, dir, user, group):
        if not (os.path.isdir(dir) or os.path.islink(dir)):
            self.logger.info("Creating %s" % dir)
            commands.getstatusoutput("mkdir -p %s" % dir)
            commands.getstatusoutput("chown %s:%s %s" % (user,group,dir))

    def fixDataPermissions(self):

        #This function is obsolete and only used as a fall back while current selinux policy doesn't work.
        #Removed existing policy so that the fall back will work as expected.

        os.system("/opt/collabnet/teamforge/bin/teamforge unload-selinux")
        httpdRWContextDirs = filter(None, [
            self.cfg.get('SUBVERSION_REPOSITORY_BASE'),
            self.cfg.get('CVS_REPOSITORY_BASE'),
            self.cfg.get('SUBVERSION_BRANDING_REPOSITORY_BASE')
        ])

        httpdContextDirs = filter(None, [
            self.cfg.get('OVERRIDES_DIR'),
            self.cfg.get('PUBLISH_DIR'),
            os.path.join(self.cfg['HOME_DIR_BASE'], self.cfg['APP_USER']),
            os.path.join(self.cfg['HOME_DIR_BASE'], self.cfg['HTTPD_USER'])
        ])

        for dir in httpdRWContextDirs:
            self.changeContext('httpd_sys_rw_content_t', dir)

        for dir in httpdContextDirs:
            self.changeContext('httpd_sys_content_t', dir)

        # Resetting to previous context after unload 
        self.doRestorecon(self.cfg['RUNTIME_LOG_DIR'], 'Yes')
        self.doRestorecon(self.cfg['DISTRIBUTION_DIR'], 'Yes')
        self.doRestorecon(self.cfg['TEMPORARY_DIR'], 'Yes')
        
        print app.colors.Color.amber("\nPlease restart the site for these changes to take effect")    
