﻿#!/usr/bin/env python2

""" Contains all the functions that are required for generating random values for Teamforge
    Site Options.
"""
__author__ = 'Moorthy Rajendran <rmoorthy@collab.net>'

import string
import random

class TokenGenerations(object):

   def genDbPassword(self):
       """
       Function for generating a valid database password
       Returns a random password with combination of alphabets(6) + number(4) + splchar(2)
       """
       alphabetsLen = 6
       numbersLen   = 4
       splLen       = 2
       alphabetsChars = string.letters
       numbersChars   = string.digits
       splChars       = '_@!|#%'
       alphabetsCharsLen = len(alphabetsChars) - 1
       numbersCharsLen   = len(numbersChars) - 1
       splCharsLen       = len(splChars) - 1
       randPasswd = ""
       for i in range(0,alphabetsLen):
           rndNum = random.randint(0, alphabetsCharsLen)
           randPasswd += alphabetsChars[rndNum]
       for i in range(0,numbersLen):
           rndNum = random.randint(0, numbersCharsLen)
           randPasswd += numbersChars[rndNum]
       for i in range(0,splLen):
           rndNum = random.randint(0, splCharsLen)
           randPasswd += splChars[rndNum]

       randPasswd = self._shuffleString(randPasswd, 2)
       return randPasswd

   def _shuffleString(self, inStr, fromOffSet=0):

       inList = list(inStr)
       if (fromOffSet == 0):
          random.shuffle(inList)
          return ''.join(inList)

       leadList  = inList[:fromOffSet]
       trialList = inList[fromOffSet:]
       random.shuffle(trialList)

       return ''.join(leadList+trialList)

   def genPassword(self, passwdLen=8):
       """
       Function for generating a random password.
       """
       passwdChars = string.digits + string.letters + string.digits
       totalLength = len(passwdChars)-1;
       randPasswd = ""

       for i in range(0,passwdLen):
           rndNum = random.randint(0, totalLength)
           randPasswd += passwdChars[rndNum]

       return randPasswd

   def genTFPassword(self):
       """
       Function for generating a valid teamforge password
       Returns a random password with combination of alphabets(5) + number(3) + splchar(2)
       """
       alphabetsLen = 5
       numbersLen   = 4
       splLen       = 1
       alphabetsChars = string.letters
       numbersChars   = string.digits
       splChars       = '_@!%'
       alphabetsCharsLen = len(alphabetsChars) - 1
       numbersCharsLen   = len(numbersChars) - 1
       splCharsLen       = len(splChars) - 1
       randPasswd = ""
       for i in range(0,alphabetsLen):
           rndNum = random.randint(0, alphabetsCharsLen)
           randPasswd += alphabetsChars[rndNum]
       for i in range(0,numbersLen):
           rndNum = random.randint(0, numbersCharsLen)
           randPasswd += numbersChars[rndNum]
       for i in range(0,splLen):
           rndNum = random.randint(0, splCharsLen)
           randPasswd += splChars[rndNum]

       randPasswd = self._shuffleString(randPasswd, 2)
       return randPasswd
