﻿
import stat
import os
import new, sys
import traceback
import time
import app.preprocessor
import re
import socket
import commands
import subprocess
import ConfigParser
import hashlib

"""
The utility module, which acts as an interface to most of the modules. All the
utility modules (where ever applicable) should be retrieved via this utility
module interface, so that changing modules will affect this utility module
alone
"""

g_runtimeConfiguration = None

def loadRuntimeConfiguration(deobfuscate = True):
    if deobfuscate:
        getRuntimeConfiguration()
    else:
        # Load and cache the raw (i.e., unobfuscated) configuration
        setRuntimeConfiguration(getRawRuntimeConfiguration())

def setRuntimeConfiguration(conf):
    global g_runtimeConfiguration

    g_runtimeConfiguration = conf

def getRawRuntimeConfiguration():
    """ Get runtime options as they are on disk - which may be obfuscated """

    runtimeOptionsFile = os.path.join(app.defaults.DEFAULT_RUNTIME_DIR, 'conf', 'runtime-options.conf')
    return app.options.RuntimeOptions.readFromFilePath(runtimeOptionsFile)

def getRuntimeConfiguration():
    """ get unobfuscated runtime options"""

    global g_runtimeConfiguration

    if g_runtimeConfiguration is None:
        runtimeOptions = getRawRuntimeConfiguration()
        obfuscator = app.options.Obfuscator.createFromRuntimeOptions(runtimeOptions)
        g_runtimeConfiguration = runtimeOptions.deobfuscate(obfuscator)

    return g_runtimeConfiguration


def executeCommand(command, env=None, useShell=True):
    """
    Executes the command, by running it as a subprocess
    Returns the content of the std out stream if no error occurs else return
    the error stream content and raise a runtime exception
    """
    # create a child process, we want the stderr as a sepearte stream
    process = subprocess.Popen(command, shell=useShell, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env)
    stdout, stderr = process.communicate()
    returncode = process.returncode

    if returncode != 0:
        raise RuntimeError, '%s failed with exit code %d\nStd Err: %s \nStd'\
                            'output: %s' % (command, returncode, stderr, stdout)

    return stdout

def runSystem(cmd, logger, skip_error=0):
    """ Execute a command using os.system()
        Raise exception on failure
    """
    logger.debug('Run -- %s' % stripPassword(cmd))
    exitCode = os.system(cmd)
    if exitCode != 0:
        if not(skip_error):
           logger.fatal('%s failed with exit code %d, %d from os.system()'
                 % (cmd, exitCode >> 8, exitCode))
           raise Exception('%s failed' % cmd)
    logger.debug('Done -- %s' % stripPassword(cmd))

    return exitCode == 0

def runCommand(cmd, logger, skip_error=0):
    """ Execute a command using commands.getstatusoutput()
        Raise exception on failure
    """
    logger.debug('Run -- %s' % stripPassword(cmd))
    (exitCode, output) = commands.getstatusoutput(cmd)
    if output != '':
        logger.info(output)
    if exitCode != 0:
        if not(skip_error):
           logger.fatal('%s failed with exit code %d, %d from commands.getstatusoutput()'
                 % (cmd, exitCode >> 8, exitCode))
           raise Exception('%s failed' % cmd)
    logger.debug('Done -- %s' % stripPassword(cmd))

# deprecated
def examineExecutability(fileName):
    """
    Modelled after the examineExecutability of create-runtime.py.
    Returns the interpreter (if applicable)
    """
    # Check for known file extensions.
    if fileName[-3:] == '.py':
        return getPythonInterpreter(), 0
    elif fileName[-3:] == '.pl':
        return getPerlInterpreter(), 0
    elif fileName[-3:] == '.sh':
        return getShellInterpreter(), 0
    else:
        # No known interpreter.  Get the file mode (in octal) and
        # examine it for executable bits.
        return None, stat.S_IMODE(os.stat(fileName)[stat.ST_MODE]) & 0111

# deprecated
def getPythonInterpreter():
    return "python2"

# deprecated
def getPerlInterpreter():
    return "perl"

# deprecated
def getShellInterpreter():
    return "sh"

def getStackTraceAndException():
    obj, inst, trc = sys.exc_info()
    return traceback.format_exception(obj, inst, trc)


# deprecated
def getOrderedFiles(folderPath):
    """
    Returns all the files in the folderPath (same as in the os sort order)
    """
    command = "find %s -type f | grep -v -e '\.pyo\|c$' | sort" % (folderPath)
    return executeCommand(command).splitlines()

# deprecated
def execPythonCode(scriptFile, moduleName, className, methodName, logger,
                    version):
    """
        Read the contents of the given scriptfile and execute the class in the
        given module. Only python scripts could be executed in this fashion
    """
    scriptContent = open(scriptFile).read()
    sys.modules[moduleName] = mod = new.module(moduleName)
    logger.debug('start -- script %s' % scriptFile)
    mod.__file__ = scriptFile
    exec scriptContent in mod.__dict__
    #FIXME: The method should be able to handle arbitrary python scripts, by
    # invoking the __main__ function
    exec """
from %s import %s
obj = %s(logger, version)
__file__ = '%s'
obj.__file__ = '%s'
mod.__dict__['result'] = obj.%s()
""" % (moduleName, className, className, scriptFile,
        scriptFile, methodName)
    logger.debug("result -- %s" % mod.__dict__['result'])
    logger.debug('end -- script %s' % scriptFile)
    return mod.__dict__['result']

# deprecated
def runScripts(scriptsList, logger, pythonScriptInfo):
    """
        From a list of files, runs them and collects its result.
    """
    if len(scriptsList) == 0:
        logger.info('No scripts found')
        return {}
    resultList = {}
    for scriptFile in scriptsList:
        try:
            # Call the one..
            result = runScriptFile(scriptFile, logger, pythonScriptInfo)
            resultList[scriptFile] = result
        except:
            logger.debug('Error in runScripts')
            raise Exception, ''.join(getStackTraceAndException())
    return resultList

# deprecated
def runAndParseScripts(scriptsList, logger, pythonScriptInfo):
    """
        From a list of files, runs them and collects its result.
    """
    if len(scriptsList) == 0:
        logger.info('No scripts found')
        return {}
    resultList = {}
    tokenProcessor = getTokenProcessor(getRuntimeConfiguration())
    for scriptFile in scriptsList:
        try:
            # Call the one..
            result = runScriptFile(scriptFile, logger, pythonScriptInfo)
            resultList[scriptFile] = tokenProcessor.substitute(result)
        except:
            logger.debug('Error in runScripts')
            raise Exception, ''.join(getStackTraceAndException())
    return resultList


# deprecated
def runScriptFile(scriptFile, logger, pythonScriptInfo = None):
    startTimeStamp = time.time()
    interpreter, executable = examineExecutability(scriptFile)
    result = None
    if interpreter or executable:
        logger.info('Run script %s' % scriptFile)
        if interpreter == getPythonInterpreter():
            if pythonScriptInfo:
                moduleName, className, methodName, pythonLogger, version\
                    = pythonScriptInfo
                # We have a python script, invoke it
                result = execPythonCode(scriptFile, moduleName, className, \
                    methodName, pythonLogger, version)
            else:
                result = executeScript(interpreter, scriptFile, '')
            logger.debug(result)
        elif interpreter is None and executable:
            # This is an executable file, invoke it without any
            # interpreter and options
            result = executeScript('', scriptFile, '')
            logger.debug(result)
        elif interpreter:
            # Pbly a shell script or some other script
            result = executeScript(interpreter, scriptFile, '')
            logger.debug(result)
        else:
            # A txt file?
            logger.debug('Could not figure out the kind of script %s is,'
                    'considering it as text' % scriptFile)
            result = open(scriptFile).read()
            logger.debug(result)
    else:
        logger.info('Use script %s' % scriptFile)
        # A txt file?
        logger.debug('Could not figure out the kind of script %s is,'
                'considering it as text' % scriptFile)
        result = open(scriptFile).read()
        logger.debug(result)
    elapsed = time.time() - startTimeStamp
    logger.info('Completed %s in %f seconds' % (scriptFile, elapsed))
    return result

# deprecated
def executeScript(interpreter, scriptName, options):
    """
        Executes the script using the interpreter as a shell process and
        returns the std output
    """
    command = " ".join([interpreter, scriptName, options])
    return executeCommand(command)

def invokedBySuperUser():
    return isSuperUser()

def isSuperUser():
    return os.getuid() == 0

def getTokenProcessor(cfg):
    return app.preprocessor.TokenProcessor(cfg)

def isLocalhost(hostname):
    if hostname != 'localhost':
        try:
            if (socket.gethostbyname(hostname) == socket.gethostbyname(socket.gethostname())):
                return True
        except:
            # If the name does not resolve, it probably is not localhost.
            pass

    return False

def getPlatform():
    """
    Return the platform string for this environment -- used by
    create-runtime.py and similar scripts which run before the runtime
    options are created, or outside the context in which they exist.
    Most scripts should not use this.
    """
    import platform
    #preparing string as 'ARCH-redhat-OS-VERSION'
    try:
       platform_string = '-'.join([platform.machine(),'redhat',platform.linux_distribution()[1].split('.')[0]])
    except:
       raise Exception, 'Invalid platform'.join(getStackTraceAndException())


    return platform_string

def getJVMPIDs():
    """ Return PIDs of all parent JVM processes
    """
    jvmPIDs = []
    # We assume the failure is because of missing 'cmd' value
    missingCmdPIDs = []
    proc_list = {}
    cfg = getRuntimeConfiguration()
    # Get all the process list (Helps us navigate to the top most parent)
    cmd = 'ps -o pid,ppid,user,comm,args -U %s,%s,%s,%s,%s' % (cfg['APP_USER'],
               cfg['INTEGRATION_USER'], cfg['JAMES_USER'], cfg['ETL_USER'], cfg['ELASTICSEARCH_USER'])
    process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout, stderr = process.communicate()
    stdout_lines = stdout.split('\n')

    # Create a data structure of process details
    for line in stdout_lines:
        try:
            if (line != ''):
                pid, ppid, user, cmd = line.split()[:4]
                command = cmd.split('/')[-1]
                proc_list[pid] = {'pid': pid, 'ppid' : ppid, 'user' : user, 'command' : command}
        except :
            errMsgs = ''.join(getStackTraceAndException())
            missingCmdPIDs.append("ps output line: '%s':\n %s" % (line, errMsgs))
    # Get the java processes started by us
    for processid, value in proc_list.iteritems():
        processcommand = value['command']
        parentprocessid = value['ppid']
        # Get the java processes which are spawned by a non java process owned by 'httpd user' or
        # 'mail user' or spawned by a process which is not owned by 'httpd user' or 'mail user'
        if (processcommand == 'java' and (proc_list.get(parentprocessid) is None or
          proc_list.get(parentprocessid).get('command') != 'java')):
            jvmPIDs.append(processid)

    return (missingCmdPIDs, jvmPIDs)

def stripPassword(cmd):
    return re.sub("--password=.* ", "--password=*** ", cmd)


def setEnvHttpEncoding(cfg=None):
  """Set environment variables to declare a locate that uses
  HTTP_ENCODING (or UTF-8 if there's no configured HTTP_ENCODING).
  Match with a call to restoreEnvEncoding(value_returned_from_here)."""

  if cfg is None:
     cfg = getRuntimeConfiguration()

  original_lc_all = os.environ.get('LC_ALL', None)
  os.environ['LC_ALL'] = "en_US.%s" % cfg.get('HTTP_ENCODING', 'UTF-8')

  return original_lc_all


def restoreEnvEncoding(encoding):
  """Undo what setHttpEncoding done did."""
  if encoding:
      os.environ['LC_ALL'] = encoding
  else:
      del(os.environ['LC_ALL'])


def readFromUser(prompt, valid_values = None):
    """Get input from the user and pretty printing"""
    ret_val = ''

    while ret_val == '':
        ret_val =  raw_input(prompt)

        if valid_values and not ret_val in valid_values:
            print
            print '"%s" is not a valid response; valid values are: ' % ret_val
            print ', '.join(valid_values)
            print
            ret_val = ''

    print
    return ret_val

def configParser():
    return ConfigParser.ConfigParser()

def getCheckSum(confFile):
    checkSum = None
    if os.path.isfile(confFile):
        checkSum = hashlib.md5(open(confFile).read()).hexdigest()
    return checkSum

def isAnIpAddress(ipAddress):
    ipTemplateMatch = re.match(r'^\d+\.\d+\.\d+\.\d+$', ipAddress)
    if ipTemplateMatch:
        return [0 <= int(x) < 256 for x in re.split('\.', ipTemplateMatch.group(0))].count(True) == 4
    return False

def shallowwalk(path, depth):
    """ Like os.walk, but doesn't recurse deeper than DEPTH. """
    path = path.rstrip(os.path.sep)
    assert os.path.isdir(path)
    start_depth = path.count(os.path.sep)

    if depth == 0:
        yield (path, [], [])
        raise StopIteration
    for root, dirs, files in os.walk(path):
        current_depth = root.count(os.path.sep)
        if current_depth >= start_depth + int(depth):
            del dirs[:]
        yield (root, dirs, files)

def byteify(input):
    if isinstance(input, dict):
        return dict((byteify(key), byteify(value)) for key, value in input.iteritems())
    elif isinstance(input, list):
        return [byteify(element) for element in input]
    elif isinstance(input, unicode):
        return input.encode('utf-8')
    else:
        return input
