﻿import time

import app.landscape
import service.constants

#
# DEPRECATED - ONLY KEPT FOR COMPATIBILITY
#

legacyServicesToNewServices = {
    service.constants.SERVICE_REPORTS_ORACLE: ["ctfcore-datamart-oracle"],
    service.constants.SERVICE_REPORTS_PGSQL: ["ctfcore-datamart-postgres", "ctfcore-datamart-postgres-cohosted"],
    service.constants.SERVICE_TOMCAT: ["integration"],
    service.constants.SERVICE_PGSQL: ["ctfcore-database-postgres"]
    }

def __initialize():
    if app.landscape.Landscape.isInitialized():
        return

    # Initialize using runtime-options.conf (instead of site-options.conf) for compatibility
    app.landscape.Landscape.initializeFromSiteOptionsPath(app.defaults.RUNTIME_OPTIONS_CONF)

def getApplicationHosts(applicationName):
    __initialize()

    service = app.landscape.LegacyHostTokenParser.appsToServicesMapping[applicationName]
    return ["localhost" if m.isLocal() else m.getPhysicalName()
            for m in app.landscape.Landscape.getSite().getMachinesByService(service)]

def getApplicationHost(applicationName):
    __initialize()

    service = app.landscape.LegacyHostTokenParser.appsToServicesMapping[applicationName]
    machines = app.landscape.Landscape.getSite().getMachinesByService(service)

    if len(machines) > 1 and not all(m.isClustered() for m in machines):
        raise TooManyHostsConfigured(applicationName, [m.getPhysicalName() for m in machines])
    elif len(machines) > 0:
        if machines[0].isLocal():
            return "localhost"
        else:
            return machines[0].getPhysicalName()
    else:
        return None

class TooManyHostsConfigured(Exception):
    def __init__(self, application, hosts):
        self.application = application
        self.hosts = hosts

    def __str__(self):
        return "Too many hosts (%s) defined for application '%s'" % \
            (",".join(self.hosts), self.application)

def getLocalNodeHostName():
    __initialize()

    # Used to return NODE_HOSTNAME, but that is always localhost when HOST_localhost is used.
    return "localhost"

def isApplicationEnabled(applicationName, hostName):
    __initialize()

    if not hostName or hostName == "None":
        return False

    service = app.landscape.LegacyHostTokenParser.appsToServicesMapping[applicationName]
    machine = app.landscape.Landscape.getSite().getMachine(hostName)
    if not machine:
        raise Exception("Unknown machine '%s'" % hostName)

    return machine.getService(service) is not None

def isApplicationEnabledLocally(applicationName):
    __initialize()

    service = app.landscape.LegacyHostTokenParser.appsToServicesMapping[applicationName]
    return app.landscape.Landscape.getSite().getLocalMachine().getService(service) is not None

def __isOneOfServicesEnabledOnMachine(machine, serviceNames):
    __initialize()

    for serviceName in serviceNames:
        if machine.getService(serviceName) is not None:
            return True

    return False

def isServiceEnabled(legacyService, hostName):
    __initialize()

    if not hostName or hostName == "None":
        return False

    if hostName == "localhost":
        hostName = app.landscape.Landscape.getSite().getLocalMachine().getPhysicalName()

    machine = app.landscape.Landscape.getSite().getMachine(hostName)
    if not machine:
        raise Exception("Unknown machine '%s'" % hostName)

    return __isOneOfServicesEnabledOnMachine(
        machine,
        legacyServicesToNewServices[legacyService])

def isServiceEnabledLocally(legacyService):
    __initialize()

    return __isOneOfServicesEnabledOnMachine(
        app.landscape.Landscape.getSite().getLocalMachine(),
        legacyServicesToNewServices[legacyService])

def isServiceRunning(serviceName):
    __initialize()
    service = app.landscape.Landscape.getSite().getLocalMachine().getService(serviceName)
    if not service:
        return False

    # This method is sometimes called in non-root context. Calling getStatus()
    # does not work then, so we can only do a shallow check here.
    return service.getController(app.logutil.getNullLogger()).isHealthy()

def getHostDomain(hostName):
    __initialize()

    return app.landscape.Landscape.getSite().getMachine(hostName).getPublicAddress().getName()

def waitForServerUp(hostName, port, timeout=30):
    import socket
    connected = False;
    for icount in range (0, timeout):
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect((hostName, int(port)))
            s.close()
        except socket.error:
            time.sleep(1)
            continue
        else:
            connected = True
            break
    return connected

def waitForServerDown(hostName, port, timeout=30):
    import socket
    disConnected = False;
    for icount in range (0, timeout):
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect((hostName, int(port)))
            s.close()
        except socket.error:
            disConnected = True
            break
        else:
            time.sleep(1)
            continue
    return disConnected
