
from app.scripts import BaseConfigScript
import service.util
import service.constants

class ConfigScript(BaseConfigScript):
    def configSnippet(self):

        rewriteStr = ""

        if "redhat-7" in self.cfg["PLATFORM"]:
	    filterProvider = "gzip DEFLATE \"(%{HTTP:Accept-Encoding} =~ /gzip/) && (%{CONTENT_TYPE} !~ /gzip/)\""
            allowString = "Require all granted"
            denyString = "Require all denied"
        else:
            filterProvider = "gzip DEFLATE Content-Type !$gzip"
            allowString = """
    Order allow,deny
    Allow from all
"""
            denyString = """
    Order allow,deny
    Deny from all
"""
        confStr = """
<IfModule !mod_filter.c>
   LoadModule filter_module modules/mod_filter.so
</IfModule>

# Enabling mod_expires and mod_deflate for performance enhancement
# Turn on mod_deflate
DeflateFilterNote Input instream
DeflateFilterNote Output outstream
DeflateFilterNote Ratio ratio
#CustomLog logs/deflate_log deflate
CustomLog {__HTTPD_LOG_DIR__}/deflate_log deflate

## Except for svn, enable mod_deflate for rest of the namespaces
<Location ~ "^/(?!(svn))">

   # Add the deflate filter only if the content type is not of type gzip already
   FilterDeclare gzip CONTENT_SET
   FilterProvider %s
   FilterChain gzip

   SetEnvIfNoCase Request_URI \.(?:gif|jpe?g|png|swf)$ no-gzip dont-vary
   SetEnvIfNoCase Request_URI \.(?:exe|t?gz|zip|bz2|sit|rar)$ no-gzip dont-vary
   SetEnvIfNoCase Request_URI \.pdf$ no-gzip dont-vary
   SetEnvIfNoCase Remote_Addr 127.0.0.1 no-gzip dont-vary
   Header append Vary User-Agent env=!dont-vary
</Location>

<Location ~ "/ctf/">
   ExpiresByType text/html "access plus 30 days"
</Location>

# mod_expires configuration
ExpiresActive On
ExpiresDefault "access plus 0 seconds"
ExpiresByType image/gif "access plus 1 days"
ExpiresByType image/jpeg "access plus 1 days"
ExpiresByType image/png "access plus 1 days"
ExpiresByType image/svg+xml "access plus 1 days"
ExpiresByType application/font-woff "access plus 1 days"
ExpiresByType font/eot "access plus 1 days"
ExpiresByType font/opentype "access plus 1 days"
ExpiresByType font/truetype "access plus 1 days"
ExpiresByType text/css "access plus 30 days"
ExpiresByType application/javascript "access plus 30 days"
ExpiresByType text/javascript "access plus 30 days"
ExpiresByType application/x-javascript "access plus 30 days"
ExpiresByType image/x-icon "access plus 1 days"
ExpiresByType application/font-woff2 "access plus 1 days"

# end mod_expires and mod_deflate settings.
""" %(filterProvider)

        if service.util.isApplicationEnabledLocally(service.constants.APPLICATION_APP):
            rewriteStr = """
# Return 503 for all svn and soap requests from external interface until
# all web services are up completely in JBoss

RewriteCond %{REQUEST_URI} ^/(svn|ce-soap|sf-soap|ce-soap60)
RewriteCond %{ENV:REDIRECT_STATUS} !=503
RewriteCond {__RUNTIME_DIR__}/temp/jboss.start !-f
RewriteRule ^(.*)$ /503.html?ws-starting [R=503,L]

"""

        rewriteStr += """

RewriteRule ^/help/(.*) http://help.collab.net/$1 [R,L]

## Javascript
RewriteCond {__DATA_DIR__}/overrides/js/$1 -f
RewriteRule ^/sf/js/(.*)$ /sf-js-overrides/$1 [PT,L]

##robots.txt redirecting
RewriteCond {__DATA_DIR__}/overrides/robots.txt -f
RewriteRule ^/robots.txt$ /sf-robots/overrides/robots.txt [PT,L]

RewriteRule ^/robots.txt$ /sf-robots/robots.txt [PT,L]
"""
        if self.cfg['DISABLE_REMOTE_PUBLISHING'] == 'false':
            rewriteStr += """

##RemotePublishing
RewriteRule ^/sf/projects/([^/]*)$ /sf/projects/$1/ [R=permanent,L]
RewriteRule ^/sf/projects/([^/]*)/(.*).(gif|jpe?g|png|bmp|css|js|pdf|ppt|doc|xls|pps|htm|swf)$ /sf-remote/$1-publishing/www/$2.$3 [NC,PT]

"""
        rewriteStr += """
# CORS headers to handle cross domain request
SetEnvIf Request_URI "^(\/ctf\/js\/saturn\/.*|\/ctf\/js\/modules\/.*|\/sf-images\/fonts\/.*|\/sf\/header.html|\/css\/fonts\/.*)" AllowCORSForGET

Header always set Access-Control-Allow-Origin * env=AllowCORSForGET
Header always set Access-Control-Allow-Methods "GET" env=AllowCORSForGET
Header always set Access-Control-Allow-Headers "x-requested-with, Content-Type, origin, authorization, accept, client-security-token" env=AllowCORSForGET

# Remove X-Powered-By
Header unset X-Powered-By

# For IE-compatibility
Header set X-UA-Compatible "IE=Edge"

# Protection against cross-site scripting
Header always set X-XSS-PROTECTION "1; mode=block"

# HighCharts Exporter App
RewriteRule ^/highcharts-export-web/(.*) http://localhost:8080/highcharts-export-web/$1 [P]
RewriteRule ^/highcharts-export-web$ http://localhost:8080/highcharts-export-web [P]

# Image Overrides
RewriteCond {__DATA_DIR__}/overrides/images/$1 -f
RewriteRule ^/sf-images/(.*)$ /sf-images-overrides/$1 [PT,L]

# CSS Overrides
RewriteCond {__DATA_DIR__}/overrides/css/$1 -f
RewriteRule ^/css/(.*)$ /sf-css-overrides/$1 [PT,L]

Alias /sf-images-overrides/ {__DATA_DIR__}/overrides/images/

# Added to Suppress HTTP Trace and Track for Security
RewriteCond %{REQUEST_METHOD} ^TRACK
RewriteRule .* - [F]
RewriteCond %{REQUEST_METHOD} ^TRACE
RewriteRule .* - [F]
"""

        redirectStr = """
#Throw 404 message on accessing /icons folders for Security
RedirectMatch 404 ^/icons(/|$)
#Throw 404 message for /cgi-bin and anything on down as long as it does't exist for Security
RedirectMatch 404 ^/cgi-bin(/?)$

<Location /sf-images-overrides/ >
    Options None
    # AllowOverride None
    %s
</Location>

Alias /sf-css-overrides/ {__DATA_DIR__}/overrides/css/

<Location /sf-css-overrides/ >
    Options None
    # AllowOverride None
    %s
</Location>

Alias /sf-js-overrides/ {__DATA_DIR__}/overrides/js/

<Location /sf-js-overrides/ >
    Options None
    # AllowOverride None
    %s
</Location>

<Location ~ /.*/\.svn(/|$) >
    %s
</Location>
""" %(allowString, allowString, allowString, denyString)

        if self.cfg.has_key('TEST_SUPPORT_ENABLED') and self.cfg['TEST_SUPPORT_ENABLED'] == 'true':
            listDocRewriteStr = ""
        else:
            listDocRewriteStr = """
# List Documents Redirection
RewriteRule ^/sf/docman/do/listDocuments/(.*) /ctf/documents/list/$1 [R=permanent,L]
"""
        return confStr + rewriteStr + redirectStr + listDocRewriteStr
