﻿
from app.scripts import BaseConfigScript
import app.landscape

class ConfigScript(BaseConfigScript):
    def __getServerAliasLine(self):
        service = app.landscape.Landscape.getSite().getLocalMachine().getService("apache")
        aliases = set([targetEndpoint.getHost(False) for
                   targetEndpoint in service.getProvidedEndpoint("http").getViaTargetEndpoints()])

        if aliases:
            return "ServerAlias %s" % " ".join(aliases)
        else:
            return "# No aliases defined for this machine"

    def configSnippet(self):
        domain=self.cfg['apache/cname/http/host']

        rules = ''
        additionalRedirectRule = ''
        catchAllRedirectRule = ''
        scheme = self.cfg['HTTP_SCHEME']
        if scheme == 'https':
            if self.cfg['NODE_HOSTNAME'] != domain:
                additionalRedirectRule = """
    RewriteCond %s ^%s
    RewriteRule /(.*) %s://%s/$1 [R=permanent,L]
""" %('%{HTTP_HOST}', self.cfg['NODE_HOSTNAME'], scheme, domain)

            catchAllRedirectRule = """
    RewriteRule /(.*) %s://%s/$1 [R=permanent,L]
""" %(scheme, domain)

        confStr = """
<VirtualHost *:80>
    %s
    RewriteEngine on
    RewriteOptions inherit

    # This is needed for Gerrit compatibility, but does not hurt other services.
    AllowEncodedSlashes On

    # Set request header for url protocol
    RequestHeader set X_FORWARDED_PROTO 'http'

    RewriteRule /server-status - [R=permanent,L]
    %s
    %s
</VirtualHost>
""" % (self.__getServerAliasLine(), additionalRedirectRule, catchAllRedirectRule)
        return confStr

