
from app.scripts import BaseConfigScript

class ConfigScript(BaseConfigScript):

    def configSnippet(self):
        if "redhat-7" in self.cfg["PLATFORM"]:
            sslMutex = "Mutex"
            sslMutexValue = "posixsem"
        else:
            sslMutex = "SSLMutex"
            sslMutexValue = "default"
        initialValue1 = """

LoadModule ssl_module modules/mod_ssl.so
Listen 443

#
#   Some MIME-types for downloading Certificates and CRLs
#
AddType application/x-x509-ca-cert .crt
AddType application/x-pkcs7-crl    .crl

#   Pass Phrase Dialog:
#   Configure the pass phrase gathering process.
#   The filtering dialog program (`builtin' is a internal
#   terminal dialog) has to provide the pass phrase on stdout.
SSLPassPhraseDialog  builtin

#   Inter-Process Session Cache
SSLSessionCache         shmcb:/var/cache/mod_ssl/scache(512000)
SSLSessionCacheTimeout  300

%s %s
""" %(sslMutex,sslMutexValue)
        initialValue2 = """
SSLRandomSeed startup file:/dev/urandom  256
SSLRandomSeed connect builtin
SSLCryptoDevice builtin

<VirtualHost *:443>

	ErrorLog {__HTTPD_LOG_DIR__}/ssl_error_log
	CustomLog {__HTTPD_LOG_DIR__}/ssl_access_log combined
	CustomLog {__HTTPD_SVN_LOG__} "%t %u %{SVN-REPOS-NAME}e %{SVN-ACTION}e %T/%D" env=SVN-ACTION
	LogLevel warn

	RewriteEngine on
	RewriteOptions inherit

    # This is needed for Gerrit compatibility, but does not hurt other services.
    AllowEncodedSlashes On

	# Set request header for url protocol
	RequestHeader set X_FORWARDED_PROTO 'https'

	# SSL Engine Switch:
	#   Enable/Disable SSL for this virtual host.
	SSLEngine on
"""
        initialValue3 = """
	# SSL Methods:
	#   Included to disable some of the less-secure methods
	SSLProtocol {__SSL_PROTOCOL__}
"""

        initialValue4 = """
	#   SSL Cipher Suite:
	#   List the ciphers that the client is permitted to negotiate.
	#   See the mod_ssl documentation for a complete list.
	#
	#   Modified to disabled some of the less-secure methods

        SSLCipherSuite {__SSL_CIPHER_SUITE__}
        SSLHonorCipherOrder     on

	SSLCertificateFile {__SSL_CERT_FILE__}
	SSLCertificateKeyFile {__SSL_KEY_FILE__}
"""
        endString = "\n</VirtualHost>\n\n"
        if self.cfg.has_key("SSL") and self.cfg["SSL"] == 'on' :
           extraSslTokens = ""
           if self.cfg.has_key("SSL_CA_CERT_FILE"):
              extraSslTokens = "\tSSLCACertificateFile " + self.cfg["SSL_CA_CERT_FILE"] + "\n"
           if self.cfg.has_key("SSL_CHAIN_FILE"):
              extraSslTokens = extraSslTokens + "\tSSLCertificateChainFile " + self.cfg["SSL_CHAIN_FILE"]
           return initialValue1 + initialValue2 + initialValue3 + initialValue4 + extraSslTokens + endString
        return initialValue1 + initialValue2 + initialValue3 + initialValue4 + endString
