
from app.scripts import BaseCreateRuntimeScript
import app.util
import app.batch
import app.landscape
import os
import shutil
from app.colors import Color

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Setup Apache main configuration"

    def createRuntime(self):
        distributionDir = self.cfg['DISTRIBUTION_DIR']

        # Create httpd.conf file
        batch = app.batch.ConfigBatch(self.log) \
            .addFolder(os.path.join(app.defaults.DEFAULT_SERVICE_DIR, 'apache', 'conf-snippets', 'httpd.d'))

        if "redhat-7" in self.cfg["PLATFORM"]:
            batch.removeFile('00-httpd22_15.conf')
        else:
            batch.removeFile('00-httpd24.conf')

        if self.cfg['SSL'] != "on":
            batch.removeFile('04-httpd-ssl.py')
            batch.removeFile('09-httpd-hsts.conf')
        else:
            batch.removeFile('01-httpd-conf-custom-log.conf')

        # Add overrides
        for service in app.landscape.Landscape.getSite().getLocalMachine().getServices():
            overridesFolder = os.path.join(
                app.defaults.DEFAULT_SERVICE_DIR,
                service.getName(),
                'overrides',
                'apache',
                'httpd.conf.d')

            if os.path.isdir(overridesFolder):
                batch = batch.addFolder(overridesFolder)

        # User overrides
        httpdIncludeDir = os.path.join(self.cfg['ETC_DIR'], 'conf.d','httpd','httpd.conf.d')
        if os.path.isdir(httpdIncludeDir) :
            self.logMesg("warn", Color.amber("\nCustom configuration found in %s has been applied." % httpdIncludeDir ))
            self.logMesg("warn", Color.amber("Please be aware of that such configuration may impact the reliability of TeamForge.\n"))
        else:
            batch.removeFile('07-httpd-include-custom.conf')

        httpdConfFilename = self.cfg["HTTPD_CONF_FILE_PATH"]
        if not(os.path.isfile(httpdConfFilename + ".cn_backup")):
            shutil.copyfile(httpdConfFilename, httpdConfFilename + ".cn_backup")

        batch.evaluateIntoFile(httpdConfFilename)

        # Create integratedapps.conf
        integratedAppConf = os.path.join(self.cfg['HTTPD_CONF_D_DIR'], 'integratedapps.conf')
        os.system("touch %s" %integratedAppConf)
