﻿
from app.scripts import BaseCreateRuntimeScript
import os
import app.util
import app.certstore
import app.defaults

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Add SSL certificates to local trust store"

    def __isSslEnabled(self):
        return self.cfg.get("SSL", "off") == "on"

    def createRuntime(self):
        if not self.__isSslEnabled():
            return

        domainName = app.landscape.Landscape.getSite().getLocalMachine().getPublicAddress().getName()

        chainFile = os.path.join(app.defaults.DEFAULT_SSLCERTS_PATH, "intermediate.crt")
        certFile = os.path.join(app.defaults.DEFAULT_SSLCERTS_PATH, domainName + ".crt")

        if chainFile and os.path.isfile(chainFile):
            # Add chain file to the trust stores. This ensures that
            # local services (and tools) are able to verify the SSL
            # certificate if they make calls to Apache.
            self.log.info("Adding intermediate certificate %s to trust stores" % chainFile)
            app.certstore.JavaTrustStore().importCertificateChainFile("ctf-local-intermediate", chainFile)
            app.certstore.LinuxTrustStore().importCertificateChainFile("ctf-local-intermediate", chainFile)

        elif certFile and os.path.isfile(certFile):
            # If there is no chain file, then we are apparently using
            # a certificate that is either...
            #  (1) trusted by a CA which is already trusted by Linux and Java (great),
            #  (2) trusted by some corporate CA which is not trusted by Linux and Java (bad), or
            #  (3) self-signed (ugh!).
            #
            # Cases (2) and (3) are problematic because any local services (and tools) will
            # not be able to verify the SSL certificate
            #
            # Adding the SSL certificate to the trust store solves (2) and (3) and does not harm
            # case (1).
            self.log.info("Adding certificate %s to trust stores" % certFile)
            app.certstore.JavaTrustStore().importCertificateChainFile("ctf-local", certFile)
            app.certstore.LinuxTrustStore().importCertificateChainFile("ctf-local", certFile)
