﻿import os
import signal
import time
import app.servicecontrol
import app.linuxservice
import urllib2
import sys

START_TIMEOUT = 20

class ServiceController(app.servicecontrol.ServiceController):
    def __init__(self, service, logger):
        super(ServiceController, self).__init__(service, logger)

        self.__lastStartupTime = 0

    def getPidFile(self):
        for pidLocation in ["/var/run/httpd.pid", "/var/run/httpd/httpd.pid"]:
            if os.path.isfile(pidLocation):
                return pidLocation

        return None

    def getKillSignal(self):
        # Apache leave some child processes, so send TERM signal to the parent causes it to
        # attempt to kill off all its children immediately.
        return signal.SIGTERM

    def canBootstrap(self):
        # Bootstrapping Apache is a no-op, so allow it always.
        return True

    def isStarting(self):
        # Apache does not immediately publish its PID file - but that does not mean
        # it is in trouble. So we need to give it some extra time before making any status
        # decisions.
        return not self.getPid() and (time.time() - self.__lastStartupTime) < START_TIMEOUT

    def __runHttpd(self, verb):
        return app.util.runSystem("httpd -k %s &> /dev/null" % verb, self.logger, skip_error=True)

    def startDaemonAsync(self):
        if app.linuxservice.ServiceManager.isAvailable():
            result = app.linuxservice.ServiceManager.getService("httpd", self.logger).start()
        else:
            result = self.__runHttpd("start")

        if result:
            self.__lastStartupTime = time.time()
            return True
        else:
            return False

    def stopDaemonAsync(self):
        if app.linuxservice.ServiceManager.isAvailable():
            return app.linuxservice.ServiceManager.getService("httpd", self.logger).stop()
        else:
            return self.__runHttpd("stop")

    def createPostDeployAction(self):
        # Start the service right after deployment
        return RestartApacheAction(self.getService(), self.logger)

    def printSnapshot(self, label):

        cfg = app.util.getRuntimeConfiguration()
        port = cfg["apache/bind/http/port"]
        httpScheme = cfg["HTTP_SCHEME"]
        url = '%s://127.0.0.1:%s/server-status' % (httpScheme, port)
        print "----------------------------------------"
        print "Start of Apache Snapshot"
        print "----------------------------------------"
        try:
            c = urllib2.urlopen(url)
            sys.stdout.write(c.read())
        except Exception as e:
            print "Problem in getting Apache snaphshot.%s"%e
        print "----------------------------------------"
        print "End of Apache Snapshot"
        print "----------------------------------------"

    def getLogPaths(self):
        return [os.path.join(app.util.getRuntimeConfiguration().get("HTTPD_LOG_DIR"), "*")]

class RestartApacheAction(app.servicecontrol.ControllerAction):
    def __init__(self, service, logger):
        self.__service = service
        super(RestartApacheAction, self).__init__(logger)

    def getInvolvedServices(self):
        return [self.__service]

    def confirm(self, isRedeployment=False):
        return True

    def execute(self):
        import app.actions.stop
        import app.actions.start

        return app.actions.stop.StopServiceAction([self.__service], self.logger).execute() and \
               app.actions.start.StartServiceAction([self.__service], self.logger).execute()
