﻿import os
import app.servicecontrol
import app.defaults
import app.util
import time

START_TIMEOUT = 30

class ServiceController(app.servicecontrol.ServiceController):
    def __init__(self, service, logger):
        super(ServiceController, self).__init__(service, logger)
        self.__lastStartupTime = 0

    def getPidFile(self):
        return "/opt/collabnet/integrations/cn-binary/binary.pid"

    def __runInit(self, verb):
        config = app.util.getRuntimeConfiguration()

        logFile = os.path.join(config['BINARY_LOG_DIR'], 'service.log')
        initScript = config["BINARY_SERVICE_CMD"]

        return os.system("%s %s < /dev/null >> %s 2>&1 &" % (initScript, verb, logFile)) == 0

    def isStarting(self):
        # Binary does not immediately publish its PID file - but that does not mean
        # it is in trouble. So we need to give it some extra time before making any status
        # decisions.
        return not self.getPid() and (time.time() - self.__lastStartupTime) < START_TIMEOUT

    def startDaemonAsync(self):
        if self.__runInit("start"):
            self.__lastStartupTime = time.time()
            return True
        else:
            return False

    def stopDaemonAsync(self):
        return self.__runInit("stop")
    
    def getLogPaths(self):
        return [os.path.join(app.util.getRuntimeConfiguration().get("BINARY_LOG_DIR"), "*")]
