#!/opt/collabnet/teamforge/runtime/bin/wrapper/python

import getopt
import os
import sys
import time

import app.util
import app.batch
import app.logutil
import app.landscape
import logging

def printLog(logger, logType, logMsg, lineFeed=True):
    """ Prints program output messages both to logfile and tty """

    _HEADER = '\033[95m'
    _OKBLUE = '\033[94m'
    _OKGREEN = '\033[92m'
    _WARNING = '\033[93m'
    _FAIL = '\033[91m'
    _INFO = '\033[0m'
    _ENDC = '\033[0m'

    if logType == "warn":
        display(_WARNING + logMsg + _ENDC, lineFeed)
        logger.warn(logMsg)
    elif logType == "error":
        display(_FAIL + logMsg + _ENDC, lineFeed)
        logger.error(logMsg)
    elif logType == "success":
        display(_OKGREEN + logMsg + _ENDC, lineFeed)
        logger.info(logMsg)
    elif logType == "header":
        display(_HEADER + logMsg + _ENDC, lineFeed)
        logger.info(logMsg)
    elif logType == "title":
        display(_OKBLUE + logMsg + _ENDC, lineFeed)
        logger.info(logMsg)
    elif logType == "notify" or logType == "info":
        display(logMsg, lineFeed)
        logger.info(logMsg)
    elif logType == "debug":
        display(logMsg, lineFeed)
        logger.debug(logMsg)
    else:
        display(logMsg)
        logger.info(logMsg, lineFeed)

def display(logMsg, lineFeed=True):

    sys.stdout.write(logMsg)
    if (lineFeed):
       sys.stdout.write("\n")
    sys.stdout.flush()

def printUsageAndExit(error=None, exit_code=0):
    print
    print "This script run cron jobs "
    print
    print "Valid options: "
    print "    -h | --help             print this usage message and exit"
    print "    -V | --verbose          print lots of debug message"
    print "    -e | --execute          Execute a cron job.Supported cron jobs are 5mins,hourly,daily,weekly and monthly"
    print "    -q | --quiet            do not show script output"
    print
    if error:
        print >> sys.stderr, error

    sys.exit(exit_code)

def main():
    verbose = False
    quiet = False
    cronJob = None
    cronJobs = [ "5mins","hourly","daily","weekly","monthly" ]

    try:
        opts, args = getopt.getopt(sys.argv[1:], "hqe:VF",
                                   ["help", "execute=",
                                    "quiet", "verbose"
                                    ])
    except getopt.GetoptError, exc:
        printUsageAndExit("Getopt Error:%s\n" % exc, 1)
    for option, value in opts:
        if option in ("-h", "--help"):
            printUsageAndExit()
        elif option in ('-e', "--execute"):
            cronJob = value
        elif option in ('-V', "--verbose"):
            verbose = True
        elif option in ('-q', "--quiet"):
            quiet = True

    if not cronJob:
        printUsageAndExit("Cron job (-e) is mandatory\n", 1)

    if not cronJob in cronJobs:
        printUsageAndExit("Please provide valid cron job\n", 1)

    conf = app.util.getRuntimeConfiguration()
    runtimeLogDir = conf["RUNTIME_DIR"]
    if verbose:
        logger = app.logutil.getCronJobLogger(runtimeLogDir, True)
        logger.setLevel(logging.DEBUG)
    else:
        logger = app.logutil.getCronJobLogger(runtimeLogDir, False)
        logger.setLevel(20)

    if quiet:
        sys.stdout = open ('/dev/null', 'w')
        logger.setLevel(logging.ERROR)
    printLog(logger, "header", "TeamForge Cron Process\n".center(78))

    printLog(logger, "title", "Cron");

    batch = app.batch.CronBatch(logger, app.util.getRuntimeConfiguration())

    # Load landscape information from runtime-options.conf
    app.landscape.Landscape.initializeFromSiteOptionsPath(app.defaults.RUNTIME_OPTIONS_CONF)

    for serviceName in app.landscape.Landscape.getSite().getLocalMachine().getServiceNames():
        scriptsFolder = os.path.join(
            app.defaults.DEFAULT_SERVICE_DIR,
            serviceName,
            "hooks",
            "cron",
            cronJob)

        if os.path.isdir(scriptsFolder):
            batch.addFolder(scriptsFolder)

    startTimeStamp = time.time()
    batch.execute()
    elapsed = time.time() - startTimeStamp
    elapsed = "{0:.2f}".format(elapsed)
    printLog(logger, "info", " ")
    printLog(logger, "title", "Cron Process Summary")
    printLog(logger, "info", "  No. of Snippets executed".ljust(50) + " : ", False)
    printLog(logger, "info", "%s" % batch.size())
    printLog(logger, "info", "  Total time taken".ljust(50) + " : ", False)
    printLog(logger, "info", "%s seconds" % elapsed)
    printLog(logger, "info", " ")


######################################################################
##  Entrance hook for when this is used as a script                 ##
######################################################################

if __name__ == "__main__":
    main()

