﻿from app.selinuxutil import Policy
from app.scripts import BaseCreateRuntimeScript
import app.util
import app.landscape
import os
import fileinput

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Copy Teamforge EAR and WARs to JBoss deployment folder"

    def makeDistributable(self, xmlPath):
        searchToken = '<!-- <distributable/> -->'
        replaceToken = '<distributable/>'
        webXml = fileinput.FileInput(xmlPath, inplace=True, backup='.bak')
        for line in webXml:
            print line.replace(searchToken, replaceToken),
        webXml.close()

    def createRuntime(self):
        earName = 'teamforge.ear'
        jbossDeployDir = self.cfg['JBOSS_SERVER_DEPLOY_DIR']
        jbossDeployEar = os.path.join(self.cfg['DEPLOY_FILES_DIR'], earName)

        # teamforge.ear
        cmdStr = "unzip -qo %s -d %s" % (jbossDeployEar, os.path.join(jbossDeployDir,earName))
        app.util.runSystem(cmdStr, self.log)
        app.util.runSystem("touch %s" % os.path.join(jbossDeployDir, earName + ".dodeploy"), self.log)

        if self.cfg.has_key('TEST_SUPPORT_ENABLED') and self.cfg['TEST_SUPPORT_ENABLED'] == 'true':
            jbossDeployTestEar = os.path.join(self.cfg['DEPLOY_FILES_DIR'], 'teamforge.test.ear')
            cmdStr = "unzip -qo %s -d %s" % (jbossDeployTestEar, os.path.join(jbossDeployDir,earName))
            app.util.runSystem(cmdStr, self.log)

        # integration.war
        jbossDeployIntegrationWar = os.path.join(self.cfg['DEPLOY_FILES_DIR'], 'integration.war')
        cmdStr = "cp -rf %s %s/" % (jbossDeployIntegrationWar, jbossDeployDir)
        app.util.runSystem(cmdStr, self.log)

        if app.landscape.Landscape.getSite().getLocalMachine().isClustered():
            self.makeDistributable(os.path.join(jbossDeployDir,earName , 'sf.war/WEB-INF/web.xml' ))

        # Reapply ownership
        app.util.runSystem("chown -R %s:%s %s" % (
            self.cfg['APP_USER'],
            self.cfg['APP_GROUP'],
            os.path.join(jbossDeployDir, earName)), self.log)

        jbossStartScript = os.path.join(self.cfg['JBOSS_HOME'], "bin", "run.sh")
        jbossStopScript = os.path.join(self.cfg['JBOSS_HOME'], "bin", "jboss-cli.sh")

        if not Policy.isSELinuxPermissionsApplied(self.cfg['JBOSS_HOME']):
            Policy.applySELinuxPermissions(self.cfg['JBOSS_HOME'])
        if not Policy.isSELinuxPermissionsApplied(jbossStartScript):
            Policy.applySELinuxPermissions(jbossStartScript)
        if not Policy.isSELinuxPermissionsApplied(jbossStopScript):
            Policy.applySELinuxPermissions(jbossStopScript)


