﻿
from app.scripts import BaseInitializationScript
import os
import socket
import app.util
import app.landscape
import app.certstore
from app.colors import Color

class InitializationScript(BaseInitializationScript):

    scriptDescription = "Import SSL certificates of servers"

    def __trustHost(self, host, port, title):
        try:
            app.certstore.LinuxTrustStore().importCertificateFromServer("%s-%s" % (host, title), host, port)
            app.certstore.JavaTrustStore().importCertificateFromServer(host, port)
        except Exception as e:
            self.logMesg("warn", Color.amber(
               "\n  Importing the SSL certificates of %s server failed as the respective services may not be fully up yet." %title +
               "\n  You can run /opt/collabnet/teamforge/bin/teamforge reinitialize to retry the operation." +
               "\n  (%s)" % str(e)))
            return

    def __trustService(self, serviceName, endpointId, scope, title):
        for server in app.landscape.Landscape.getSite().getMachinesByService(serviceName):
            endpoint = server.getService(serviceName).getProvidedEndpoint(endpointId, scope)

            # Import certificate. This is necessary even if the service is local since there
            # may be a SSL-terminating load balancer in between.
            self.log.info("Importing %s SSL certificate from %s:%d" % (title, endpoint.getHost(False), endpoint.getPort()))
            self.__trustHost(endpoint.getHost(False), endpoint.getPort(), title)

    def __trustScmServers(self):
        if (self.cfg.get('SSL', 'off') != 'on'):
            self.logMesg("debug", "SSL not enabled, importing SSL certificates is not necessary");
            return

        self.__trustService("subversion", "http-soap", "public", "SVN")
        self.__trustService("cvs", "http-soap", "public", "CVS")
        self.__trustService("gerrit", "http-web", "public", "Gerrit")
        self.__trustService("eventq", "http-eventq", "internal", "EventQ")
        self.__trustService("binary", "http", "public", "Binary")
        self.__trustService("reviewboard", "http", "public", "ReviewBoard")

    def initialize(self):
        self.__trustScmServers()
