﻿from app.scripts import BaseInitializationScript
import app.util
from app.selinux_permissions import SELinuxSetting
import os
import glob
import shutil
import ConfigParser
import app.selinuxutil
import app.landscape

class InitializationScript(BaseInitializationScript):

    scriptDescription = "Synchronize CLI Reports"

    def __canRun(self):
        if not app.landscape.Landscape.getSite().getLocalMachine().hasService("cliserver"):
           self.logMesg("debug", "Teamforge CLI Server is not deployed.")
           return False

        reportsDistDir = os.path.join(self.cfg['DISTRIBUTION_DIR'], "clireports")
        if not os.path.exists(reportsDistDir):
           self.logMesg("debug", "Teamforge CLI Reports is not installed.")
           return False

        return True

    def __synchronize(self):
        reportsDistDir = os.path.join(self.cfg['DISTRIBUTION_DIR'], "clireports")
        reportsDeployDir = os.path.join(self.cfg['DATA_DIR'], "cliserver", "app")

        httpdUser  = self.cfg["HTTPD_USER"]
        httpdGroup = self.cfg["HTTPD_GROUP"]

        if os.path.isdir("%s/reports" %reportsDeployDir):
             shutil.rmtree("%s/reports" %reportsDeployDir)

        app.util.runCommand("cp -rf %s/reports %s/reports" %(reportsDistDir, reportsDeployDir), self.log)

        self.__setCLIReportsReadOnlyMode('true')

        self.log.info("Configuring ownership permission to CLI reports binaries...")
        app.util.runCommand("chown -R %s:%s %s" %(httpdUser, httpdGroup, reportsDeployDir), self.log)

        cliSyncScript = os.path.join(self.cfg['RUNTIME_SCRIPTS'], 'clireports', 'syncreports.py')

        # Set httpd_sys_content_t to syncreports.py for branding post-commit to pass
        if app.selinuxutil.File.isSELinuxEnabled():
            selinuxsetting = SELinuxSetting()
            selinuxsetting.changeContext('httpd_sys_content_t', cliSyncScript)
            if self.cfg.has_key('ORACLE_HOME'):
                oracle_libocci_path = glob.glob(os.path.join(self.cfg['ORACLE_HOME'],"lib","libocci.so.*"))[0]
                selinuxsetting.changeContext('textrel_shlib_t', oracle_libocci_path)

        # Sync Productized CLI Reports to TeamForge DB
        cliReportsVarDir = os.path.join(self.cfg['DATA_DIR'], 'cliserver', 'app', 'reports', 'pkg')
        if (os.path.isdir(cliReportsVarDir) and os.path.isfile(cliSyncScript)):
            app.util.runCommand('%s native' % (cliSyncScript), self.log)

        # Sync Custom CLI Reports to TeamForge DB
        customCliReportsDir = os.path.join(self.cfg['DATA_DIR'], 'cliserver', 'cli', 'custom-reports', 'pkg')
        if (os.path.isdir(customCliReportsDir) and os.path.isfile(cliSyncScript)):
            app.util.runCommand('%s custom' % (cliSyncScript), self.log)

        # Setting the httpduser permission for sf-images/cn-reports directory.
        cnReportsImagePath = os.path.join(self.cfg['HTTPD_DOCUMENT_ROOT'], "sf-images", "cn-reports")
        try:
            app.util.runCommand("chown -R %s:%s %s" % (httpdUser, httpdGroup, cnReportsImagePath), self.log)
        except Exception as e:
            self.logMesg("debug", "Error while setting the permission for cn-reports image directory.[%r]" % e)

        # Add Sync CLI Reports to branding post-commit hook
        try:
            hookFile = os.path.join(self.cfg['SUBVERSION_BRANDING_REPOSITORY_BASE'], self.cfg['SUBVERSION_BRANDING_REPOSITORY_NAME'], 'hooks','post-commit')
            if (os.path.isfile(hookFile)):
                hookEntry = "%s custom"%(cliSyncScript)
                if self.__checkEntryExists(hookFile, hookEntry):
                    # Remove syncreports.py custom entry if already available to
                    # address change issue with change in order of invocation
                    # of cli_server add-on's update.pl and syncreports.py
                    self.__removeEntryFromFile(hookFile, hookEntry)
                self.logMesg("debug", "Adding sync cli reports call to branding repository post-commit hook")
                self.__appendEntryToFile(hookFile, hookEntry)
        except:
            self.logMesg("debug", "Error while adding sync cli reports call to branding repository post-commit hook")
            # No need to stop in case of exception as this is not very critical

    def __appendEntryToFile(self, inputFile, content):
        with open (inputFile, 'a') as wfHandler:
            wfHandler.write(content + "\n")

    def __removeEntryFromFile(self, inputFile, entry):
        with open (inputFile, 'r') as rfHandler:
            lines = rfHandler.readlines()

        #And reopen it in write mode:
        with open (inputFile, 'w') as wfHandler:
            for line in lines:
                if line!=entry+"\n":
                    wfHandler.write(line)

    def __checkEntryExists(self, inputFile, verifyLine):
        with open(inputFile, 'r') as fHandler:
            fileLines = fHandler.read().split("\n")
            return len(filter(lambda x: x == verifyLine, fileLines)) > 0

    def __setCLIReportsReadOnlyMode(self, mode):
        cliConfFile = os.path.join(self.cfg['DATA_DIR'], 'cliserver', 'app', 'reports', 'conf', 'clireports.properties')
        if (not os.path.isfile(cliConfFile)):
            with open(cliConfFile, 'wb') as fH:
                fH.close()
        parseObj = ConfigParser.ConfigParser()
        parseObj.read(cliConfFile);

        if (not parseObj.has_section('admin')):
            parseObj.add_section('admin')
            parseObj.set('admin','clireports','enabled')
            parseObj.set('admin','debug','false')
        parseObj.set('admin','readonlymode', mode)

        with open(cliConfFile, 'wb') as fH:
            parseObj.write(fH)

    def initialize(self):
        if not self.__canRun():
            return

        self.__synchronize()
