from app.scripts import BaseMigrationScript
import os
import app.util

class MigrationScript(BaseMigrationScript):

    scriptDescription = "Create symbolic link of branding repository base location"

    def migrate(self):

        scmRepoDataDir = self.cfg['SCM_REPOSITORY_DATA_DIR']

        if not os.path.isdir(scmRepoDataDir):
            cmd = "mkdir -p %s" % scmRepoDataDir
            app.util.executeCommand(cmd)

        self.createLinkToBranding(scmRepoDataDir)

    def createLinkToBranding(self, scmDataDir):

        # Softlink will be created to an new SCM data directory while
        # SUBVERSION_BRANDING_REPOSITORY_BASE token is referring to an
        # old location like /sf-svnroot

        brandRepoRootPath = self.cfg['SUBVERSION_BRANDING_REPOSITORY_BASE']
        brandSymlinkPath = "%s/sf-svnroot" % scmDataDir

        # In upgrade scenario The installer framework will create folder under /opt/collabnet/teamforge/var/scm/sf-svnroot
        # before calling migration hooks but which needs to be removed and created as soft link.
        if brandSymlinkPath != brandRepoRootPath and not os.path.islink(brandSymlinkPath) and \
                os.path.isdir(brandSymlinkPath) and not os.listdir(brandSymlinkPath):
            os.rmdir(brandSymlinkPath)

        if os.path.exists(brandSymlinkPath):
            return
        self.createSoftLink(brandRepoRootPath, brandSymlinkPath)

    def createSoftLink(self, originalPath, symlinkPath):

        scmUser = self.cfg["HTTPD_USER"]
        scmGroup = self.cfg["HTTPD_GROUP"]
        cmd = "chown -h %s:%s %s"  % (scmUser, scmGroup, symlinkPath)

        os.symlink(originalPath, symlinkPath)
        app.util.runSystem(cmd, self.log)

