﻿
from app.scripts import BaseBootstrapScript
import app.util
import app.schemerator
import app.defaults
import ext.lock

class BootstrapScript(BaseBootstrapScript):

    scriptDescription = "Bootstrap App database"

    def bootstrap(self):
        try:
            # Acquire lock to avoid concurrent bootstrapping in a cluster.
            with ext.lock.NetworkFolderLock(
                    app.defaults.DEFAULT_DATA_DIR,
                    app.landscape.Landscape.getSite().getLocalMachine().getPhysicalName()):
                self.__lockedBootstrap()

        except ext.lock.NetworkFolderOwnedByOtherPeer, e:
            # Lock acquisition failed.
            self.log.warn("Failed to lock folder - %s seems to be bootstrapping" % e.getPeer())

    def __lockedBootstrap(self):
        dbHost = self.cfg['ctfcore-database-admin-oracle/dependency/ctfcore-database/jdbc/host']
        dbPort = self.cfg['ctfcore-database-admin-oracle/dependency/ctfcore-database/jdbc/port']
        dbName = self.cfg['DATABASE_NAME']

        dbUrl = "jdbc:oracle:thin:@%s:%s:%s" % (dbHost, dbPort, dbName)

        try:
            app.schemerator.Schemerator(
                self.cfg['BOOTSTRAP_JAR'],
                dbUrl,
                self.cfg.get('DATABASE_USERNAME'),
                self.cfg.get('DATABASE_PASSWORD')).run(self.log)

        except app.schemerator.SchemeratorSchemaExistsException:
            raise Exception("Database schema already exists. To recreate the schema, "+
                            "drop and recreate the database user (%s) on %s" %
                            (self.cfg.get('DATABASE_USERNAME'), dbHost))
