﻿import os
import app.servicecontrol
import app.migratorator

class ServiceController(app.servicecontrol.ServiceController):

    def getDataStatus(self):
        # Deployment is already done, so it is fine to access runtime-options.conf
        config = app.util.getRuntimeConfiguration()

        try:
            dbHost = config['ctfcore-database-admin-oracle/dependency/ctfcore-database/jdbc/host']
            dbPort = config['ctfcore-database-admin-oracle/dependency/ctfcore-database/jdbc/port']
            dbName = config['DATABASE_NAME']

            dbUrl = "jdbc:oracle:thin:@%s:%s:%s" % (dbHost, dbPort, dbName)

            if app.migratorator.Migratorator.forDatabase(
                os.path.join(config['DISTRIBUTION_DIR'], "migratorator/migratorator.jar"),
                dbUrl,
                config['DATABASE_USERNAME'],
                config['DATABASE_PASSWORD']).isSchemaUpToDate(self.logger):

                return app.servicecontrol.ServiceStatus.AVAILABLE
            else:
                return app.servicecontrol.ServiceStatus.NOT_MIGRATED

        except app.migratorator.MigratoratorException:
            if not self.areRequiredEndpointsListening():
                # The database is not even available.
                return app.servicecontrol.ServiceStatus.DEPENDENCIES_UNAVAILABLE
            else:
                self.logger.warn("Database is available, but invoking migratorator failed - "+
                                 "database probably has not been bootstrapped yet")

                return app.servicecontrol.ServiceStatus.NOT_BOOTSTRAPPED

    def canBootstrap(self):
        if not super(ServiceController, self).canBootstrap():
            return False

        # We need Oracle running in order to be able to bootstrap.
        if not self.areRequiredEndpointsListening():
            self.logger.error("Cannot bootstrap as Postgres server is not available")
            return False
        else:
            return True
