﻿import os
import app.servicecontrol
import app.migratorator
from app.pgdbadmin import PGCTFDBManager
from app.landscape import Landscape

class ServiceController(app.servicecontrol.ServiceController):

    def getDataStatus(self):
        # Deployment is already done, so it is fine to access runtime-options.conf
        config = app.util.getRuntimeConfiguration()

        try:
            dbHost = config['ctfcore-database-admin-postgres/dependency/ctfcore-database/jdbc/host']
            dbPort = config['ctfcore-database-admin-postgres/dependency/ctfcore-database/jdbc/port']
            dbName = config['DATABASE_NAME']
            dbSSL  = config['DATABASE_SSL']

            dbUrl = "jdbc:postgresql://%s:%s/%s" % (dbHost, dbPort, dbName)
            if dbSSL == "on":
                dbUrl += '?ssl=true&sslfactory=org.postgresql.ssl.NonValidatingFactory&'

            if app.migratorator.Migratorator.forDatabase(
                os.path.join(config['DISTRIBUTION_DIR'], "migratorator/migratorator.jar"),
                dbUrl,
                config['DATABASE_USERNAME'],
                config['DATABASE_PASSWORD']).isSchemaUpToDate(self.logger):

                return app.servicecontrol.ServiceStatus.AVAILABLE
            else:
                return app.servicecontrol.ServiceStatus.NOT_MIGRATED

        except app.migratorator.MigratoratorException:
            if not self.areRequiredEndpointsListening():
                # The database is not even available.
                return app.servicecontrol.ServiceStatus.DEPENDENCIES_UNAVAILABLE
            else:
                # If the database is local, then migratorator failing could be do to the passwords being out of sync.
                # By returning NOT_MIGRATED, we make sure that the passwords will be synchronized during the migrate phase.
                # In a scenario where the database is remote, this situation should not occur as the database server is always
                # first in the provisioning sequence.
                for dbServer in Landscape.getSite().getMachinesByService("ctfcore-database-postgres"):
                    if dbServer.isLocal():
                        db = PGCTFDBManager(
                                 config['DATABASE_NAME'],
                                 config['DATABASE_USERNAME'],
                                 config['DATABASE_PASSWORD'],
                                 config['DATABASE_READ_ONLY_USER'],
                                 config['DATABASE_READ_ONLY_PASSWORD'],
                                 self.logger,
                                 config['ctfcore-database-admin-postgres/dependency/ctfcore-database/jdbc/port'],
                                 config['ctfcore-database-admin-postgres/dependency/ctfcore-database/jdbc/host'],
                                 config['PGPASSFILE'])

                        if db.isDatabaseExisting():
                            return app.servicecontrol.ServiceStatus.NOT_MIGRATED

                self.logger.warn("Database is available, but invoking migratorator failed - "+
                                 "database probably has not been bootstrapped yet")

                return app.servicecontrol.ServiceStatus.NOT_BOOTSTRAPPED

    def canBootstrap(self):
        if not super(ServiceController, self).canBootstrap():
            return False

        # We need Postgres running in order to be able to bootstrap.
        if not self.areRequiredEndpointsListening():
            self.logger.error("Cannot bootstrap as Postgres server is not available")
            return False
        else:
            return True

