﻿from app.scripts import BaseMigrationScript

import os
import app.util
import app.defaults
import ext.lock

class MigrationScript(BaseMigrationScript):

    scriptDescription = "Migrate CTF Reports Database to latest schema"

    def migrate(self):
        try:
            # Acquire lock to avoid concurrent migrating in a cluster.
            with ext.lock.NetworkFolderLock(
                    app.defaults.DEFAULT_DATA_DIR,
                    app.landscape.Landscape.getSite().getLocalMachine().getPhysicalName()):
                self.__lockedMigrate()

        except ext.lock.NetworkFolderOwnedByOtherPeer, e:
            # Lock acquisition failed.
            self.log.warn("Failed to lock folder - %s seems to be migrating" % e.getPeer())

    def __lockedMigrate(self):
        databaseType = self.cfg['DATABASE_TYPE']

        jdbcUrl = "jdbc:oracle:thin:@%s:%s:%s" % \
            (self.cfg['ctfcore-datamart-admin-oracle/dependency/ctfcore-database/jdbc/host'], self.cfg['ctfcore-datamart-admin-oracle/dependency/ctfcore-database/jdbc/port'], self.cfg['DATABASE_NAME'])

        reportsJdbcUrl = "jdbc:oracle:thin:@%s:%s:%s" % \
            (self.cfg['ctfcore-datamart-admin-oracle/dependency/ctfcore-datamart/jdbc/host'], self.cfg['ctfcore-datamart-admin-oracle/dependency/ctfcore-datamart/jdbc/port'], self.cfg['REPORTS_DATABASE_NAME'])

        app.migratorator.Migratorator.forDatamart(
            os.path.join(self.cfg['DISTRIBUTION_DIR'], "migratorator/migratorator.jar"),
            jdbcUrl,
            self.cfg['DATABASE_USERNAME'],
            self.cfg['DATABASE_PASSWORD'],
            reportsJdbcUrl,
            self.cfg['REPORTS_DATABASE_USERNAME'],
            self.cfg['REPORTS_DATABASE_PASSWORD']).migrate(self.log)
