﻿
from app.scripts import BaseBootstrapScript
import app.util
import app.pgdbadmin
import app.schemerator
import app.defaults
import ext.lock

class BootstrapScript(BaseBootstrapScript):

    scriptDescription = "Bootstrap Reporting database"

    def bootstrap(self):
        try:
            # Acquire lock to avoid concurrent bootstrapping in a cluster.
            with ext.lock.NetworkFolderLock(
                    app.defaults.DEFAULT_DATA_DIR,
                    app.landscape.Landscape.getSite().getLocalMachine().getPhysicalName()):
                self.__lockedBootstrap()

        except ext.lock.NetworkFolderOwnedByOtherPeer, e:
            # Lock acquisition failed.
            self.log.warn("Failed to lock folder - %s seems to be bootstrapping" % e.getPeer())

    def __lockedBootstrap(self):
        dbHost = self.cfg['ctfcore-datamart-admin/dependency/ctfcore-datamart/jdbc/host']
        dbName = self.cfg['REPORTS_DATABASE_NAME']
        dbPort = self.cfg['ctfcore-datamart-admin/dependency/ctfcore-datamart/jdbc/port']
        dbSSL  = self.cfg['REPORTS_DATABASE_SSL']

        dbUrl = "jdbc:postgresql://%s:%s/%s" % (dbHost, dbPort, dbName)
        if dbSSL == "on":
            dbUrl += '?ssl=true&sslfactory=org.postgresql.ssl.NonValidatingFactory&'

        try:
            app.schemerator.Schemerator(
                self.cfg['REPORTS_BOOTSTRAP_JAR'],
                dbUrl,
                self.cfg.get('REPORTS_DATABASE_USERNAME'),
                self.cfg.get('REPORTS_DATABASE_PASSWORD')).run(self.log)

        except app.schemerator.SchemeratorSchemaExistsException:
            raise Exception("Database schema already exists. To recreate the schema, "+
                            "bootstrap the service 'ctfcore-datamart-postgres[-cohosted]' first")

        db = app.pgdbadmin.PGCTFDBManager(
            dbName,
            self.cfg.get('REPORTS_DATABASE_USERNAME'),
            self.cfg.get('REPORTS_DATABASE_PASSWORD'),
            self.cfg.get('REPORTS_DATABASE_READ_ONLY_USER'),
            self.cfg.get('REPORTS_DATABASE_READ_ONLY_PASSWORD'),
            self.log,
            dbPort,
            dbHost,
            self.cfg.get('REPORTS_PGPASSFILE'))
        db.grantReadOnlyPermissionsForAllTablesInSchema()
