﻿from app.scripts import BaseMigrationScript

import os
import app.util
from app.pgdbadmin import PGCTFDBManager
import app.defaults
import ext.lock

class MigrationScript(BaseMigrationScript):

    scriptDescription = "Migrate CTF Reports Database to latest schema"

    def migrate(self):
        try:
            # Acquire lock to avoid concurrent migrating in a cluster.
            with ext.lock.NetworkFolderLock(
                    app.defaults.DEFAULT_DATA_DIR,
                    app.landscape.Landscape.getSite().getLocalMachine().getPhysicalName()):
                self.__lockedMigrate()

        except ext.lock.NetworkFolderOwnedByOtherPeer, e:
            # Lock acquisition failed.
            self.log.warn("Failed to lock folder - %s seems to be migrating" % e.getPeer())

    def __lockedMigrate(self):
        jdbcUrl = "jdbc:postgresql://%s:%s/%s" % \
            (self.cfg['ctfcore-datamart-admin/dependency/ctfcore-database/jdbc/host'], self.cfg['ctfcore-datamart-admin/dependency/ctfcore-database/jdbc/port'], self.cfg['DATABASE_NAME'])

        reportsJdbcUrl = "jdbc:postgresql://%s:%s/%s" % \
            (self.cfg['ctfcore-datamart-admin/dependency/ctfcore-datamart/jdbc/host'], self.cfg['ctfcore-datamart-admin/dependency/ctfcore-datamart/jdbc/port'], self.cfg['REPORTS_DATABASE_NAME'])

        if self.cfg['DATABASE_SSL'] == 'on':
            jdbcUrl = "%s?ssl=true&sslfactory=org.postgresql.ssl.NonValidatingFactory" % jdbcUrl

        if self.cfg['REPORTS_DATABASE_SSL'] == 'on':
            reportsJdbcUrl = "%s?ssl=true&sslfactory=org.postgresql.ssl.NonValidatingFactory" % reportsJdbcUrl

        app.migratorator.Migratorator.forDatamart(
            os.path.join(self.cfg['DISTRIBUTION_DIR'], "migratorator/migratorator.jar"),
            jdbcUrl,
            self.cfg['DATABASE_USERNAME'],
            self.cfg['DATABASE_PASSWORD'],
            reportsJdbcUrl,
            self.cfg['REPORTS_DATABASE_USERNAME'],
            self.cfg['REPORTS_DATABASE_PASSWORD']).migrate(self.log)

        db = PGCTFDBManager(
            self.cfg.get('REPORTS_DATABASE_NAME'),
            self.cfg.get('REPORTS_DATABASE_USERNAME'),
            self.cfg.get('REPORTS_DATABASE_PASSWORD'),
            self.cfg.get('REPORTS_DATABASE_READ_ONLY_USER'),
            self.cfg.get('REPORTS_DATABASE_READ_ONLY_PASSWORD'),
            self.log,
            self.cfg.get('ctfcore-datamart-admin/dependency/ctfcore-datamart/jdbc/port'),
            self.cfg.get('ctfcore-datamart-admin/dependency/ctfcore-datamart/jdbc/host'),
            self.cfg.get('REPORTS_PGPASSFILE'))

        # Below may not be required for schema changes after 16.10 as
        # bootstrap alter default privileges to accomodate this. Please
        # feel free to remove this when ideal.
        db.grantReadOnlyPermissionsForAllTablesInSchema()
