#!/usr/bin/env python2




# backup-data.py: -*- Python -*-

""" backs up the database and the data directory"""

import sys
import getopt
import os
import shutil

import app.util
import service.util
import service.constants

cfg = app.util.getRuntimeConfiguration()

def usage(errorMessage=None):
    if errorMessage:
        stream = sys.stderr
    else:
        stream = sys.stdout

    stream.write("Usage: \n"
                 "%s [--help] --destination_dir={dir}\n"
                 % (sys.argv[0],))

    stream.write("\n"
                 "This script handles storing the current data for the\n"
                 "site.  It will remove a previous backup if it finds one\n"
                 "in the destination dir; then it backups up everything \n"
                 "into that directory.By default, both ctf and reporting db \n"
		 "is backed up to the destination directory. If reporting is \n"
		 "disabled, only the ctf db is backed up. \n")

    if errorMessage:
        stream.write("\nERROR:%s\n" % errorMessage)
        exit_code = 1
    else:
        exit_code = 0

    sys.exit(exit_code)


def main():
    opts, args = getopt.getopt(sys.argv[1:], 'hr:',
                               ['help', 'destination_dir='])
    destinationDir = None
    wipe = 1
    create = 1
    for opt, value in opts:
        if (opt in ('-h', '--help')):
            usage()
        elif (opt in ('-r', '--destination_dir')):
            destinationDir = value
        elif (opt == '--nowipe'):
            wipe = 0
        elif (opt == '--nocreate'):
            create = 0
        else:
            usage('unknown option : ' + opt)
    if not destinationDir or destinationDir == '':
        usage('--destination_dir={dir} is required')
    storeData(destinationDir, 'ctf')
    if (service.util.isServiceEnabledLocally(service.constants.SERVICE_REPORTS_PGSQL)):
        storeData(destinationDir, 'reporting')

## we want to make sure that there's nothing else in the destination dir to
## get in the way.  we also want to make sure that it exists (or can be
## created).
def storeData(dir, databaseType):
    if databaseType == 'ctf':
        dataDirFile = "data.tar.gz"
        dbDir = "db"
    else:
        dataDirFile = "reportingdata.tar.gz"
        dbDir = "reporting-db"
    dataTarball = os.path.join(dir, dataDirFile)
    if os.path.isfile(dataTarball):
        os.unlink(dataTarball)
    dbDumpDir = os.path.join(dir, dbDir)
    if os.path.isdir(dbDumpDir):
        shutil.rmtree(dbDumpDir)
    os.makedirs(dbDumpDir)
    os.system("chown -R postgres:postgres %s" %dbDumpDir)
    tar = cfg.get("GTAR_BINARY")

    dataDir = cfg.get("DATA_DIR")
    if dataDir[-1] == '/': # remove trailing slash if present
        dataDir = dataDir[0:-1]
    dataDirParent, dataDirName = os.path.split(dataDir)

    # FIXME: Perform better error handling of os.system()
    # Backup the entire DATA_DIR dir, from its parent directory.
    os.system("cd %s; %s cfzp %s %s/. " %
                          (dataDirParent, tar, dataTarball, dataDirName))
    # Now we deal with the database backup.
    runtimeDir = cfg.get("RUNTIME_DIR")
    # DB-Backup is commented for time being will be uncommented once
    # the db.pl script is ready.
    os.system(os.path.join(runtimeDir, 'scripts', 'db.py') +
              ' --path=%s --action=dump --type=%s' % (dbDumpDir, databaseType))
    # Now let's use scmadmin to backup the svn (bdb) folders
    if databaseType == 'ctf':
        os.system(os.path.join(runtimeDir, 'scripts', 'scmadmin') +
                  ' snapto "%s" ' % dir)

if __name__ == '__main__':
    main()
