#!/usr/bin/env python
"""This script does sync of cli reporttypes configuration to TeamForge
database. This is executed via post-installation and branding post-commit
hook call.

Input Specifications:
1. Takes only one argument.
2. The argument should be any one of this "native" or "custom".
3. Any other values in the argument doesnot errors out but fille backback
   to the default value "native".

Argument Type Specifications:
1. native -> scans the cli reports directory provided by collabnet.
2. custom -> scans the custom cli reports directory typically the
             branding repository's working copy of look project.

What it Does:
1. It scans through the clireports directory and parses the report
   configuration file viz config.ini in every report directory.
2. The parsed values are synched to teamforge database into
   "report_type_meta_data".
3. The corresponding sample images in cli reports directories are copied
   to /sf-images/cn-reports directory and the corresponding url location
   of the coped images are updated into report_type_meta_data table for the
   respective cli report.

Output Specifications:
Since this script is used by teamforge system components, it doesnot
produce any useful outputs on the console and it exists silently for all
success and failure cases.
"""

__version__ = "1.0"

import os
import sys
import app.util
import app.clireports.syncapp

cfg = app.util.getRuntimeConfiguration()

try:
   reportType = 'native'
   if len(sys.argv) > 1:
      reportType = sys.argv[1]

   if reportType == 'custom':
      cliReportsDir = os.path.join(cfg['DATA_DIR'], 'cliserver', 'cli', 'custom-reports', 'pkg')
   else:
      cliReportsDir = os.path.join(cfg['DATA_DIR'], 'cliserver', 'app', 'reports', 'pkg')

   sync = app.clireports.syncapp.SyncApp(cliReportsDir)
   sync.syncCliReports()
except:
   print "Error while synchronising clireports with teamforge!!"

sys.exit(0)
