#!/usr/bin/env python

"""This script's sole purpose is for dump or restore of the database """

import sys, os, getopt
import app.util
import service.util

class DB:
    def __init__(self, argv):

        try:
            opts, args = getopt.getopt(sys.argv[1:], "Fha:f:t:", ["help", "action=", "path=", "type=", "force"])
        except getopt.GetoptError, exc:
            self.printUsageAndExit("Getopt Error:%s\n" % exc, 1)

        self.force = False
        self.type = ""

        for option, value in opts:
            if option in ("-h", "--help"):
                self.printUsageAndExit()
            elif option in ('-f', "--path"):
                self.db_backup_file_path = value or ''
            elif option in ('-a', "--action"):
                self.action = value
            elif option in ('-t', "--type"):
                self.type = value
            elif option in ('-F', "--force"):
                self.force = True

        # get configuration details
        self.cfg = app.util.getRuntimeConfiguration()

        if self.action == 'dump':
            self.dumpDatabases()
        elif self.action == 'restore':
            self.restoreDatabases()
        else:
            self.printErrorAndExit("Not a valid action\n", 2)

    # Dump the databases based on the type attribute
    def dumpDatabases(self):
        if self.type == 'reporting':
	    if not service.util.isServiceEnabledLocally(service.constants.SERVICE_REPORTS_PGSQL):
	        self.printErrorAndExit("Reporting is Disabled\n", 8)
            # Dump Report Db
            self.dumpReportDb()
        elif self.type == 'ctf':
            # Dump Ctf Db
            self.dumpCtfDb()
	elif self.type == "":
            # Dump both Report Db And Ctf Db
	    if service.util.isServiceEnabledLocally(service.constants.SERVICE_REPORTS_PGSQL):
                self.dumpReportDb()
            self.dumpCtfDb()
        else:
            self.printErrorAndExit("Not a valid type\n", 3)

    # Set the Report database properties and dump the database
    def dumpReportDb(self):
        self.setReportDbProperties()
        self.dump_db()

    # Set the Ctf database properties and dump the database
    def dumpCtfDb(self):
        self.setCtfDbProperties()
        self.dump_db()

    # Restore the databases based on the type attribute
    def restoreDatabases(self):
        if self.type == 'reporting':
	    if not service.util.isServiceEnabledLocally(service.constants.SERVICE_REPORTS_PGSQL):
                self.printErrorAndExit("Reporting is Disabled\n", 8)
            # Restore Report Db"
            self.restoreReportDb()
        elif self.type == 'ctf':
            # Restore Ctf Db"
            self.restoreCtfDb()
        elif self.type == "":
            # Restoreboth Report Db and Ctf Db
	    if service.util.isServiceEnabledLocally(service.constants.SERVICE_REPORTS_PGSQL):
                self.restoreReportDb()
            self.restoreCtfDb()
	else:
	    self.printErrorAndExit("Not a valid type\n", 3)


    # Set the Report database properties and restore the database
    def restoreReportDb(self):
        self.setReportDbProperties()
	if os.path.exists(self.db_backup_file):
            self.restore_db()

    # Set the Ctf database properties and restore the database
    def restoreCtfDb(self):
        self.setCtfDbProperties()
        self.restore_db()

    # Set the Report database properties
    def setReportDbProperties(self):
        self.dbUserName = 'REPORTS_DATABASE_USERNAME'
        self.dbHostName = 'ctfcore/dependency/ctfcore-datamart/jdbc/host'
        self.dbPort = 'ctfcore/dependency/ctfcore-datamart/jdbc/port'
        self.dbName = 'REPORTS_DATABASE_NAME'
        self.dbPassword = 'REPORTS_DATABASE_PASSWORD'
        self.pgpassFile = 'REPORTS_PGPASSFILE'
        self.dataDir = "reporting-data"
        self.setAdditionalDbProperties()
        self.setPassFileToken()

    # Set the Ctf database properties
    def setCtfDbProperties(self):
        self.dbUserName = 'DATABASE_USERNAME'
        self.dbHostName = 'ctfcore/dependency/ctfcore-database/jdbc/host'
        self.dbPort = 'ctfcore/dependency/ctfcore-database/jdbc/port'
        self.dbName = 'DATABASE_NAME'
        self.dbPassword = 'DATABASE_PASSWORD'
        self.pgpassFile = 'PGPASSFILE'
        self.dataDir = "data"
        self.setAdditionalDbProperties()
        self.setPassFileToken()

    # Set the additional database properties
    def setAdditionalDbProperties(self):
        if self.db_backup_file_path == '':
            self.db_backup_file_path = os.path.join(self.cfg['DATA_DIR'], self.dataDir)
        # getting the db details
        self.db_user = self.cfg[self.dbUserName]
        self.db_host = self.cfg[self.dbHostName]
        self.db_port = self.cfg[self.dbPort]
        self.db_name = self.cfg[self.dbName]
        self.db_pass = self.cfg[self.dbPassword]
        self.db_backup_file_name = self.db_name + ".sql"
        self.db_backup_file = os.path.join(self.db_backup_file_path, self.db_backup_file_name)

    # Set the pgpassfile token
    def setPassFileToken(self):
        # setting the pgpassfile token
        try:
            os.environ['PGPASSFILE'] = self.cfg[self.pgpassFile]
        except:
            self.printErrorAndExit("PGPASSFILE is not available", 1)

    def printUsageAndExit(self, error=None, exit_code=0):
        print
        print "Valid options: "
        print "    -h | --help             print this usage message and exit"
        print "    -a | --action=          perform the action: dump or restore"
        print "    -t | --type=            specify the type of database: ctf or reporting"
        print "    -f | --path=            specify the path of the appropriate directory which has the postgres permission (or) a new path for the database backup file"
        print
        print " -a is mandatory"
        print " -f is mandatory"
        print
        if error:
            print >> sys.stderr, error

        sys.exit(exit_code)

    def printErrorAndExit(self, errMsg, exitCode = 1):
        print >> sys.stderr, errMsg
        sys.exit(exitCode)


    # dumps database
    def dump_db(self):
        if not os.path.exists(self.db_backup_file_path):
            try:
                os.makedirs(self.db_backup_file_path)
                os.system("chown -R postgres:postgres %s" %self.db_backup_file_path)
            except:
                self.printErrorAndExit("Unable to create path for DB backup: %s " %self.db_backup_file_path, 3)

        if self.db_name != '':
            try:
                self.cmd = 'export PGPASSWORD=\'%s\''%(self.db_pass)
                ret = app.util.executeCommand(self.cmd)
		if (service.util.isServiceEnabledLocally(service.constants.SERVICE_REPORTS_PGSQL)
		    and self.cfg['ctfcore/dependency/ctfcore-datamart/jdbc/port'] == self.cfg['ctfcore/dependency/ctfcore-database/jdbc/port']):
                    self.cmd = "su - postgres -c 'pg_dump -C -p %s %s > %s'" %(self.db_port, self.db_name, self.db_backup_file)
                else:
                    self.cmd = "su - postgres -c 'pg_dumpall -p %s > %s'" %(self.db_port, self.db_backup_file)
                ret = app.util.executeCommand(self.cmd)
            except Exception as err:
                self.printErrorAndExit("Unable to take DB backup: %s\n" % str(err), 4)

    # restores database
    def restore_db(self):
        if self.force:
            option = 'YES'
            print "Force option selected, so proceeding to replace the existing data by the dump you have selected."
        else:
            option  = app.util.readFromUser("This script is about to replace the existing db data with the source provided. Do you want to proceed? YES/NO", ['YES', 'NO'])
            if option == 'YES':
                pass
            else:
                self.printErrorAndExit("Exiting restore db on user request..")

        if not os.path.exists(self.db_backup_file):
            self.printErrorAndExit("Unable to find DB backup file: %s " %self.db_backup_file, 5)

        if self.db_name != '':
            try:
		self.cmd = """
echo "SELECT datname FROM pg_database ;" \
| psql -p %s -U postgres --tuples-only 2>/dev/null\
| awk -F\| '{ sub(/^ */,"",$1); sub(/ *$/,"",$1); print $1 }' \
|  grep ^%s$""" % (self.db_port, self.db_name)
            	try:
                    ret = app.util.executeCommand(self.cmd)
		    self.cmd = "su - postgres -c 'dropdb %s -p %s'" %(self.db_name, self.db_port)
		    ret = app.util.executeCommand(self.cmd)
		    self.cmd = "su - postgres -c 'createdb -E UTF8 -O %s %s -p %s'" %(self.db_user, self.db_name, self.db_port)
		    ret = app.util.executeCommand(self.cmd)
		except:
		    pass
                self.cmd = "su - postgres -c 'psql -p %s < %s'" %(self.db_port, self.db_backup_file)
                print "Restoring %s db, please wait...." %self.db_name
                ret = app.util.executeCommand(self.cmd)
            except:
                print " ".join(app.util.getStackTraceAndException())
        else:
            self.printErrorAndExit("Unable to restore db, due to empty db name\n", 7)

if __name__ == '__main__':
        dbObj = DB(sys.argv)
        #dbObj.run()

