#!/usr/bin/env python2




# restore-data.py: -*- Python -*-

""" restores the database and the data directory"""

import sys
import getopt
import os

import app.util
import app.preprocessor
import service.util
import service.constants

cfg = app.util.getRuntimeConfiguration()

def usage(errorMessage=None):
    if errorMessage:
        stream = sys.stderr
    else:
        stream = sys.stdout

    stream.write("Usage: \n"
                 "%s [--help] --source={dir} \n"
                 % (sys.argv[0],))

    stream.write("\n"
                 "This script handles restoring the current data from the\n"
                 "named source directory.  It will remove any data currently\n"
                 "present.By default, both ctf and reporting db \n"
                 "is restored from the source directory. If reporting is \n"
                 "disabled, only the ctf db is restored.\n")

    if errorMessage:
        stream.write("\nERROR:%s\n" % errorMessage)
        exit_code = 1
    else:
        exit_code = 0

    sys.exit(exit_code)


def main():

    opts, args = getopt.getopt(sys.argv[1:], 'hnr:',
                               ['help', 'source=', 'non_interactive'])
    sourceDir = None
    wipe = 1
    create = 1
    non_interactive = False
    for opt, value in opts:
        if (opt in ('-h', '--help')):
            usage()
        elif (opt in ('-r', '--source')):
            sourceDir = value
        elif (opt == '--nowipe'):
            wipe = 0
        elif (opt == '--nocreate'):
            create = 0
        elif (opt in ('-n', '--non_interactive')):
            non_interactive = True
        else:
            usage('unknown option : ' + opt)
    if not sourceDir or sourceDir == '':
        usage('--source={dir} is required')
    restoreData(sourceDir, 'ctf', non_interactive)
    if (service.util.getApplicationHost(service.constants.APPLICATION_REPORTS_DATABASE) is not None
        and os.path.isdir(os.path.join(sourceDir, "reporting-db"))
        and os.path.isfile(os.path.join(sourceDir, "reportingdata.tar.gz"))):
        restoreData(sourceDir, 'reporting', non_interactive)

## we want to make sure there's nothing in the actual data directory
## to confuse matters -- so we start by removing all the subdirs.  we make
## a minimal effort to support symlinks.
def restoreData(dir, databaseType, non_interactive):
    option = "YES"
    if databaseType == 'ctf':
       dataDirFile = "data.tar.gz"
       dbDir = "db"
    else:
       dataDirFile = "reportingdata.tar.gz"
       dbDir = "reporting-db"
    if not non_interactive and databaseType == 'ctf':
       option  = app.util.readFromUser("This script is about to replace the existing filesystem data with the source provided (%s).Do you want to proceed? YES/NO" % (dir), ['YES', 'NO'])
    if option == 'YES':
        pass
    else:
        usage("Exiting restore(svn,cvs,datadir,db) on user request..")
        sys.exit()

    dataTarball = os.path.join(dir, dataDirFile)
    dbDumpDir = os.path.join(dir, dbDir)
    tar = cfg.get("GTAR_BINARY")

    dataDir = cfg.get("DATA_DIR")
    if dataDir[-1] == '/': # remove trailing slash
        dataDir = dataDir[0:-1]
    dataDirParent, dataDirName = os.path.split(dataDir)

    # We better have a database backup directory.
    if not os.path.isdir(dbDumpDir):
        usage("db subdir not found in the source -- no restore")

    # We better have a DATA_DIR tarball.
    if not os.path.isfile(dataTarball):
        usage(dataDirFile + "tarball not found in the source -- no restore")

    def _restoreTarball(tarCmd, tarball, parentDir, basename):
        ## we remove everything in the target dir, but not the target
        ## dir itself (in case it's a symlink somewhere else)
        os.system("cd %s; rm -fr %s/*" % (parentDir, basename))

        ## restore the entire dir.  we go to the parent of the target
        ## dir and get the basename to try to handle symlinks in a
        ## minimal sense
        os.system("cd %s; %s xfzp %s " % (parentDir, tarCmd, tarball))

    _restoreTarball(tar, dataTarball, dataDirParent, dataDirName)
    if databaseType == 'ctf':
        runtimeDir = cfg.get("RUNTIME_DIR")
        os.system(os.path.join(runtimeDir, 'scripts', 'scmadmin') +
                  ' restorefrom "%s" ' % dir)

    ## now we deal with the db restore
    runtimeDir = cfg.get("RUNTIME_DIR")
    os.system(os.path.join(runtimeDir, 'scripts', 'db.py') +
              ' --path=%s --action=restore --type=%s -F' % (dbDumpDir, databaseType))

if __name__ == '__main__':
    main()
