#!/usr/bin/env python

"""
This script helps to synchronize the ctf projects with the orchestrate by triggering the world-state message for the given project.
"""

import os
import os.path
import sys
import getopt
import getpass
import app.util

from suds.client import Client
from suds.cache import ObjectCache

def usage(error=None, exit_code=0):

    scriptName = __file__
    print "\n Usage:\n %s --projectId [[--tfuser=<username>] [--tfpasswd=<password>]]" % scriptName
    print '''
 Description:\n This script helps to synchronize the teamforge project with the orchestrate by triggering the world-state message for the given project.

 Arguments:
     --projectId= the teamforge project id
 -u, --tfuser=    the teamforge site-admin user name
 -p, --tfpasswd=  the password for the given teamforge user
 -h, --help       this help information
'''
    if (error is not None):
       print " ERROR MESSAGE:\t%s" % error
       sys.exit(1)
    else:
       sys.exit(0)


def get_session_id(username, password):
    """Calls the login soap api to get the session id"""
    cfg = app.util.getRuntimeConfiguration()
    soap_url = "%s/ce-soap60/services" % cfg.get('API_SOAP')
    sudsCacheDir = os.path.join(cfg.get('TEMPORARY_DIR'), "suds")
    sudsCache = ObjectCache(sudsCacheDir)
    cnapp = Client("%s/CollabNet?wsdl" % soap_url, cache=sudsCache, location="%s/CollabNet?wsdl" % soap_url)
    return cnapp.service.login(username, password)


def sync_orchestrate(sessionId, projectId):
    """Calls the soap api to trigger the world-state message for the given project"""
    cfg = app.util.getRuntimeConfiguration()
    soap_url = "%s/ce-soap/services" % cfg.get('API_SOAP')
    sudsCacheDir = os.path.join("%s" % cfg.get('TEMPORARY_DIR'), "suds")
    sudsCache = ObjectCache(sudsCacheDir)
    orcapp = Client("%s/OrchestrateSyncListener?wsdl" % soap_url, cache=sudsCache,
                    location="%s/OrchestrateSyncListener?wsdl" % soap_url)
    orcapp.service.sendWorldStateMessage(sessionId, projectId)
    print "World-State message triggered for project %s" % projectId


def main():
    """ Main function """
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hu:p:", ["help", "projectId=", 'tfuser=', 'tfpasswd='])
    except getopt.GetoptError:
        usage('Invalid option supplied.')

    projectId = None
    tf_username = ''
    tf_password = ''
    for opt, val in opts:
        if opt in ("-h", "--help"):
            usage()
        elif opt == '--projectId':
            projectId = val
        elif (opt in ("-u", "--tfuser")):
           tf_username = val
        elif (opt in ("-p", "--tfpasswd")):
            tf_password= val
        else:
            usage('Invalid option supplied.' + opt )
            sys.exit()


    if not projectId:
	usage('Argument missing : --projectId')

    # Get TeamForge username and password
    if tf_username == '':
        tf_username = raw_input("Enter TeamForge Site-admin Username: ")
    if tf_password == '':
        tf_password = getpass.getpass("Enter TeamForge Site-admin Password: ")

    sync_orchestrate(get_session_id(tf_username, tf_password), projectId)


if __name__ == "__main__":
    main()

