#!/bin/bash

# Source the tokens
eval "`$RUNTIME_DIR/scripts/options-to-shell-variables.py`"

wait_for_appserver() {
	echo "Waiting for application server to start up.. this can take a few minutes."
        # Actually, waits for sf-soap/AdminService to appear
	RESULT=1;
    COUNT=0
	while [ $RESULT -ne 0 -a $COUNT -lt 60 ]; do
		$JAVA_HOME/bin/java -cp "$SOURCEFORGE_HOME_LIB_CLASSPATH" org.apache.axis.client.AdminClient list -l "http://127.0.0.1:8080/ce-soap60/services/AdminService" 2> /dev/null > /dev/null
		RESULT=$?
	        if [ $RESULT -ne 0 ]; then
              COUNT=$(($COUNT+1))
		    sleep 5
		fi
	done

}

wait_for_webservice_deployer() {
	SOAP_PACKAGE=$1
	echo "Deploy webservice $SOAP_PACKAGE"
	RESULT=1;
    COUNT=0
	while [ $RESULT -ne 0 -a $COUNT -lt 60 ]; do
		$JAVA_HOME/bin/java -cp "$SOURCEFORGE_HOME_LIB_CLASSPATH" org.apache.axis.client.AdminClient list -l "http://127.0.0.1:8080/$SOAP_PACKAGE/services/AdminService" 2> /dev/null > /dev/null
		RESULT=$?
	        if [ $RESULT -ne 0 ]; then
              COUNT=$(($COUNT+1))
		    sleep 5
		fi
	done
}

deploy_webservice() {
	SOAP_PACKAGE=$1
	WSDD_FILE=$2

	if [ -f "$WSDD_FILE" ]; then
	        echo "	deploying" `basename $WSDD_FILE Deploy.wsdd`
		$JAVA_HOME/bin/java -cp "$SOURCEFORGE_HOME_LIB_CLASSPATH" org.apache.axis.client.AdminClient "$WSDD_FILE" -l "http://127.0.0.1:8080/$SOAP_PACKAGE/services/AdminService" 2> /dev/null > /dev/null
		if [ 0 -ne 0 ]; then
			echo "Could not start service $WSDD_FILE"
		fi
	else
		echo "No such file: $WSDD_FILE"
	fi
}

case "$1" in
start)
	wait_for_appserver
	wait_for_webservice_deployer "ce-soap"
	deploy_webservice "ce-soap" "$SOURCEFORGE_HOME/etc/webservices/deploy/integration/ScmListenerDeploy.wsdd"
	deploy_webservice "ce-soap" "$SOURCEFORGE_HOME/etc/webservices/deploy/integration/DiscussionListenerDeploy.wsdd"
	deploy_webservice "ce-soap" "$SOURCEFORGE_HOME/etc/webservices/deploy/integration/DocmanListenerDeploy.wsdd"
	deploy_webservice "ce-soap" "$SOURCEFORGE_HOME/etc/webservices/deploy/integration/TrackerListenerDeploy.wsdd"
        deploy_webservice "ce-soap" "$SOURCEFORGE_HOME/etc/webservices/deploy/integration/MessagingListenerDeploy.wsdd"
	deploy_webservice "ce-soap" "$SOURCEFORGE_HOME/etc/webservices/deploy/integration/SimbelListenerDeploy.wsdd"
        if [ -n "$TEST_SUPPORT_ENABLED" -a "$TEST_SUPPORT_ENABLED" == "true" ]; then
            deploy_webservice "ce-soap" "$SOURCEFORGE_HOME/etc/webservices/deploy/integration/IntegratedAppTestSoapDeploy.wsdd"
        fi
        deploy_webservice "ce-soap" "$SOURCEFORGE_HOME/etc/webservices/deploy/integration/IAFDummyServiceDeploy.wsdd"
        deploy_webservice "ce-soap" "$SOURCEFORGE_HOME/etc/webservices/deploy/etl/EtlListenerDeploy.wsdd"
        deploy_webservice "ce-soap" "$SOURCEFORGE_HOME/etc/webservices/deploy/integration/CodeSearchListenerDeploy.wsdd"        
        deploy_webservice "ce-soap" "$SOURCEFORGE_HOME/etc/webservices/deploy/integration/OrchestrateSyncListenerDeploy.wsdd" 
        deploy_webservice "ce-soap" "$SOURCEFORGE_HOME/etc/webservices/deploy/iafservice/IAFServiceDeploy.wsdd"

        wait_for_webservice_deployer "ce-soap60"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/cemain/CollabNetSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/filestorage/FileStorageAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/filestorage/SimpleFileStorageAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/discussion/DiscussionAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/docman/DocumentAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/frs/FrsAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/integrationdata/IntegrationDataAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/news/NewsAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/page/PageAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/taskmgr/TaskAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/tracker/TrackerAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/scm/ScmAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/rbac/RbacAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/wiki/WikiAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/categorization/CategorizationAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/pluggable/PluggableAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/planning/PlanningAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/team/TeamAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/monitoring/MonitoringAppSoapDeploy.wsdd"
        deploy_webservice "ce-soap60" "$SOURCEFORGE_HOME/etc/webservices/deploy/soap/tag/TagAppSoapDeploy.wsdd"
        # Create $RUNTIME_DIR/temp/jboss.start which indicates jboss is deployed 
        # completely including webservices. Useful to install 
        # project templates which is dependent on webservices deployment
        touch "$RUNTIME_DIR/temp/jboss.start"
        ;;

*)

	echo "webservices.sh <start>"
	;;
esac

