﻿
from app.scripts import BaseConfigScript
import app.util
import app.preprocessor
import app.options

class ConfigScript(BaseConfigScript):

    def configSnippet(self):
        xmlStr = self.beginContextXML()
        # Generates DataSource for Reports Database
        xmlStr += self.reportsDataSourceXML()
        # Generates DataSource for Teamforge Database
        xmlStr += self.tfDataSourceXML()
        xmlStr += self.endContextXML()

        return xmlStr

    def reportsDataSourceXML(self):
        dType = self.cfg['DATABASE_TYPE']
        dHost = self.cfg['etl/dependency/ctfcore-datamart/jdbc/host']
        dName = self.cfg['REPORTS_DATABASE_NAME']
        dPort = self.cfg['etl/dependency/ctfcore-datamart/jdbc/port']
        dSSL = self.cfg['REPORTS_DATABASE_SSL']

        if dType == 'postgresql':
            dUrl = "jdbc:postgresql://%s:%s/%s" % (dHost, dPort, dName)
            if dSSL == "on":
                dUrl += '?ssl=true&amp;sslfactory=org.postgresql.ssl.NonValidatingFactory&amp;'
        elif dType == 'oracle':
            dUrl = "jdbc:oracle:thin:@%s:%s:%s" % (dHost, dPort, dName)


        databasePassword = self.cfg['REPORTS_DATABASE_PASSWORD']
        databasePassword =app.options.Obfuscator.createFromRuntimeOptions(self.cfg) \
            .obfuscate(self.cfg['REPORTS_DATABASE_PASSWORD'])

        return """
    <Resource name="jdbc/ReportsDS" auth="Container"
          type="javax.sql.DataSource" driverClassName="%s"
          url="%s" factory="com.vasoftware.sf.system.utils.TeamforgeTomcatEncryptedFactory"
          username="%s" password="%s"
          maxActive="%s" maxIdle="%s"
          initialSize="0" minIdle="%s"
          testOnBorrow="%s"
          validationQuery="%s"/>
""" %(self.cfg['JDBC_DRIVER_CLASS'],
      dUrl,
      self.cfg['REPORTS_DATABASE_USERNAME'], databasePassword,
      self.cfg['REPORTS_DATABASE_MAX_POOL_SIZE'], self.cfg['REPORTS_DATABASE_MAX_IDLE_SIZE'],
      self.cfg['REPORTS_DATABASE_INITIAL_POOL_SIZE'],
      self.cfg['REPORTS_DATABASE_TEST_ON_BORROW'],
      self.cfg['REPORTS_VALIDATION_QUERY'])

    def tfDataSourceXML(self):
        dType = self.cfg['DATABASE_TYPE']
        dSSL = self.cfg['DATABASE_SSL']

        if app.landscape.Landscape.getSite().getMachinesByService("ctfcore-database-mirror"):
            # Use mirror for accessing operational data.
            dHost = self.cfg['etl/dependency/ctfcore-database-mirror/jdbc/host']
            dPort = self.cfg['etl/dependency/ctfcore-database-mirror/jdbc/port']
            databaseUsername = self.cfg['MIRROR_DATABASE_USERNAME']
            databasePassword = self.cfg['MIRROR_DATABASE_PASSWORD']
            dName = self.cfg['MIRROR_DATABASE_NAME']

        else:
            # Use actual operational database.
            dHost = self.cfg['etl/dependency/ctfcore-database/jdbc/host']
            dPort = self.cfg['etl/dependency/ctfcore-database/jdbc/port']
            databaseUsername = self.cfg['DATABASE_USERNAME']
            databasePassword = self.cfg['DATABASE_PASSWORD']
            dName = self.cfg['DATABASE_NAME']

        if dType == 'postgresql':
            databaseUrl = "jdbc:postgresql://%s:%s/%s" % (dHost, dPort, dName)
            if dSSL == "on":
                databaseUrl += '?ssl=true&amp;sslfactory=org.postgresql.ssl.NonValidatingFactory&amp;'
        elif dType == 'oracle':
            databaseUrl = "jdbc:oracle:thin:@%s:%s:%s" % (dHost, dPort, dName)

        # Obfuscate password if necessary.
        databasePassword = app.options.Obfuscator.createFromRuntimeOptions(self.cfg).obfuscate(databasePassword)

        return """
    <Resource name="jdbc/CtfDS" auth="Container"
          type="javax.sql.DataSource" driverClassName="%s"
          url="%s" factory="com.vasoftware.sf.system.utils.TeamforgeTomcatEncryptedFactory"
          username="%s" password="%s"
          maxActive="%s" maxIdle="%s"
          initialSize="0" minIdle="%s"
          testOnBorrow="%s"
          validationQuery="%s"/>
""" %(self.cfg['JDBC_DRIVER_CLASS'],
      databaseUrl,
      databaseUsername, databasePassword,
      self.cfg['REPORTS_CTFDB_MAX_POOL_SIZE'], self.cfg['REPORTS_CTFDB_MAX_IDLE_SIZE'],
      self.cfg['REPORTS_CTFDB_INITIAL_POOL_SIZE'],
      self.cfg['REPORTS_CTFDB_TEST_ON_BORROW'],
      self.cfg['REPORTS_VALIDATION_QUERY'])

    def beginContextXML(self):
        return """<?xml version='1.0' encoding='utf-8'?>
<!--
  Licensed to the Apache Software Foundation (ASF) under one or more
  contributor license agreements.  See the NOTICE file distributed with
  this work for additional information regarding copyright ownership.
  The ASF licenses this file to You under the Apache License, Version 2.0
  (the "License"); you may not use this file except in compliance with
  the License.  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
-->
<!-- The contents of this file will be loaded for each web application -->
<Context privileged="true" reloadable="true">

    <!-- Default set of monitored resources -->
    <WatchedResource>WEB-INF/web.xml</WatchedResource>

    <!-- Uncomment this to disable session persistence across Tomcat restarts -->
    <!--
    <Manager pathname="" />
    -->

    <!-- Uncomment this to enable Comet connection tacking (provides events
         on session expiration as well as webapp lifecycle) -->
    <!--
    <Valve className="org.apache.catalina.valves.CometConnectionManagerValve" />
    -->
"""
    def endContextXML(self):
        return """
</Context>
"""
