﻿
import os
import sys

from app.scripts import BaseInitializationScript
import app.util
import app.logutil
import app.db

class InitializationScript(BaseInitializationScript):

    scriptDescription = "Initialize ETL Job Schedules"

    def __countIncrementalJobRuns(self, datamart):
        countIncrementalRunsResult = datamart.query("select count(*) from etl_attributes where attribute_name = 'ALLOW_TRACKER_INCREMENTAL_RUN' and attribute_value = 'false'")
        if type(countIncrementalRunsResult) is list:
            return int(countIncrementalRunsResult[0][0])
        else:
            return 0

    def __countFlexFieldJobRuns(self, datamart):
        countFlexFieldJobsResult = datamart.query("select count(1) from etl_job where job_name = 'load_flex_fields' and status = 1")
        return int(countFlexFieldJobsResult[0][0])

    def __getCurrentSchemaVersion(self, database):
        schemaVersionResult = database.query("select major || '.' || minor || '.' || sp || '.' || hotfix from schema_version")
        return str(schemaVersionResult[0][0])

    def __getSchemaVersionWhenInitialJobRan(self, datamart):
        schemaVersionWhenInitialJobRanResult = datamart.query("select ctf_version from etl_job where job_name = 'tracker_initial_etl'")
        return str(schemaVersionWhenInitialJobRanResult[0][0])

    def __getInitialVersionOfSite(self, database):
        initialVersionResult = database.query("select from_version from migration_log where migration_date = (select min(migration_date) from migration_log)")
        if len(initialVersionResult) > 0:
            return str(initialVersionResult[0][0])
        else:
            return None

    def __check_version(self, version):
        if version is not None:
            major, minor = [int(c) for c in version.split('.')[0:2]]
            return major < 8 or (major == 8 and minor <=1)
        else:
            return None

    def initialize(self):
        logger = app.logutil.getRuntimeLogger()

        with app.db.Database.connect(
            self.cfg.get('DATABASE_TYPE'),
            self.cfg.get('etl/dependency/ctfcore-database/jdbc/host'),
            self.cfg.get('etl/dependency/ctfcore-database/jdbc/port'),
            self.cfg.get('DATABASE_NAME'),
            self.cfg.get('DATABASE_USERNAME'),
            self.cfg.get('DATABASE_PASSWORD')) as database:


            with app.db.Database.connect(
                self.cfg.get('DATABASE_TYPE'),
                self.cfg.get('etl/dependency/ctfcore-datamart/jdbc/host'),
                self.cfg.get('etl/dependency/ctfcore-datamart/jdbc/port'),
                self.cfg.get('REPORTS_DATABASE_NAME'),
                self.cfg.get('REPORTS_DATABASE_USERNAME'),
                self.cfg.get('REPORTS_DATABASE_PASSWORD')) as datamart:

                try:
                    incrementalJobCount = self.__countIncrementalJobRuns(datamart)
                    flexFieldJobCount = self.__countFlexFieldJobRuns(datamart)

                    etlJobs = []

                    if incrementalJobCount != 0 :
                        etlJobs = ['TrackerInitialJob','SCMCommitInitialJob']

                    elif incrementalJobCount == 0 and flexFieldJobCount == 0:
                        if self.__getSchemaVersionWhenInitialJobRan(datamart) != self.__getCurrentSchemaVersion(database):
                            initialVersion = self.__getInitialVersionOfSite(database)
                            if self.__check_version(initialVersion) is not None and self.__check_version(initialVersion):
                               etlJobs = ["LoadFlexFields"]

                    etlClientScript = os.path.join(self.cfg.get('RUNTIME_SCRIPTS'), "etl-client.py")

                    for etlJob in etlJobs:
                        logger.info("Running ETL job '%s'" % etlJob)
                        etlJobCmd = "%s -r %s"%(etlClientScript, etlJob)
                        app.util.runCommand(etlJobCmd, logger)
                except:
                    self.warn("Connecting to DB succeeded, but accessing tables failed: %r" % sys.exc_info())
                    self.logMesg("warn", "Initializing ETL schedule failed, possibly because database schema has not"+
                        "\n  been initialized yet. Run 'teamforge reinitialize -s etl' at a later point in time")
