﻿import os
import app.servicecontrol
import app.defaults
import app.util
import app.health
import app.threaddump

class ServiceController(app.servicecontrol.SimpleDaemonController):

    def getPidFile(self):
        return os.path.join(app.defaults.DEFAULT_RUN_DIR, "etl.pid")

    def __getInitScript(self):
        return os.path.join(app.defaults.DEFAULT_SERVICE_DIR, "etl", "bin", "catalina.sh")

    def getCommand(self):
        return "%s run" % self.__getInitScript()

    def stopDaemonAsync(self):
        return os.system("%s stop < /dev/null >> %s 2>&1 &" % (
            self.__getInitScript(),
            self.getServiceLogPath())) == 0

    def printSnapshot(self, label):
        config = app.util.getRuntimeConfiguration()
        etlFile = os.path.join(config['ETL_LOG_DIR'], 'catalina.out')
        etlPid = self.getPid()
        app.threaddump.ThreadDump.dumpJVMthread(etlPid, self.logger, etlFile, "etl", label)
        if etlPid is not None:
            print "etl JVM threads dumped to %s." %etlFile
            print "Find thread dump between lines from log demarked by - %s"%label

    def isHealthy(self):
        #
        # We directly invoke the Tomcat endpoint (not via Apache). This is okay
        # as Apache has its own health check. Also, it avoids trouble with self-
        # signed SSL certificates.
        #
        jbossHttpEndpoint = self.getService().getProvidedEndpoint("http-soap")

        url = "http://%s:%d/etl/health?verbose=true&thorough=false" % \
            (jbossHttpEndpoint.getHost(), jbossHttpEndpoint.getPort())

        return app.health.HealthCheck("ETL", url, self.logger).invoke()
    
    def getLogPaths(self):
        return [os.path.join(app.util.getRuntimeConfiguration().get("ETL_LOG_DIR"), "*")]
