#!/opt/collabnet/teamforge/runtime/bin/wrapper/sh
# chkconfig: 345 99 01
# description: Control script for SourceForge Enterprise

eval "`$RUNTIME_DIR/scripts/options-to-shell-variables.py`"

. $RUNTIME_DIR/conf/set-env.sh

FILESTORAGE_LOCATION="$DATA_DIR/filestorage"
DATE=`date '+%Y-%m-%d.%H:%M:%S'`
DATETIME=`date +%Y%m%d%H%M%S`
MVCOMMAND=`which --skip-alias mv`

SOURCEFORGE_START_DIR="$LOG_DIR"
if [ ! -d "$SOURCEFORGE_START_DIR" ]; then
  echo "SOURCEFORGE_START_DIR is invalid ($SOURCEFORGE_START_DIR)"
  exit 1
fi
cd "$SOURCEFORGE_START_DIR"

JBOSS_HOME="$JBOSS_HOME"
JBOSS_CONSOLE="$APPLICATION_LOG_DIR/jboss_console.log"
JBOSSUS="$APP_USER"
export JBOSS_CONSOLE JBOSS_HOME JBOSSUS JAVA_OPTS
if [ ! -f $JBOSS_CONSOLE ]; then
  touch $JBOSS_CONSOLE
  chown $JBOSSUS $JBOSS_CONSOLE
fi

# If SELINUX is enabled, run in unconfined security context
RUNCON=
if [ -x /usr/sbin/selinuxenabled ]; then
  /usr/sbin/selinuxenabled
  if [ $? -eq 0 ]; then
    semodule -l | grep "tf_jboss" 1>/dev/null 2>&1
    if [ $? -ne 0 ];then
    	RUNCON="/usr/bin/runcon -t unconfined_t --"
    fi
  fi
fi

# Check for $RUNTIME_DIR/temp/jboss.start file which indicates jboss is running
# This file will be created by webservices.sh
# Look for /tmp/jboss.start file and install project templates
if [ -f "$RUNTIME_DIR/temp/jboss.start" ]; then
   rm -f "$RUNTIME_DIR/temp/jboss.start"
fi

case "$1" in
start)
  platform=`echo $PLATFORM | cut -f2,3 -d'-'`
  
  grep -q "RewriteRule ^\\/sf\\$.*\\/503.html [R,L]" $HTTPD_CONF_FILE_PATH || sed -i "/RewriteCond.*jboss.marker.*\\!-f/a\RewriteRule ^\\/sf\\$     \\/503.html [R,L]" $HTTPD_CONF_FILE_PATH
  
  if [ "$platform" == "redhat-7" ]; then
     $HTTPD_SERVICE_CMD reload 
  else
     $HTTPD_SERVICE_CMD graceful 
  fi

  FILESTORAGE_TEMP_DIR="$FILESTORAGE_LOCATION/TEMP"
  if [ -d "$FILESTORAGE_TEMP_DIR" ]; then
    $RUNCON rm -rf "$FILESTORAGE_TEMP_DIR"
  fi

  for ((i=0;$i<30;i=$i+1))
  do
    if ( nc -zv -w2 localhost 8080 || nc -zv -w2 localhost 1098-1099 ) &> /dev/null
    then
      sleep 2
    else
      break
    fi
  done

  if [ "X$USER" != "X$JBOSSUS" ]; then
    $RUNCON su - $JBOSSUS -c "$JBOSS_BIN_DIR/run.sh | $RUNTIME_SCRIPTS/timestamp >> \"$JBOSS_CONSOLE\" 2>&1 &"
    $RUNCON $RUNTIME_SCRIPTS/webservices.sh start
  else
    $JBOSS_BIN_DIR/run.sh | $RUNTIME_SCRIPTS/timestamp >> "$JBOSS_CONSOLE" 2>&1 &
    $RUNCON $RUNTIME_SCRIPTS/webservices.sh start
  fi

  if  grep -q "RewriteRule.*503.html" $HTTPD_CONF_FILE_PATH; then
      sed -i "/RewriteRule ^\\/sf\\$.*\\/503.html.*/d" $HTTPD_CONF_FILE_PATH
  fi

  if [ "$platform" == "redhat-7" ]; then
    $HTTPD_SERVICE_CMD reload 
  else
    $HTTPD_SERVICE_CMD graceful 
  fi

  echo "All Services Started: $DATE"
  ;;

stop)
  unset JAVA_OPTS
  $RUNCON $JBOSS_BIN_DIR/jboss-cli.sh --connect command=:shutdown --controller=127.0.0.1:9990
  echo "Kill signal send to Jboss and James."
  echo "Check the logs for shutdown confirmation."
  echo "All Services stopped:  $DATE"
  # Wait until JBoss (that is listening on port 8080) stops
  KILL_JBOSS=''
  for ((i=0;$i<30;i=$i+1))
  do
    if nc -zv -w2 localhost 8080 &> /dev/null
    then
      sleep 2
    else
      KILL_JBOSS='false'
      break
    fi
  done
  if [ -z $KILL_JBOSS ]
  then
    $RUNCON $JBOSS_BIN_DIR/shutdown.sh -S -h
  else
    echo "Jboss is down"
  fi
# Start of phantomjs cleanup
  # Kill phantomjs daemon if still running after jboss is down
  killall -9 phantomjs 2> /dev/null
  # Cleanup the temporary files created by phantomjs daemon in /tmp/ directory
  rm -rf /tmp/export[0-9]*
# End of phantomjs cleanup
  ;;
restart)
  $0 stop
  $0 start
  ;;
*)
  echo "usage: $0 (start|stop|restart)"
esac
