# -*- coding: utf-8 -*-

from app.scripts import BaseConfigScript
import app.util
import app.preprocessor
import app.defaults
import os
import subprocess
import app.landscape
import sys

class ConfigScript(BaseConfigScript):

    def configSnippet(self):
        reportsEnabled = len(app.landscape.Landscape.getSite().getMachinesByService("ctfcore-datamart")) > 0
        dbPassword = "{__DATABASE_PASSWORD__}"
        dbRdOnlyPassword = "{__DATABASE_READ_ONLY_PASSWORD__}"

        dType = self.cfg['DATABASE_TYPE']
        dHost = self.cfg['jboss/dependency/ctfcore-database/jdbc/host']
        dName = self.cfg['DATABASE_NAME']
        dPort = self.cfg['jboss/dependency/ctfcore-database/jdbc/port']
        dSSL = self.cfg['DATABASE_SSL']

        if dType == 'postgresql':
            dUrl = "jdbc:postgresql://%s:%s/%s" % (dHost, dPort, dName)
            if dSSL == "on":
                dUrl += '?ssl=true&amp;sslfactory=org.postgresql.ssl.NonValidatingFactory&amp;'
        elif dType == 'oracle':
            dUrl = "jdbc:oracle:thin:@%s:%s:%s" % (dHost, dPort, dName)

        # Obfuscate Database passwords using JBOSS Vault if obfuscation is enabled.
        dbPassword = self.generateVaultPassword("SourceForgeDS", self.cfg['DATABASE_PASSWORD'])
        dbRdOnlyPassword = self.generateVaultPassword("SourceForgeReadOnlyDS", self.cfg['DATABASE_READ_ONLY_PASSWORD'])

        subSystem_DS_Begin = """
    <subsystem xmlns="urn:jboss:domain:datasources:2.0">
      <datasources>
"""
        dataSource_Section = """
        <datasource jndi-name="java:/SourceForgeDS"
          pool-name="SourceForgeDSPool" enabled="true" use-java-context="true">
          <connection-url>%s</connection-url>
          <driver>{__DATABASE_TYPE__}</driver>
          <pool>
            <min-pool-size>25</min-pool-size>
            <max-pool-size>{__DATABASE_MAX_POOL_SIZE__}</max-pool-size>
            <prefill>true</prefill>
            <use-strict-min>true</use-strict-min>
          </pool>
          <security>
            <user-name>{__DATABASE_USERNAME__}</user-name>
            <password>%s</password>
          </security>
          <validation>
            <check-valid-connection-sql>{__DATABASE_CONNECTION_VALID_SQL__}</check-valid-connection-sql>
            <background-validation>true</background-validation>
            <background-validation-millis>60000</background-validation-millis>
	  </validation>
           <timeout>
	     <set-tx-query-timeout>true</set-tx-query-timeout>
          </timeout>
        </datasource>
        <datasource jndi-name="java:/SourceForgeReadOnlyDS"
          pool-name="SourceForgeReadOnlyDSPool" enabled="true" use-java-context="true">
          <connection-url>%s</connection-url>

          <driver>{__DATABASE_TYPE__}</driver>
          <pool>
            <min-pool-size>10</min-pool-size>
            <max-pool-size>20</max-pool-size>
            <prefill>true</prefill>
            <use-strict-min>true</use-strict-min>
          </pool>
          <security>
            <user-name>{__DATABASE_READ_ONLY_USER__}</user-name>
            <password>%s</password>
          </security>
	  <validation>
            <check-valid-connection-sql>{__DATABASE_CONNECTION_VALID_SQL__}</check-valid-connection-sql>
            <background-validation>true</background-validation>
            <background-validation-millis>60000</background-validation-millis>
	  </validation>
          </datasource>
""" %(dUrl, dbPassword, dUrl, dbRdOnlyPassword)

        if reportsEnabled:
           rdHost = self.cfg['jboss/dependency/ctfcore-datamart/jdbc/host']
           rdName = self.cfg['REPORTS_DATABASE_NAME']
           rdPort = self.cfg['jboss/dependency/ctfcore-datamart/jdbc/port']
           rdSSL = self.cfg['REPORTS_DATABASE_SSL']

           if dType == 'postgresql':
               rdUrl = "jdbc:postgresql://%s:%s/%s" % (rdHost, rdPort, rdName)
               if rdSSL == "on":
                   rdUrl += '?ssl=true&amp;sslfactory=org.postgresql.ssl.NonValidatingFactory&amp;'
           elif dType == 'oracle':
               rdUrl = "jdbc:oracle:thin:@%s:%s:%s" % (rdHost, rdPort, rdName)

           dbRptPassword = "{__REPORTS_DATABASE_READ_ONLY_PASSWORD__}"

           # Obfuscate Database passwords using JBOSS Vault if obfuscation is enabled.
           dbRptPassword = self.generateVaultPassword("jdbc.ReportsDS", self.cfg['REPORTS_DATABASE_READ_ONLY_PASSWORD'])

           dataSource_Section = dataSource_Section + """
        <datasource jndi-name="java:/jdbc.ReportsDS"
          pool-name="ReportsDSPool" enabled="true" use-java-context="true">
          <connection-url>%s</connection-url>
          <driver>{__DATABASE_TYPE__}</driver>
          <pool>
            <min-pool-size>{__REPORTS_DATABASE_CTF_DATAMART_INITIAL_POOL_SIZE__}</min-pool-size>
            <max-pool-size>{__REPORTS_DATABASE_CTF_DATAMART_MAX_POOL_SIZE__}</max-pool-size>
            <prefill>true</prefill>
            <use-strict-min>true</use-strict-min>
          </pool>
          <security>
            <user-name>{__REPORTS_DATABASE_READ_ONLY_USER__}</user-name>
            <password>%s</password>
          </security>
          <validation>
            <check-valid-connection-sql>{__DATABASE_CONNECTION_VALID_SQL__}</check-valid-connection-sql>
            <background-validation>true</background-validation>
            <background-validation-millis>60000</background-validation-millis>
          </validation>
         </datasource>
""" %(rdUrl, dbRptPassword)

        subSystem_DS_End = """
        <drivers>
          <driver name="postgresql" module="org.postgresql">
            <xa-datasource-class>org.postgresql.xa.PGXADataSource</xa-datasource-class>
          </driver>
          <driver name="oracle" module="com.oracle">
            <xa-datasource-class>oracle.jdbc.OracleDriver</xa-datasource-class>
          </driver>
        </drivers>
      </datasources>
    </subsystem>
"""
        subSystem_DS = subSystem_DS_Begin + dataSource_Section + subSystem_DS_End
        # Disable hot deploy in production environment
        if (self.cfg.get('TEST_SUPPORT_ENABLED') == "true"):
          scanInterval = "5000"
        else:
          scanInterval = "-1"
        subSystem_Scanner = """
	<subsystem xmlns="urn:jboss:domain:deployment-scanner:1.1">
	  <deployment-scanner path="deployments"
		relative-to="jboss.server.base.dir" scan-interval="%s"
		deployment-timeout="600" />
	</subsystem>
"""%(scanInterval)
        configuration = subSystem_DS + subSystem_Scanner
        return configuration

    def generateVaultPassword(self, attrDomainName, attrValue):
        """Generates Obfuscated password for attrValue using JBOSS Vault"""

        vaultKeyStorePath  = os.environ['VAULT_ENC_PATH']
        vaultKeyStoreFile  = os.environ['VAULT_KEYSTORE_FILE']
        vaultKeyStoreAlias = os.environ['VAULT_KEYSTORE_ALIAS']
        vaultKeyStorePass  = os.environ['VAULT_KEYSTORE_FILE_PASSWORD']
        vaultKeyPass       = os.environ['VAULT_KEYSTORE_PASSWORD']
        vaultKeySalt       = os.environ['VAULT_KEYSTORE_SALT']
        vaultScript        = os.path.join(app.defaults.DEFAULT_SERVICE_DIR, "jboss", "bin", "vault.sh")

        runCmd = "%s -k %s -p '%s' -e %s -i 50 -s '%s' -v %s -b %s -a 'password'  -x '%s' -V" \
                   %(vaultScript, vaultKeyStoreFile, vaultKeyStorePass, vaultKeyStorePath, \
                     vaultKeySalt, vaultKeyStoreAlias, attrDomainName, attrValue)
        runProc = subprocess.Popen(runCmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        (stdOut, stdErr) = runProc.communicate()
        exitCode = runProc.returncode

        if (exitCode != 0):
           self.log.error("Unable to obfuscate database password for %s using JBOSS Vault !!"%(attrDomainName))
           self.log.error("Reason: %s"%(stdErr))
           sys.exit(1)

        vaultPassword = stdOut.replace("VAULT_PASSWORD=", '')
         # Constructure Vault Structure here. Ex: ${VAULT-PASSWORD-CONTENT}
        vaultPassword = "${%s}"%(vaultPassword.rstrip())

        return vaultPassword
