﻿
from app.scripts import BaseCreateRuntimeScript
import app.util
import app.batch
import app.preprocessor
import os
import service.util
import service.constants
import string
import random
import commands
import sys

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Setup Jboss standalone-full.xml"

    def createRuntime(self):
        if service.util.isApplicationEnabledLocally(service.constants.APPLICATION_APP):

            # Initialise JBOSS Vault since obfuscation is enabled always
            self.handleVaultInitialization()

            app.batch.ConfigBatch(self.log) \
                .addFolder(os.path.join(app.defaults.DEFAULT_SERVICE_DIR, 'jboss', 'conf-snippets', 'standalone-full.xml.d')) \
                .evaluateIntoFile(os.path.join(self.cfg['JBOSS_SERVER_CONF_DIR'], "standalone-full.xml"))

    def handleVaultInitialization(self):
        """Function to initialse the Vault Key store system"""
        vaultKeyStorePath = os.path.join(self.cfg['RUNTIME_DIR'], "temp", "vault")
        vaultKeyStoreFile = os.path.join(vaultKeyStorePath, "vault.keystore")
        vaultKeyStoreAlias = "vault"

        # Generate Vault Key Store password randomly
        vaultKeyStorePass = self.generateRandomString(10)
        vaultKeyPass      = vaultKeyStorePass
        # Generate 8 digit salt string for Vault Key management randomly
        vaultKeySalt      = self.generateRandomString(8)

        self.removeCreateKeyStoreFS(vaultKeyStorePath)
        self.createValueKeyStore(vaultKeyStoreAlias, vaultKeyStoreFile, vaultKeyStorePass, vaultKeyPass)

        # Setting these values in OS Environment as these will be used by vault script to
        # generated vault based obfuscated string for a given password.
        os.environ['VAULT_ENC_PATH'] = vaultKeyStorePath
        os.environ['VAULT_KEYSTORE_FILE'] = vaultKeyStoreFile
        os.environ['VAULT_KEYSTORE_ALIAS'] = vaultKeyStoreAlias
        os.environ['VAULT_KEYSTORE_FILE_PASSWORD'] = vaultKeyStorePass
        os.environ['VAULT_KEYSTORE_PASSWORD'] = vaultKeyPass
        os.environ['VAULT_KEYSTORE_SALT'] = vaultKeySalt

    def createValueKeyStore(self, aliasName, keyStoreFile, keyStorePass, keyPass):
        """Function that creates Vault Key store"""

        keyToolScript = os.path.join(self.cfg['JAVA_HOME'], 'bin', 'keytool')
        runCmd = "%s -genseckey -alias %s -keystore %s -keyalg AES -keysize 128 -storepass '%s' -keypass '%s' -dname 'CN=TFJBoss' -storetype JCEKS" \
                     %(keyToolScript, aliasName, keyStoreFile, keyStorePass, keyPass)

        (retVal, outputVal) = commands.getstatusoutput(runCmd)
        if (retVal != 0):
           self.logMesg("error", "Unable to generate keystore for JBOSS Vault!!")
           self.logMesg("error", "Reason: %s"%(outputVal))
           sys.exit(1)

        return

    def removeCreateKeyStoreFS(self, keyStorePath):
        """Function that removes and recreates Vault Keystore path"""

        # Create and return if Vault Key store path doesnot exists
        if not (os.path.isdir(keyStorePath)):
           runCmd = "mkdir -p %s"%(keyStorePath)
           commands.getstatusoutput(runCmd)
           return

        # If Vault Key store path exists then remove all files in it
        for ksFile in (os.listdir(keyStorePath)):
            ksFileAbsPath = os.path.join(keyStorePath, ksFile)
            os.unlink(ksFileAbsPath)

        return

    def generateRandomString(self, strLength):
         """Function for generating a Random string"""

         passwdChars = string.digits + string.letters + string.digits
         totalLength = len(passwdChars)-1;
         randPasswd = ""

         for i in range(0,strLength):
             rndNum = random.randint(0, totalLength)
             randPasswd += passwdChars[rndNum]

         return randPasswd

