__author__ = "Barathi Sukumar <barathi@collab.net>"

from app.scripts import BaseCreateRuntimeScript
from app.selinux_permissions import SELinuxSetting
import os
import sys

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Setup SELinux settings"

    def createRuntime(self):
        if self.cfg['SELINUX_ENABLED'] == "true":
            selinuxsetting = SELinuxSetting()
            pythonWrapper = self.cfg["PYTHON_WRAPPER"]
            distDir = self.cfg["DISTRIBUTION_DIR"]
            httpdLogDir = self.cfg["HTTPD_LOG_DIR"]
            sourceforgeHome = self.cfg["SOURCEFORGE_HOME"]
            integrationDir = os.path.join(sourceforgeHome, 'integration')
            avatarDir = os.path.join(sourceforgeHome, 'avatar')
            restDir = os.path.join(sourceforgeHome, 'svn-rest')
            LogDir = self.cfg["LOG_DIR"]
            site_dir = self.cfg["SITE_DIR"]
            runtime_dir = self.cfg["RUNTIME_DIR"]
            install_dir = self.cfg["SRC_DIR"]

            # Soap client SUDS cache location
            sudsCacheDir = os.path.join('%s' %self.cfg["TEMPORARY_DIR"],'suds')
            semanageEquDir = ''
            if os.path.islink(site_dir):
                semanageEquDir = site_dir
            if os.path.islink("/opt/collabnet"):
                semanageEquDir = "/opt/collabnet"
            if os.path.islink ("/opt"):
                self.logMesg("error", "/opt as symbolic link is not supported. Symlink /opt/collabnet or /opt/collabnet/teamforge instead")
                sys.exit(1)
            if semanageEquDir != '':
               selinuxsetting.changeContext ('usr_t',semanageEquDir,'No')

            # Create httpd log directory
            selinuxsetting.createDir(httpdLogDir, self.cfg['HTTPD_USER'], self.cfg['HTTPD_GROUP'])
            selinuxsetting.changeContext('httpd_log_t', httpdLogDir)
            # Create httpd wsgi socket file directory
            wsgiSockDir = self.cfg["HTTPD_WSGI_SOCKET_DIR"]
            selinuxsetting.createDir(wsgiSockDir, self.cfg['HTTPD_USER'], self.cfg['HTTPD_GROUP'])
            selinuxsetting.changeContext('httpd_var_run_t', wsgiSockDir)

            # Create others/publish log directory
            publishLogDir = os.path.join(LogDir, 'others', 'publish')
            selinuxsetting.createDir(publishLogDir, self.cfg['HTTPD_USER'], self.cfg['HTTPD_GROUP'])

            httpdContextDirs = [pythonWrapper, integrationDir, publishLogDir,
                                   avatarDir, restDir, sudsCacheDir]
            for dir in httpdContextDirs:
                selinuxsetting.changeContext('httpd_sys_content_t', dir)

            cvsshFile = os.path.join(sourceforgeHome, 'bin/Linux/cvssh')
            selinuxsetting.changeContext('usr_t', cvsshFile)

            # Change label to lib_t for mod_authnz_ctf.so. This is required
            # for same box upgrade from TF 6.2 in RHEL 6.3 . By default,
            # the label is lib_t in RHEL6.4
            authnz_path = os.path.join(distDir, 'multiplatform','collabnet-core')
            selinuxsetting.changeContext('httpd_modules_t', authnz_path)

            # Change type to var_log_t for runtime and bootstrap,install log to avoid avc deny message in audit log"
            runtimeLog = os.path.join(LogDir,'runtime','runtime.log')
            bootstrapLog = os.path.join(LogDir,'runtime','bootstrap.log')
            installLog = os.path.join(install_dir,'install.log')
            serviceLog = os.path.join(LogDir,'runtime','service.log')
            apps_serviceLog = os.path.join(LogDir,'apps','service.log')


            runtimeConfDir = os.path.join(runtime_dir,'conf')
            # Set selinux context for runtime conf dir only in case of symlinks
            if os.path.exists(runtimeConfDir):
                reallocation = os.path.realpath(runtimeConfDir)
                if runtimeConfDir != reallocation:
                    selinuxsetting.changeContext('httpd_sys_content_t', runtimeConfDir)

            # Set cert_t context for ssl files
            if self.cfg['SSL'] == "on":
                sslCertFile = self.cfg["SSL_CERT_FILE"]
                sslKeyFile = self.cfg["SSL_KEY_FILE"]
                sslChainFile = ''
                sslCaCertFile = ''
                if self.cfg.has_key("SSL_CHAIN_FILE"):
                    sslChainFile = self.cfg["SSL_CHAIN_FILE"]
                if self.cfg.has_key("SSL_CA_CERT_FILE"):
                    sslCaCertFile = self.cfg["SSL_CA_CERT_FILE"]
                if os.path.isfile(sslCertFile) and os.path.isfile(sslKeyFile):
                    selinuxsetting.changeContext('cert_t', sslCertFile)
                    selinuxsetting.changeContext('cert_t', sslKeyFile)
                if os.path.isfile(sslChainFile):
                    selinuxsetting.changeContext('cert_t', sslChainFile)
                if os.path.isfile(sslCaCertFile):
                    selinuxsetting.changeContext('cert_t', sslCaCertFile)

            # Set booleans
            booleanList = ['httpd_can_network_connect', 'httpd_use_nfs', 'httpd_unified', 'httpd_execmem']
            for bool in booleanList:
                outputVal = selinuxsetting.checkBooleanOn ( bool )
                if outputVal == "off":
                   selinuxsetting.changeBoolean('--on', bool)

