
from app.scripts import BaseOptionScript
import os

class OptionScript(BaseOptionScript):

    def optionSnippet(self):
        pgsql_version = self.cfg["PGSQL_VERSION"]
        dataDir = self.cfg["DATA_DIR"]
        if "redhat-7" in self.cfg['PLATFORM']:
            short_ver = pgsql_version.replace(".","")
            initScript = "/usr/pgsql-%s/bin/postgresql%s-setup" %(pgsql_version, short_ver)
            homeDir = "%s/pgsql/%s" % (dataDir, pgsql_version)
            tfSystemd = os.path.join(self.cfg["RUNTIME_DIR"], 'scripts', 'tfsystemd')
            serviceCmd = '%s ctf-postgresql-%s' %(tfSystemd, pgsql_version)
        else:
            initScript = '/etc/init.d/postgresql-%s' %(pgsql_version)
            homeDir = "%s/pgsql/%s" % (dataDir, pgsql_version)
            serviceCmd = initScript

        # Check if DATABASE_USERNAME and DATABASE_READ_ONLY_USER have the same value
        if self.cfg['DATABASE_USERNAME'] == self.cfg['DATABASE_READ_ONLY_USER']:
            raise Exception("DATABASE_USERNAME and DATABASE_READ_ONLY_USER cannot have same value")

        passFile = os.path.join(self.cfg['TEMPORARY_DIR'], 'pgpass')

        dType = self.cfg['DATABASE_TYPE']
        dName = self.cfg['DATABASE_NAME']
        distDir = self.cfg['DISTRIBUTION_DIR']

        if dType == 'postgresql':
            jdbcClass = "org.postgresql.Driver"
            bootstrapJar = os.path.join(distDir, 'bootstrap', 'lib',
                'dbinstall-postgresql.jar')
            return "\n".join(["JDBC_DRIVER_CLASS=%s" % jdbcClass,
                    "DATABASE_CONNECTION_VALID_SQL=select 1;",
                    "BOOTSTRAP_JAR=%s" % bootstrapJar,
                    "PGSQL_INIT_SCRIPT=%s" % initScript,
                    "PGSQL_SERVICE_CMD=%s" % serviceCmd,
                    "PGPASSFILE=%s" % passFile,
                    "PGSQL_HOME_DIR=%s" % homeDir])
        elif dType == 'oracle':
            jdbcClass = "oracle.jdbc.driver.OracleDriver"
            bootstrapJar = os.path.join(distDir, 'bootstrap', 'lib',
                'dbinstall-oracle.jar')
            if self.cfg.has_key('ORACLE_HOME'):
                oracleHome = self.cfg['ORACLE_HOME']
            else:
                oracleHome = '/usr/lib/oracle/12.1'
                if os.uname()[4] == 'x86_64':
                    oracleHome = "%s/%s/" % (oracleHome, 'client64')
                else:
                    oracleHome = "%s/%s/" % (oracleHome, 'client')
            if self.cfg.has_key('DATABASE_SERVICE_NAME'):
                dbServiceName = self.cfg['DATABASE_SERVICE_NAME']
            else:
                dbServiceName = dName
            return "\n".join([
                    "JDBC_DRIVER_CLASS=%s" % jdbcClass,
                    "DATABASE_CONNECTION_VALID_SQL=select 1 from dual",
                    "BOOTSTRAP_JAR=%s" % bootstrapJar,
                    "DATABASE_SERVICE_NAME=%s" % dbServiceName,
                    "ORACLE_HOME=%s" % oracleHome,
                    "PGSQL_INIT_SCRIPT=%s" % initScript,
                    "PGSQL_SERVICE_CMD=%s" % serviceCmd,
                    "PGPASSFILE=%s" % passFile,
                    "PGSQL_HOME_DIR=%s" % homeDir])
        else:
            raise Exception("DATABASE_TYPE=%s is unsupported" % dType)
