
from app.scripts import BaseOptionScript
import os
import service.util
import service.constants

class OptionScript(BaseOptionScript):

    def optionSnippet(self):
        passFile = os.path.join(self.cfg['TEMPORARY_DIR'], 'pgpass-reporting')
        if "redhat-7" in self.cfg["PLATFORM"]:
            tfSystemd = os.path.join(self.cfg["RUNTIME_DIR"], 'scripts', 'tfsystemd')
            initCommand = '%s reporting-postgresql-%s' %(tfSystemd, self.cfg["PGSQL_VERSION"])
        else:
            initCommand = '/etc/init.d/reporting-postgresql-%s' %(self.cfg["PGSQL_VERSION"])

        # Check if REPORTS_DATABASE_USERNAME and REPORTS_DATABASE_READ_ONLY_USER have the same value
        if self.cfg['REPORTS_DATABASE_USERNAME'] == self.cfg['REPORTS_DATABASE_READ_ONLY_USER']:
            raise Exception("REPORTS_DATABASE_USERNAME and REPORTS_DATABASE_READ_ONLY_USER cannot have same value")

        if (len(service.util.getApplicationHosts(service.constants.APPLICATION_DATABASE)) > 0 and
                service.util.getApplicationHost(service.constants.APPLICATION_REPORTS_DATABASE) is not None):
            # Check for missing tokens
            missing = []
            for token in ['REPORTS_DATABASE_USERNAME','REPORTS_DATABASE_PASSWORD','REPORTS_DATABASE_NAME']:
                if not self.cfg.has_key(token):
                    missing.append(token)
            if len(missing) > 0:
                raise Exception("Missing the following database parameters in site-options.conf: %s." % ", ".join(missing))

            dPassword = self.cfg['REPORTS_DATABASE_PASSWORD']
            if dPassword is None or len(dPassword) == 0:
                raise Exception("REPORTS_DATABASE_PASSWORD must be specified and must be a non-empty string.")

            dType = self.cfg['DATABASE_TYPE']
            dName = self.cfg['REPORTS_DATABASE_NAME']
            distDir = self.cfg['DISTRIBUTION_DIR']
            if dType == 'postgresql':
                jdbcClass = "org.postgresql.Driver"
                bootstrapJar = os.path.join(distDir, 'bootstrap', 'lib',
                    'dbinstall-reports-postgresql.jar')
                return "\n".join(["JDBC_DRIVER_CLASS=%s" % jdbcClass,
                        "DATABASE_CONNECTION_VALID_SQL=select 1;",
                        "REPORTS_BOOTSTRAP_JAR=%s" % bootstrapJar,
                        "REPORTS_PGSQL_SERVICE_CMD=%s" % initCommand,
                        "REPORTS_PGPASSFILE=%s" % passFile])
            elif dType == 'oracle':
                jdbcClass = "oracle.jdbc.driver.OracleDriver"
                bootstrapJar = os.path.join(distDir, 'bootstrap', 'lib',
                    'dbinstall-reports-oracle.jar')
                return "\n".join(["JDBC_DRIVER_CLASS=%s" % jdbcClass,
                        "DATABASE_CONNECTION_VALID_SQL=select 1 from dual",
                        "REPORTS_BOOTSTRAP_JAR=%s" % bootstrapJar,
                        "REPORTS_PGSQL_SERVICE_CMD=%s" % initCommand,
                        "REPORTS_PGPASSFILE=%s" % passFile])
            else:
                raise Exception("DATABASE_TYPE=%s is unsupported" % dType)
        else:
            return "\n".join(["REPORTS_PGSQL_SERVICE_CMD=%s" % initCommand,
                              "REPORTS_PGPASSFILE=%s" % passFile])
