#!/usr/bin/env python

import commands
import sys
import pgdb
import app.util
import time
import os

cfg = app.util.getRuntimeConfiguration()
logFile = os.path.join(cfg['RUNTIME_LOG_DIR'], "domain_change_rb.log")

#The location of the review board conf file.
rb_props = '/etc/reviewboard.properties'

def usage():
    print "Usage: domain_change_db.py <old_domain> <new_domain>"
    sys.exit(1)

def printLog (logMessage):
    """Logs the output to the log file in append mode. Also prints the output to stdout"""

    global logFile

    timeStr = time.ctime(time.time())
    logOutput = "%s: %s"%(timeStr, logMessage)

    wFH = open(logFile, 'a')
    wFH.write("%s\n"% logOutput)
    wFH.close()

    print("%s"% logOutput)


if len(sys.argv) < 3:
    usage()

# Retrieve the domains from the caller script
old_domain = sys.argv[1]
new_domain = sys.argv[2]
regexp = "E'(^|\\\\s|\\\\W)%s', E'\\\\1%s', 'g'" % (old_domain, new_domain)
cursor = None

def get_value(token):
    status, output = commands.getstatusoutput("grep ^%s= '%s'" % (token, rb_props))
    if status != 0:
        raise Exception("%s token not found in %s: %s" % (token, rb_props, output))

    return output.split('=')[1]

def execute_update(table, column):
    update_sql = "update %s set %s = regexp_replace(%s, %s) where position('%s' in %s) > 0" % (table, column, column, regexp, old_domain, column)
    cursor.execute(update_sql)
    printLog ("Completed updating [%s], column [%s]. Total records modified: %s"%(table, column, cursor.rowcount))

def main():
    """This script replaces old domain with new domain in the Review Board database tables."""

    obfuscateUtil = os.path.join(cfg['RUNTIME_SCRIPTS'],'obfuscate_util.sh')

    rbdbuser = get_value('rb_database_user')
    rbdbpwd = commands.getoutput("%s -decrypt %s"% (obfuscateUtil, get_value('rb_database_password')))

    rbdbname = get_value('rb_database_name')
    rbdbhost = get_value('rb_database_host')
    rbdbport = get_value('rb_database_port')

    con = pgdb.connect(user = rbdbuser, password = rbdbpwd, database = rbdbname,
                       host = "%s:%s" % (rbdbhost, rbdbport))
    global cursor
    cursor = con.cursor()

    printLog ("============ Begin Domain Change in Reviewboard Database ============")
    execute_update('django_site', 'domain')
    execute_update('django_site', 'name')
    execute_update('scmtools_repository', 'path')
    execute_update('scmtools_repository', 'bug_tracker')
    execute_update('auth_user', 'email')
    execute_update('siteconfig_siteconfiguration', 'settings')
    execute_update('changedescs_changedescription', 'text')
    execute_update('changedescs_changedescription', 'fields_changed')
    execute_update('reviews_comment', 'text')
    execute_update('reviews_review', 'body_top')
    execute_update('reviews_review', 'body_bottom')
    execute_update('reviews_reviewrequest', 'summary')
    execute_update('reviews_reviewrequest', 'description')
    execute_update('reviews_reviewrequest', 'testing_done')
    execute_update('reviews_reviewrequest', 'bugs_closed')
    execute_update('reviews_reviewrequest', 'branch')
    execute_update('reviews_reviewrequestdraft', 'summary')
    execute_update('reviews_reviewrequestdraft', 'description')
    execute_update('reviews_reviewrequestdraft', 'testing_done')
    execute_update('reviews_reviewrequestdraft', 'bugs_closed')
    execute_update('reviews_reviewrequestdraft', 'branch')

    con.commit()
    cursor.close()
    con.close()

    printLog ("============ End Domain Change in Reviewboard Database ============")

if __name__ == '__main__':
    main()
