import telnetlib
import time
import socket

from app.scripts import BaseInitializationScript

class JamesAdminClient(object):
    READ_TIMEOUT = 2 # seconds

    def __init__(self, host, port):
        try:
            self.__connection = telnetlib.Telnet(host, port)
        except socket.error:
            raise ConnectionRefusedException()

    def login(self, adminUsername, adminPassword):
        self.__connection.read_until("Login id:\n", JamesAdminClient.READ_TIMEOUT)
        self.__connection.write(adminUsername + "\n")

        self.__connection.read_until("Password:", JamesAdminClient.READ_TIMEOUT)
        self.__connection.write(adminPassword + "\n")

        # If the login was successful, we should be greeted.
        while True:
            result = self.__connection.read_until("\n", JamesAdminClient.READ_TIMEOUT)
            if result.startswith("Welcome "):
                return True
            elif result.startswith("Login failed "):
                return False
            elif len(result.strip()) == 0:
                pass
            else:
                raise Exception("Received unrecognized response " + result)

    def addUser(self, username, password):
        self.__connection.write("adduser %s %s\n" % (username, password))

        while True:
            result = self.__connection.read_until("\n", JamesAdminClient.READ_TIMEOUT)
            if result.startswith("User %s added" % username):
                return True
            elif result.startswith("User %s already exists" % username):
                # More to come...
                pass
            elif result.startswith("Error adding user %s" % username):
                return False
            elif len(result.strip()) == 0:
                pass
            else:
                raise Exception("Received unrecognized response " + result)

    def setPassword(self, username, password):
        self.__connection.write("setpassword %s %s\n" % (username, password))

        while True:
            result = self.__connection.read_until("\n", JamesAdminClient.READ_TIMEOUT)
            if result.startswith("Password for %s reset" % username):
                return
            elif len(result.strip()) == 0:
                pass
            else:
                raise Exception("Received unrecognized response " + result)

class ConnectionRefusedException(Exception):
    pass

class InitializationScript(BaseInitializationScript):
    scriptDescription = "Configure user for email relay"

    def __connect(self):
        for i in range(1, 10):
            try:
                return JamesAdminClient("127.0.0.1", 4555)
            except ConnectionRefusedException:
                # James is often slow to initialize, so give it a little time.
                self.log.info("Connection refused, will try again...")
                time.sleep(2)

        raise ConnectionRefusedException("Failed to connect to James management port")

    def initialize(self):
        cli = self.__connect()

        self.log.info("Adding relay user to James...")
        if not cli.login(self.cfg["JAMES_ADMIN_USER"], self.cfg["JAMES_ADMIN_PASSWORD"]):
            raise Exception("Failed authenticate at James admin interface - please check JAMES_ADMIN_PASSWORD")

        if cli.addUser(self.cfg["JAMES_RELAY_USER"], self.cfg["JAMES_RELAY_PASSWORD"]):
            self.log.info("Relay user %s added" % self.cfg["JAMES_RELAY_USER"])

        else:
            self.log.info("Relay user %s exists, updating password..." % self.cfg["JAMES_RELAY_USER"])

            # User exists already, so just update its password.
            cli.setPassword(self.cfg["JAMES_RELAY_USER"], self.cfg["JAMES_RELAY_PASSWORD"])
