
from app.scripts import BaseCreateRuntimeScript
import app.util
import app.batch
import app.preprocessor
import os
from app.linuxservice import ServiceManager

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Create Reports PostgreSQL control script"

    def createRuntime(self):
        version = self.cfg["PGSQL_VERSION"]
        platform = self.cfg['PLATFORM']

        initScript = os.path.join(self.cfg['RUNTIME_SCRIPTS'], 'reporting-postgresql')

        app.util.getTokenProcessor(self.cfg).copyWithSubstitution(
            os.path.join(app.defaults.DEFAULT_SERVICE_DIR, 'postgres-datamart', 'code-templates', 'reporting-postgresql-%s.tmpl' % version),
            initScript)

        # This instance is exclusively used for the datamart.

        app.util.runSystem("chmod 755 %s " % initScript, self.log)

        if "redhat-7" in platform:
            # Cleanup old junk
            initPgScriptFile = '/etc/init.d/reporting-postgresql-%s' % version
            if os.path.islink(initPgScriptFile) or os.path.isfile(initPgScriptFile):
                app.util.runSystem("rm -f %s" % initPgScriptFile, self.log)
        else:
            # Expose init script to sysv
            sysvInitFile = self.cfg['REPORTS_PGSQL_SERVICE_CMD'].split(" ")[0]
            if os.path.isfile(sysvInitFile) or os.path.islink(sysvInitFile):
                app.util.runSystem("rm -f %s" % sysvInitFile, self.log)

            app.util.runSystem('ln -s %s %s' % (initScript, sysvInitFile), self.log)

        pgVersion = self.cfg["PGSQL_VERSION"]

        # Stop and disable default postgresql service from starting on boot
        ServiceManager.getService("postgresql-%s" % pgVersion).stopAndDisable(False)

        # Disable old custom postgresql script not to start automatically at boot
        ServiceManager.getService("reporting-postgresql-%s" % pgVersion).stopAndDisable(False)
