﻿
from app.scripts import BaseCreateRuntimeScript
import app.certstore
import app.defaults
import shutil
import os

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Autogenerate/copy SSL certificates"

    def __isSSLEnabled(self):
        return self.cfg.get("REPORTS_DATABASE_SSL", "off") == "on"

    def createRuntime(self):
        if not self.__isSSLEnabled():
            return

        # Postgres is talked to via the internal network, so use the internal FQDN
        # for the certificate.
        domainName = app.landscape.Landscape.getSite().getLocalMachine().getInternalAddress().getName()

        targetCertPath = os.path.join(self.cfg['REPORTS_PGSQL_DATA_DIR'], "server.crt")
        targetKeyPath  = os.path.join(self.cfg['REPORTS_PGSQL_DATA_DIR'], "server.key")

        userCertPath = self.cfg.get("POSTGRES_DATAMART_SSL_CERT_FILE")
        userKeyPath  = self.cfg.get("POSTGRES_DATAMART_SSL_KEY_FILE")

        if not (userCertPath and os.path.isfile(userCertPath) and \
                userKeyPath and os.path.isfile(userKeyPath)):
            self.log.info("Generating self-signed SSL certiticates for '%s'" % domainName)
            app.certstore.CertificateGenerator.createSelfSignedCertificate(
                domainName,
                app.defaults.DEFAULT_CERTIFICATE_VALIDITY_PERIOD,
                targetCertPath,
                targetKeyPath)

        else:
            self.log.info("Using user-provided SSL certiticates for '%s'" % domainName)
            shutil.copy(userCertPath, targetCertPath)
            shutil.copy(userKeyPath, targetKeyPath)

        self.__setFilePermission(targetCertPath)
        self.__setFilePermission(targetKeyPath)


    def __setFilePermission(self, certFile):
        os.system("chown postgres:postgres %s" %certFile)
        os.chmod(certFile, 0600)
