from app.scripts import BaseCreateRuntimeScript
from app.pgdbadmin import PGCluster
from app.selinuxutil import Policy
import app.defaults
import os

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Check and Initialise TeamForge Database"

    def createRuntime(self):
       pgDataDir = self.cfg['PGSQL_DATA_DIR']
       pgHomeDir = self.cfg['PGSQL_HOME_DIR']

       if os.path.exists(app.defaults.OLD_PGCLUSTER):
           oldPGDataDir = os.path.join(app.defaults.OLD_PGCLUSTER, "data")
       else:
           oldPGDataDir = os.path.join("/var/lib/pgsql/",app.defaults.OLD_PGVERSION,"data")

       initCommand = "%s/postgresql" % self.cfg['RUNTIME_SCRIPTS']

       # Initialising and Migrate TeamForge Database
       PGCluster.doInitializeandMigrate(self.log, pgHomeDir, oldPGDataDir, pgDataDir, initCommand)

       # Applying selinux permissions
       postgresDaemonBinary = os.path.join("/usr", "pgsql-%s" % self.cfg["PGSQL_VERSION"], "bin", "postmaster")
       Policy.applySELinuxPermissions(postgresDaemonBinary, recursive=False)

       pgsqlHomeDir = os.path.split(self.cfg['PGSQL_HOME_DIR'])[0]
       if not Policy.isSELinuxPermissionsApplied(pgsqlHomeDir):
           Policy.applySELinuxPermissions(pgsqlHomeDir) 

       if not Policy.isSELinuxPermissionsApplied(self.cfg['PGSQL_LOG_DIR']):
           Policy.applySELinuxPermissions(self.cfg['PGSQL_LOG_DIR'])
