
from app.scripts import BaseCreateRuntimeScript
import app.util
import app.batch
import app.landscape
import os
from app.colors import Color

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Setup PostgreSQL pg_hba.conf"

    def createRuntime(self):
        confFile = os.path.join(self.cfg['PGSQL_DATA_DIR'], 'pg_hba.conf')
        if os.path.exists(confFile):
            batch = app.batch.ConfigBatch(self.log)

            # Add overrides
            for service in app.landscape.Landscape.getSite().getLocalMachine().getServices():
                overridesFolder = os.path.join(
                    app.defaults.DEFAULT_SERVICE_DIR,
                    service.getName(),
                    'overrides',
                    'postgres',
                    'pg_hba.conf.d')

                if os.path.isdir(overridesFolder):
                    batch = batch.addFolder(overridesFolder)

            batch.evaluateIntoFile(confFile)

            pghbaIncludeDir = os.path.join(self.cfg['ETC_DIR'], 'conf.d','pgsql','pg_hba.conf.d')
            if os.path.isdir(pghbaIncludeDir) :
              self.logMesg("warn", Color.amber("\nCustom configuration found in %s has been applied." % pghbaIncludeDir ))
              self.logMesg("warn", Color.amber("Please be aware of that such configuration may impact the reliability of TeamForge.\n"))
              app.batch.ConfigBatch(self.log) \
                .addFolder(pghbaIncludeDir) \
                .evaluateIntoFile(confFile,'a')
