﻿
from app.scripts import BaseOptionScript

import app.landscape

class OptionScript(BaseOptionScript):
    def optionSnippet(self):
        localServices = app.landscape.Landscape.getSite().getLocalMachine().getServices()

        options = []

        #
        # Provided endpoints - binding address.
        #
        for service in localServices:
            serviceName = service.getName()

            # Resolve ambiguity if the same endpoint is available in multiple scopes.
            for endpointId in set([endp.getId() for endp in service.getProvidedEndpoints()]):
                for scope in ["public", "internal", None]:
                    provEndpoint = service.getProvidedEndpoint(endpointId, scope)
                    if not provEndpoint:
                        # Keep looking for other scopes.
                        provEndpoint = None
                        continue

                    if provEndpoint.isViaOtherPort():
                        # Vias are not useful here since they do not have a port number.
                        provEndpoint = None
                        continue

                    # Use this endpoint
                    break

                if provEndpoint:
                    # Endpoint to bind to.
                    options.append("%s/bind/%s/host=%s" % (serviceName, endpointId, provEndpoint.getBindingAddress().getName()))
                    options.append("%s/bind/%s/port=%s" % (serviceName, endpointId, provEndpoint.getPort()))

                    # Determine IP to bind to.
                    try:
                        bindingIp = provEndpoint.getBindingAddress().getIp()
                    except Exception:
                        self.logMesg("warn", "\n  Failed to determine binding IP address for service '%s' from name %s - using 0.0.0.0. \n  You can change this by adding %s:%s:LISTEN_IP to site-options.conf." %
                                     (serviceName, provEndpoint.getBindingAddress().getName(), app.landscape.Landscape.getSite().getLocalMachine().getPhysicalName(), serviceName))
                        bindingIp = "0.0.0.0"

                    options.append("%s/bind/%s/ip=%s" %   (serviceName, endpointId, bindingIp))

        #
        # Provided endpoints - canonical name.
        #
        for service in localServices:
            serviceName = service.getName()

            # Resolve ambiguity if the same endpoint is available in multiple scopes.
            for endpointId in set([endp.getId() for endp in service.getProvidedEndpoints()]):
                for scope in ["public", "internal", None]:
                    provEndpoint = service.getProvidedEndpoint(endpointId, scope)
                    if not provEndpoint:
                        # Keep looking for other scopes.
                        provEndpoint = None
                        continue

                    # Use this endpoint
                    break

                if provEndpoint:
                    # Canonical endpoint (might use Via).
                    options.append("%s/cname/%s/host=%s" % (serviceName, endpointId, provEndpoint.getAddress().getName()))
                    options.append("%s/cname/%s/port=%s" % (serviceName, endpointId, provEndpoint.getPort()))


        #
        # Required endpoint addresses.
        #
        for service in localServices:
            serviceName = service.getName()

            for reqEndpoint in service.getRequiredEndpoints():
                if reqEndpoint.getProviderEndpoint():
                    options.append("%s/dependency/%s/%s/host=%s" % (
                        serviceName,
                        reqEndpoint.getServiceName(),
                        reqEndpoint.getId(),
                        reqEndpoint.getProviderEndpoint().getHost()))
                    options.append("%s/dependency/%s/%s/remote-host=%s" % (
                        serviceName,
                        reqEndpoint.getServiceName(),
                        reqEndpoint.getId(),
                        reqEndpoint.getProviderEndpoint().getHost(False)))
                    options.append("%s/dependency/%s/%s/port=%s" % (
                        serviceName,
                        reqEndpoint.getServiceName(),
                        reqEndpoint.getId(),
                        reqEndpoint.getProviderEndpoint().getPort()))
                else:
                    # No provider available for this endpoint, must be an endpoint
                    # scoped as 'optional'.
                    pass

        self.log.info("Endpoints: \n- %s" % "\n- ".join(sorted(options)))

        return "\n".join(options)
