
from app.scripts import BaseCreateRuntimeScript
import app.util
import app.batch
from app.colors import Color
import os

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Create Monit configuration"

    def createRuntime(self):

        configFolder = os.path.join(app.defaults.DEFAULT_RUNTIME_DIR, "service-monitor", "etc")
        if not os.path.isdir(configFolder):
            os.makedirs(configFolder)

        batch = app.batch.ConfigBatch(self.log)

        # Add overrides
        for service in app.landscape.Landscape.getSite().getLocalMachine().getServices():
            overridesFolder = os.path.join(
                app.defaults.DEFAULT_SERVICE_DIR,
                service.getName(),
                'overrides',
                'monit',
                'monit.conf.d')

            if os.path.isdir(overridesFolder):
                batch = batch.addFolder(overridesFolder)

        customConfigDir = os.path.join(
                          self.cfg["ETC_DIR"],
                          "conf.d",
                          "monit",
                          "monitrc.d")

        if os.path.isdir(customConfigDir):
            self.logMesg("warn", Color.amber("\nCustom configuration found in %s has been applied." %customConfigDir ))
            self.logMesg("warn", Color.amber("Please be aware of that such configuration may impact the reliability of TeamForge.\n"))
        else:
            batch.removeFile('01-monitrc-custom.conf')

        monitrcFile = os.path.join(configFolder, "monitrc")
        batch.evaluateIntoFile(monitrcFile)
        os.chmod(monitrcFile, 0700)

        # Create symlink to /etc/monitrc
        if "redhat-7" in self.cfg["PLATFORM"]:
            monitrclnkFile = '/etc/monitrc'
        else:
            monitrclnkFile = '/etc/monit.conf'
        if os.path.isfile(monitrclnkFile) or os.path.islink(monitrclnkFile):
            app.util.runSystem("rm -f %s" % monitrclnkFile, self.log)

        app.util.runSystem('ln -s %s %s' % (monitrcFile, monitrclnkFile), self.log)
